/*
 *  This file is part of the VBMP3 package.
 *
 *  VBMP3, MP3 Operating Library
 *  originally Copyright (C) 1999-2000 Daisuke Karikomi
 *
 *  Portions Copyright (C) 2002 Kobarin
 *  Portions Copyright (C) 2002 Lenn
 */

#include "MP3Enc.h"
#include "MP3Output.h"
#include "vbmp3_common.h"
#include "MP3Input.h"
typedef unsigned int (__stdcall *THREADFUNC)(void* pParam);

void __stdcall _NullProc(int msg, int param) {}

BOOL input_fileOpen(const char* pszName, const wchar_t* pszNameW, DWORD dwFlag1, DWORD dwFlag2, DWORD dwFlag3, HANDLE *ptrhFile)
{
	if (pszName != nullptr)
	{
		//t@CI[v`FbN
		if (!isVar(pszName)) {
			//͂Ȃꍇ͏I
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}

		//t@CI[v
		*ptrhFile = ::CreateFile(pszName, dwFlag1, dwFlag2,
			NULL, dwFlag3, FILE_ATTRIBUTE_NORMAL, NULL);
		if (*ptrhFile == INVALID_HANDLE_VALUE) {
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}
	}
	else
	{
		//t@CI[v`FbN
		if (!isVarW(pszNameW)) {
			//͂Ȃꍇ͏I
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}

		//t@CI[v
		*ptrhFile = ::CreateFileW(pszNameW, dwFlag1, dwFlag2,
			NULL, dwFlag3, FILE_ATTRIBUTE_NORMAL, NULL);
		if (*ptrhFile == INVALID_HANDLE_VALUE) {
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}
	}
	return TRUE;
}

void input_getAtTagData(AT_TAG* pAtTagData)
{
    *pAtTagData = m_atTag;
}

int input_getLyricsCount()
{
    return m_lyricsCount;
}

int input_getLyricsData(LYRICS_DATA** pLyricsData, int elmNo)
{
    int     lyCnt;

    if ( elmNo < 0 ) return FALSE;

    if ( elmNo > m_lyricsCount ) elmNo = m_lyricsCount;

    if (m_lyricsCount == 0 ) {
        _setError(ERR_LYRICS_NON_DATA);
        return FALSE;
    }

    for(lyCnt = 0; lyCnt < elmNo; lyCnt++){
        ::memset(&pLyricsData[lyCnt], 0, sizeof(LYRICS_DATA));
        pLyricsData[lyCnt]->lineno = m_lyrics[lyCnt].lineno;
        char    lyrics[128] = "\0";
        strcat( lyrics, m_lyrics[lyCnt].lyrics );
        ::memcpy(pLyricsData[lyCnt]->lyrics, lyrics, strlen(lyrics));

        pLyricsData[lyCnt]->point = m_lyrics[lyCnt].point;
        pLyricsData[lyCnt]->sec = m_lyrics[lyCnt].sec;
        pLyricsData[lyCnt]->tagFlag = m_lyrics[lyCnt].tagFlag;
        pLyricsData[lyCnt]->textSize = m_lyrics[lyCnt].textSize;
        pLyricsData[lyCnt]->lineno = m_lyrics[lyCnt].lineno;
        pLyricsData[lyCnt]->no = lyCnt;
    }
    return TRUE;
}

BOOL input_getCallbackLyricsData(LYRICS_PROC pProc)
{
    int     lyCnt;
    LYRICS_DATA lyData;

    if (m_lyricsCount == 0 ) {
        _setError(ERR_LYRICS_NON_DATA);
        return FALSE;
    }
    
    for(lyCnt = 0; lyCnt < m_lyricsCount; lyCnt++){
        ::memset(&lyData, 0, sizeof(LYRICS_DATA));
        lyData.lineno = m_lyrics[lyCnt].lineno;
        char    lyrics[128] = "\0";
        strcat( lyrics, m_lyrics[lyCnt].lyrics );
        ::memcpy(lyData.lyrics, lyrics, strlen(lyrics));

        lyData.point = m_lyrics[lyCnt].point;
        lyData.sec = m_lyrics[lyCnt].sec;
        lyData.tagFlag = m_lyrics[lyCnt].tagFlag;
        lyData.textSize = m_lyrics[lyCnt].textSize;
        lyData.lineno = m_lyrics[lyCnt].lineno;
        lyData.no = lyCnt;
        pProc(&lyData);
    }
    return TRUE;
}

int input_checkTimeTagType()
{
    if ( m_ExistKaraokeTag ) {
        return 2;
    } 
    else if ( m_ExistTimeTag ){
        return 1;
    } 
    else {
        return 0;
    }
}

void input_setTimeRatio(double timeRatio)
{
    m_timeRatio = timeRatio;
}

void input_useAtMarkTag(int useFlag)
{
    m_atTagUseFlag = useFlag;
}

BOOL input_outLyricsFile(const char* pszLyricsName, const wchar_t* pszLyricsNameW)
{
    HANDLE          hFile;
    char            buf[1024];
    int             lyCnt;
    BOOL            result;
    DWORD           bytesWrite;
    DWORD           dataSize;

    if (m_lyricsCount == 0 || m_decInfo.frequency == 0) {
        _setError(ERR_LYRICS_NON_DATA);
        return FALSE;
    }

	// t@CI[v
	if (input_fileOpen(pszLyricsName, pszLyricsNameW, 
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, CREATE_ALWAYS, &hFile) == FALSE)
			return FALSE;
	
    for (lyCnt = 0; lyCnt < m_lyricsCount; lyCnt++){
        ::memset(buf, 0, sizeof(buf));
        input_getLyricsText(m_lyrics[lyCnt], buf, 1);
        dataSize = ::strlen(buf);

        result = ::WriteFile(hFile, buf, dataSize, &bytesWrite, NULL);
        if (!result || bytesWrite != dataSize) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFile);
            return FALSE;
        }
    }
    //::strcpy(buf, "\r\n" );
    ::memset(buf, 0, sizeof(buf));
    ::strcpy(buf, "\n" );

    result = ::WriteFile(hFile, "\n", 1, &bytesWrite, NULL);
    if (!result || bytesWrite != 1) {
        _setError(ERR_MP3_FILE_WRITE);
        ::CloseHandle(hFile);
        return FALSE;
    }

    ::CloseHandle(hFile);
    return TRUE;

}

BOOL input_convTimeN2W()
{
    int     lyCnt;

    if (m_lyricsCount == 0 || m_decInfo.frequency == 0) {
        _setError(ERR_LYRICS_NON_DATA);
        return FALSE;
    }

    for (lyCnt = 0; lyCnt < m_lyricsCount; lyCnt++){
        m_lyrics[lyCnt].sec = getWinampMs(m_decInfo.frequency, input_msToSample(m_lyrics[lyCnt].sec));
    }

    return TRUE;
}

BOOL input_convTimeW2N()
{
    int     lyCnt;

    if (m_lyricsCount == 0 || m_decInfo.frequency == 0) {
        _setError(ERR_LYRICS_NON_DATA);
        return FALSE;
    }

    for (lyCnt = 0; lyCnt < m_lyricsCount; lyCnt++){
        m_lyrics[lyCnt].sec = getNomalMs(m_decInfo.frequency, input_msToSample(m_lyrics[lyCnt].sec));
    }

    return TRUE;
}

void input_quickSeek(int quickSeekFlag)
{
    if ( quickSeekFlag != 0 )
        quickSeekFlag = TRUE;
    else
        quickSeekFlag = FALSE;
    m_quickSeek = quickSeekFlag;
    _setDebug(m_quickSeek);
}

void input_setSeekPlay(int seekPlayFlag)
{
    if ( seekPlayFlag != 0 )
        seekPlayFlag = TRUE;
    else
        seekPlayFlag = FALSE;
    m_seekPlayFlag = seekPlayFlag;
}

int input_getVersion()
{
    return VBMP3_VER;
}

void input_setLyricsNextAdjustTime(long ms)
{
    int     lyCnt;
    long    nextSec;
    long    befSec;

    m_lyricsNextAdjustTime = ms;

    for ( lyCnt = 1; lyCnt < m_lyricsCount - 1; lyCnt++ ){
        if ( m_lyrics[lyCnt].point == 0 ){
            nextSec = m_lyrics[m_lyricsCount + 1].sec;
            befSec = m_lyrics[m_lyricsCount - 1].sec;
            if ( (nextSec - befSec) > m_lyricsNextAdjustTime + 100)  m_lyrics[lyCnt].sec -= m_lyricsNextAdjustTime;
        }
    }

}

void input_setLyricsAdjustTime(long ms)
{
    m_lyricsAdjustTime = ms;
}

void input_setFrameReadFlag(int readFlag)
{
    if ( readFlag != 0 ) readFlag = 1;
    m_frameReadFlag = readFlag;
}

void input_setNotDataSiWrite(int writeFlag)
{
    if ( writeFlag != 0 ) writeFlag = 1;
    m_NotDataSiWrite = writeFlag;
}

void input_setNotTagLyricsLine(int readFlag)
{
    if ( readFlag != 0 ) readFlag = 1;
    m_NotTagLineRead = readFlag;
}

void input_setWaveOutDeviceId(int id){
    output_setWaveOutDeviceId(id);
}

void input_setLyricsTime(int flag){
    m_lyricsTotalFlag = flag;
}

void input_setFftWindow(int window)
{
    output_setFftWindow(window);
}

void input_setKaraokeUse(int useKaraoke)
{
    if ( useKaraoke == 1 ) {
        m_KaraokeTagFlag = TRUE;
    } else {
        m_KaraokeTagFlag = FALSE;
    }
}

BOOL input_getListInfoEX(const char* pszName, const wchar_t* pszNameW, LIST_INFO_EX* lpListInfo)
{
    HANDLE          hFileMpeg;              //Mp3t@C
    BOOL            result;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE, OPEN_EXISTING, &hFileMpeg) == FALSE)
			return FALSE;
	
    result = getListInfoEX(hFileMpeg, lpListInfo);
    ::CloseHandle(hFileMpeg);

    return result;
}

BOOL input_setListInfoEX(const char* pszName, const wchar_t* pszNameW, LIST_INFO_EX* lpListInfo)
{
    HANDLE          hFileMpeg;              //Mp3t@C
    BOOL            result;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, OPEN_EXISTING, &hFileMpeg) == FALSE)
			return FALSE;

    result = setListInfoEX(hFileMpeg, lpListInfo);
    //::CloseHandle(hFileMpeg);//C by Kobarin
                               //CloseHandle  setListInfoEX ŌĂ΂Ă

    return result;
}

BOOL input_setListInfoEX2(const char* pszName, const wchar_t* pszNameW, LIST_INFO_EX2* lpListInfo)
{
    HANDLE          hFileMpeg;              //Mp3t@C
    BOOL            result;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, OPEN_EXISTING, &hFileMpeg) == FALSE)
			return FALSE;

    result = setListInfoEX2(hFileMpeg, lpListInfo);
    //::CloseHandle(hFileMpeg);//C by Kobarin
                               //CloseHandle  setListInfoEX2 ŌĂ΂Ă

    return result;
}

BOOL input_clearLyrics()
{
/*
    if (m_state != STATE_STOP) {
        //~ȊO͎ss
        _setError(ERR_NOT_STATE_STOP);
        return FALSE;
    }
*/
    m_ExistKaraokeTag = FALSE;
    m_ExistTimeTag = FALSE;
    m_lyricsCount = 0;
    m_lyricsLineNo = 0;
    m_lyricsCurrent = 0;
    ::memset(m_lyrics, 0, sizeof(m_lyrics));
    ::memset(&m_atTag, 0, sizeof(AT_TAG));
    m_atTag.AlbumLine = -1;
    m_atTag.ArtistLine = -1;
    m_atTag.BgfileLine = -1;
    m_atTag.BgfolderLine = -1;
    m_atTag.EditedByLine = -1;
    m_atTag.FlamesLine = -1;
    m_atTag.OffsetLine = -1;
    m_atTag.SilencemSecLine = -1;
    m_atTag.TaggingByLine = -1;
    m_atTag.TimeRatioLine = -1;
    m_atTag.TimeTypeLine = -1;
    m_atTag.TitleLine = -1;
    m_atTag.TotalSecLine = -1;
    return TRUE;

}

BOOL input_readLyrics3Data()
{
    HANDLE          hFile;                  //Ώۂ̃t@C
    BOOL            result;

    if (m_hFile == INVALID_HANDLE_VALUE) {
        _setError(ERR_MP3_FILE_NOT_OPEN);
        return FALSE;
    }

	if (m_openFileName[0] != 0)
	{
		//t@CI[v
		hFile = ::CreateFile(m_openFileName, GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFile == INVALID_HANDLE_VALUE) {
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}
	}
	else
	{
		//t@CI[v
		hFile = ::CreateFileW(m_openFileNameW, GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFile == INVALID_HANDLE_VALUE) {
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}
	}


    result =  input_readLyrics3(hFile);
    ::CloseHandle(hFile);

    return result;
}

void input_setLyrics3Use(int useLyrics3)
{
    if ( useLyrics3 == 1 ) {
        m_Lyrics3UseFlag = TRUE;
    } else {
        m_Lyrics3UseFlag = FALSE;
    }
}

void input_setLyrics3InsField(int field)
{
    if ( field < 0 || field > 2 ) {
        m_Lyrics3InsertPoint = 0;
    } else {
        m_Lyrics3InsertPoint = field;
    }
}

int input_getSilentFrames(const char* pszName, const wchar_t* pszNameW)
{
    HANDLE          hFileMpeg;              //^OҏWt@C
    BOOL            result;
    int             fSync;                  //wb_̈ʒu
    DWORD           dSize;                  //MPEG Audiõf[^TCY
    BYTE            buf[10240];
    DWORD           bytesRead;
    MPEG_DECODE_INFO dInfo;
    LIST_INFO       ListInfo;
    DWORD           fSize;
    int             headSize;
    int             frameSize;
    int             dCnt; 
    int             sCnt = 0;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE, OPEN_EXISTING, &hFileMpeg) == FALSE)
			return -1;

    fSync = 0;
    dSize = GetFileSize(hFileMpeg, NULL);
    fSize = dSize;

    SetFilePointer(hFileMpeg, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileMpeg, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileMpeg);
        return -1;
    }

    //RIFF`AID3 Tag v2`FbN
    if (memcmp(buf, "RIFF", 4) == 0) {
        ::readRIFF(hFileMpeg, &ListInfo, &fSync, &dSize);
        if (RiffStatus != 0) {
            ::CloseHandle(hFileMpeg);
            _setError(ERR_UNKNOWN_FILE);
            return -1;
        } else {
            if (readID3v2(hFileMpeg, &fSync)) {
                dSize -= fSync;
            }
        }
    }
    else if (memcmp(buf, "ID3", 3) == 0) {
        if (readID3v2(hFileMpeg, &fSync)) {
            dSize -= fSync;
        }
    }
    //Tag f[^
    else if (buf[0] == 0x00) {
            fSync = 128;
    }
    else {
        fSync = 0;
    }

    for(;;){

        //wb_擾
        ::SetFilePointer(hFileMpeg, fSync, NULL, FILE_BEGIN);
        result = ::ReadFile(hFileMpeg, buf, sizeof(buf), &bytesRead, NULL);
        if (!result || bytesRead == 0) {
            if( sCnt == 0 ){
                ::CloseHandle(hFileMpeg);
                _setError(ERR_MP3_FILE_READ);
                return -1;
            }
            return sCnt - 1;
        }
        //wb_T
        int sync;
        if (!::mp3FindSync(buf, bytesRead, &sync)) {
            if( sCnt == 0 ){
                ::CloseHandle(hFileMpeg);
                _setError(ERR_FRAME_HEADER_NOT_FOUND);
                return -1;
            }
            return sCnt - 1;
        }
        fSync += sync;

        //wb_ʒu擾
        ::SetFilePointer(hFileMpeg, fSync, NULL, FILE_BEGIN);
        result = ::ReadFile(hFileMpeg, buf, sizeof(buf), &bytesRead, NULL);
        if (!result || bytesRead == 0) {
            if( sCnt == 0 ){
                ::CloseHandle(hFileMpeg);
                _setError(ERR_MP3_FILE_READ);
                return -1;
            }
            return sCnt - 1;
        }
        
        //wb_擾
        if (!::mp3GetDecodeInfo(buf, bytesRead, &dInfo, 0)) {
            if( sCnt == 0 ){
                _setError(ERR_FRAME_HEADER_READ);
                ::CloseHandle(hFileMpeg);
                return -1;
            }
            fSync += 900;
            continue;
        }

        //wb_TCY擾
        headSize = 4;
        if ( dInfo.header.error_prot == 1 ) headSize = headSize + 2;
        if ( dInfo.header.layer == 3 ) {
            if ( dInfo.header.version == 1 ) {
                headSize = headSize + 17 * dInfo.channels;
            } else {
                headSize = headSize + 9 * dInfo.channels;
            }
        }

        //t[TCY擾
        if ( dInfo.header.padding == 0 ) {
            frameSize = dInfo.minInputSize;
        } else {
            frameSize = dInfo.maxInputSize;
        }

        sCnt++;
        if ( !(dInfo.skipSize != 0 && sCnt == 1) ) {
            for ( dCnt = headSize; dCnt < frameSize; dCnt++ ) {
                if ( buf[dCnt] != 0xFF && buf[dCnt] != 0x00 ) {
                    ::CloseHandle(hFileMpeg);
                    return sCnt - 1;
                }
            }
        }
        fSync = fSync + frameSize;
    }

    ::CloseHandle(hFileMpeg);
    return -1;

}

int input_getSampleToSeekPoint(int seekSample, int* pOutputBytes)
{
    HANDLE          hFileSeek;
    BOOL            result;
    int             fSync = m_firstSync;    //wb_̈ʒu
    BYTE            buf[10240];
    DWORD           bytesRead;
    MPEG_DECODE_INFO dInfo;
    int             headSize;
    int             frameSize;
    int             sCnt = 0;


    //t@CI[v
	if (m_openFileName[0] != 0)
	{
		hFileSeek = ::CreateFile(m_openFileName, GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFileSeek == INVALID_HANDLE_VALUE) {
			_setError(ERR_MP3_FILE_OPEN);
			return sCnt;
		}
	}
	else
	{
		hFileSeek = ::CreateFileW(m_openFileNameW, GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFileSeek == INVALID_HANDLE_VALUE) {
			_setError(ERR_MP3_FILE_OPEN);
			return sCnt;
		}
	}

    //wb_TCY擾
    headSize = 4;
    if ( m_decInfo.header.error_prot == 1 ) headSize = headSize + 2;
    if ( m_decInfo.header.layer == 3 ) {
        if ( m_decInfo.header.version == 1 ) {
            headSize = headSize + 17 * m_decInfo.channels;
        } else {
            headSize = headSize + 9 * m_decInfo.channels;
        }
    }

    //t[TCY擾
    if ( m_decInfo.header.padding == 0 ) {
        frameSize = m_decInfo.minInputSize;
    } else {
        frameSize = m_decInfo.maxInputSize;
    }

    for(;;){
        int samples = input_bytesToSample((sCnt + 20) * m_decInfo.outputSize);
        if (samples >= seekSample) {
            break;
        }
        sCnt++;
        *pOutputBytes += m_decInfo.outputSize;
        fSync = fSync + frameSize;
    }

    if ( (DWORD)(dataSize + m_firstSync) < (DWORD)fSync ){
        int lastFlame = 2;
        fSync = (dataSize + m_firstSync) - frameSize * lastFlame;
        sCnt = m_decInfo.frames - lastFlame;
        *pOutputBytes = m_decInfo.outputSize * (sCnt);
    }

    for(;;){

        //wb_擾
        ::SetFilePointer(hFileSeek, fSync, NULL, FILE_BEGIN);
        result = ::ReadFile(hFileSeek, buf, sizeof(buf), &bytesRead, NULL);
        if (!result || bytesRead == 0) {
            if( sCnt == 0 ){
                _setError(ERR_MP3_FILE_READ);
            }
            ::CloseHandle(hFileSeek);
            return fSync;
        }
        //wb_T
        int sync;
        if (!::mp3FindSync(buf, bytesRead, &sync)) {
            if( sCnt == 0 ){
                _setError(ERR_FRAME_HEADER_NOT_FOUND);
            }
            ::CloseHandle(hFileSeek);
            return fSync;
        }
        fSync += sync;

        //wb_ʒu擾
        ::SetFilePointer(hFileSeek, fSync, NULL, FILE_BEGIN);
        result = ::ReadFile(hFileSeek, buf, sizeof(buf), &bytesRead, NULL);
        if (!result || bytesRead == 0) {
            if( sCnt == 0 ){
                _setError(ERR_MP3_FILE_READ);
            }
            ::CloseHandle(hFileSeek);
            return fSync;
        }
        
        //wb_擾
        if (!::mp3GetDecodeInfo(buf, bytesRead, &dInfo, 0)) {
            if( sCnt == 0 ){
                _setError(ERR_FRAME_HEADER_READ);
                ::CloseHandle(hFileSeek);
                return fSync;
            }
            fSync += 900;
            continue;
            //return fSync;
        }

        //wb_TCY擾
        headSize = 4;
        if ( dInfo.header.error_prot == 1 ) headSize = headSize + 2;
        if ( dInfo.header.layer == 3 ) {
            if ( dInfo.header.version == 1 ) {
                headSize = headSize + 17 * dInfo.channels;
            } else {
                headSize = headSize + 9 * dInfo.channels;
            }
        }

        //t[TCY擾
        if ( dInfo.header.padding == 0 ) {
            frameSize = dInfo.minInputSize;
        } else {
            frameSize = dInfo.maxInputSize;
        }

        sCnt++;

        *pOutputBytes += dInfo.outputSize;
        int samples = input_bytesToSample(sCnt * dInfo.outputSize);
        if (samples >= seekSample) {
            ::CloseHandle(hFileSeek);
            return fSync;
        }

        fSync = fSync + frameSize;
    }

    ::CloseHandle(hFileSeek);
    return fSync;
}

int input_getTotalFrames(const char* pszName, const wchar_t* pszNameW)
{
    HANDLE          hFileMpeg;              //^OҏWt@C
    BOOL            result;
    int             fSync;                  //wb_̈ʒu
    DWORD           dSize;                  //MPEG Audiõf[^TCY
    BYTE            buf[10240];
    DWORD           bytesRead;
    MPEG_DECODE_INFO dInfo;
    LIST_INFO       ListInfo;
    DWORD           fSize;
    int             headSize;
    int             frameSize;
    int             sCnt = 0;


	
	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE, OPEN_EXISTING, &hFileMpeg) == FALSE)
		return -1;

    fSync = 0;
    dSize = GetFileSize(hFileMpeg, NULL);
    fSize = dSize;

    SetFilePointer(hFileMpeg, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileMpeg, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileMpeg);
        return -1;
    }

    //RIFF`AID3 Tag v2`FbN
    if (memcmp(buf, "RIFF", 4) == 0) {
        ::readRIFF(hFileMpeg, &ListInfo, &fSync, &dSize);
        if (RiffStatus != 0) {
            ::CloseHandle(hFileMpeg);
            _setError(ERR_UNKNOWN_FILE);
            return -1;
        } else {
            if (readID3v2(hFileMpeg, &fSync)) {
                dSize -= fSync;
            }
        }
    }
    else if (memcmp(buf, "ID3", 3) == 0) {
        if (readID3v2(hFileMpeg, &fSync)) {
            dSize -= fSync;
        }
    }
    //Tag f[^
    else if (buf[0] == 0x00) {
            fSync = 128;
    }
    else {
        fSync = 0;
    }

    for(;;){

        //wb_擾
        ::SetFilePointer(hFileMpeg, fSync, NULL, FILE_BEGIN);
        result = ::ReadFile(hFileMpeg, buf, sizeof(buf), &bytesRead, NULL);
        if (!result || bytesRead == 0) {
            if( sCnt == 0 ){
                ::CloseHandle(hFileMpeg);
                _setError(ERR_MP3_FILE_READ);
            }
            return sCnt;
        }
        //wb_T
        int sync;
        if (!::mp3FindSync(buf, bytesRead, &sync)) {
            if( sCnt == 0 ){
                ::CloseHandle(hFileMpeg);
                _setError(ERR_FRAME_HEADER_NOT_FOUND);
                return -1;
            }
            return sCnt;
        }
        fSync += sync;

        //wb_ʒu擾
        ::SetFilePointer(hFileMpeg, fSync, NULL, FILE_BEGIN);
        result = ::ReadFile(hFileMpeg, buf, sizeof(buf), &bytesRead, NULL);
        if (!result || bytesRead == 0) {
            if( sCnt == 0 ){
                ::CloseHandle(hFileMpeg);
                _setError(ERR_MP3_FILE_READ);
                return -1;
            }
            return sCnt;
        }
        
        //wb_擾
        if (!::mp3GetDecodeInfo(buf, bytesRead, &dInfo, 0)) {
            if( sCnt == 0 ){
                _setError(ERR_FRAME_HEADER_READ);
                ::CloseHandle(hFileMpeg);
                return -1;
            }
            return sCnt;
        }

        //wb_TCY擾
        headSize = 4;
        if ( dInfo.header.error_prot == 1 ) headSize = headSize + 2;
        if ( dInfo.header.layer == 3 ) {
            if ( dInfo.header.version == 1 ) {
                headSize = headSize + 17 * dInfo.channels;
            } else {
                headSize = headSize + 9 * dInfo.channels;
            }
        }

        //t[TCY擾
        if ( dInfo.header.padding == 0 ) {
            frameSize = dInfo.minInputSize;
        } else {
            frameSize = dInfo.maxInputSize;
        }

        sCnt++;
        fSync = fSync + frameSize;
    }

    ::CloseHandle(hFileMpeg);
    return sCnt;

}

void input_setOverTime(int on)
{
    if ( on == 0 ) {
        m_timeFlag = FALSE;
    } else {
        m_timeFlag = TRUE;
    }
}

void input_fadeOut()
{
    output_getVolume(&m_olvol,&m_orvol);
    m_lvol = m_olvol;
    m_rvol = m_orvol;
    m_fadeOutFlag = true;
}

void input_fadeOutVolume()
{
    if ( m_fadeOutFlag ){
        if ( m_lvol > 0 )m_lvol--;
        if ( m_rvol > 0 )m_rvol--;
        output_setVolume(m_lvol, m_rvol);

        if (m_lvol == 0 && m_rvol == 0) {
            m_lvol=m_olvol;
            m_rvol=m_orvol;
            m_fadeOutFlag = false;
            output_stop();
        }
    }
}

int input_getDecodeOutputSize()
{
    return output_sampleToBytes(m_firstSample);
}

int input_getTotalOutputSize()
{
    return output_sampleToBytes(input_getTotalSamples());
}


void input_setFadeIn(int on)
{
    output_setFadeIn(on);
}

void input_setFadeOut(int on)
{
    output_setFadeOut(on);
}


BOOL input_reload()
{
    if ( m_state == STATE_STOP ) {
        _setError(ERR_STATE_STOP);
        return FALSE;
    }

    int w_state = m_state;
    int curSample = output_getCurrentSample() + m_firstSample;

    m_reloadFlag = TRUE;
    m_stateWork = w_state;

    //input_stop();폜 by Kobarin
    input_play();
    input_seekBySample(curSample);
    if ( w_state == STATE_PAUSE ) 
        input_pause();
    m_reloadFlag = FALSE;
    return TRUE;
}

BOOL input_setStepPitch(int pitch, int frames)
{
    //WinNT b΍
    /*if ( m_platForm == 1 ) {
        _setError(ERR_PLATFORM_NT);
        return FALSE;
    }*/
    //폜 by Kobarin

    if ( pitch < 1 || frames < 1)
    {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }
    int w_state = m_state;
    int curSample = output_getCurrentSample() + m_firstSample;

    if ( pitch == 1 ) {
        m_outputBlock_in = m_outputBlock_in_tmp;
        m_stepFlag = FALSE;
    } else {
        if ( m_stepFlag == FALSE ) m_outputBlock_in_tmp = m_outputBlock_in; 
        m_outputBlock_in = frames;
        m_stepFlag = TRUE;
    }

    output_setStepPitch(pitch);
    if ( m_state != STATE_STOP) {
        input_stop();
        input_play();
        input_seekBySample(curSample);
        if ( w_state == STATE_PAUSE )
            input_pause();
    }
    return TRUE;
}

int input_getStepPitch()
{
    return output_getStepPitch();
}

BOOL input_setPitch(int pitch)
{
    if ( pitch < 20 )
    {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }
    output_setPitch(pitch);
    int w_state = m_state;
    int curSample = output_getCurrentSample() + m_firstSample;
    if ( m_state != STATE_STOP) {
        input_stop();
        input_play();
        input_seekBySample(curSample);
        if ( w_state == STATE_PAUSE )
            input_pause();
    }
    return TRUE;
}

int input_getPitch()
{
    return output_getPitch();
}

void input_callBackTime()
{
    if ( m_seekFlag != TRUE && m_reloadFlag != TRUE ) {
        if ( callBackFlag == TRUE ) {
            m_playSamples = output_getCurrentSample();
            m_playSec = (int)(input_sampleToSec(m_playSamples + m_firstSample) * m_playSpeed);
            if ( m_timeFlag && m_playSec > m_totalSec ) m_playSec = m_totalSec;
            m_pProc(INPUT_MSG_PLAY, m_playSec);
        }
    }
}

long input_getWinampPlayMs()
{
    long winampMs;
    long playSamples;

    if ( m_decInfo.frequency == 0 || m_totalSec == 0 ) {
        return 0;
    }

    //JgTv擾
    playSamples = output_getCurrentSample();

    long samples = playSamples + m_firstSample;
    //ĐԎ擾
    winampMs = getWinampMs(m_decInfo.frequency, samples);

    //{Đp
    winampMs = (long)((double)winampMs * m_playSpeed);

    return winampMs;
}

int input_getWinampTotalSec()
{
    return m_winampTotalSec;
}

int input_getPlayBitRate()
{
    if ( m_waveFlag == FALSE ) {
        return m_bitRate / 1000;
    } else {
        return m_bitRate;
    }
}

int input_getPlayFrames()
{
    m_playFrames = 0;

    //t[擾
    if ( m_decInfo.outputSize > 0 ){
        if ( decFlag == 1 ){    
            m_playFrames = (int)(((m_firstSample + input_bytesToSample(decodePos)) * m_decInfo.bitsPerSample * m_decInfo.channels / 8) / m_decInfo.outputSize);
        } else {
            m_playFrames = (int)(output_sampleToBytes(output_getCurrentSample() + m_firstSample) / m_decInfo.outputSize);
        }
    }
    return m_playFrames;
}

int input_getPlaySamples()
{
    if ( decFlag == 1 ) return (m_firstSample + input_bytesToSample(decodePos));
    return output_getCurrentSample() + m_firstSample;
}

int input_getTotalSamples()
{
    return input_bytesToSample(m_decInfo.frames * m_decInfo.outputSize);
}

int getFileType(const char* pszName, const wchar_t* pszNameW)
{
    HANDLE          hFileType;              //^OҏWt@C
    BYTE            buf[1024];
    BOOL            result;
    DWORD           bytesRead;
    int             fType = 0;
    RiffChunk       riff;
    CommonChunk     common;

	if (pszName != nullptr)
	{
		//t@CI[v`FbN
		if (!isVar(pszName)) {
			//͂Ȃꍇ͏I
			return -1;
		}

		//t@CI[v
		//hFileType = ::CreateFile(pszName, GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ,
		hFileType = ::CreateFile(pszName, GENERIC_READ, FILE_SHARE_READ,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFileType == INVALID_HANDLE_VALUE) {
			return -1;
		}
	}
	else
	{
		//t@CI[v`FbN
		if (!isVarW(pszNameW)) {
			//͂Ȃꍇ͏I
			return -1;
		}

		//t@CI[v
		//hFileType = ::CreateFile(pszName, GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ,
		hFileType = ::CreateFileW(pszNameW, GENERIC_READ, FILE_SHARE_READ,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFileType == INVALID_HANDLE_VALUE) {
			return -1;
		}
	}

    //RIFF`AID3 Tag v2`FbN
    SetFilePointer(hFileType, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileType, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        ::CloseHandle(hFileType);
        return -1;
    }
    if (memcmp(buf, "RIFF", 4) == 0) {
        //t@C̓ɖ߂ARIFF`Nǂݍ
        ::SetFilePointer(hFileType, 0, NULL, FILE_BEGIN);
        result = ::ReadFile(hFileType, &riff, sizeof(riff), &bytesRead, NULL);
        if (!result || bytesRead != sizeof(riff)) {
            ::CloseHandle(hFileType);
            return -1;
        }
        if (memcmp(riff.formatType, "WAVE", 4) == 0) {
            fType += 1;
        }
        if (memcmp(riff.formatType, "RMP3", 4) == 0) {
            fType += 2;
        }
        //data`NT
        for (;;) {
            result = ::ReadFile(hFileType, &common, sizeof(common), &bytesRead, NULL);
            if (!result || bytesRead != sizeof(common)) {
                ::CloseHandle(hFileType);
                return -1;
            }

            if (::memcmp(common.chunkID, "fmt ", 4) == 0) {
                //MpegȊÕtH[}bg`FbN
                ::ReadFile(hFileType, &m_waveFormat, sizeof(WAVEFORMATEX), &bytesRead, NULL);
                if (!result || bytesRead != sizeof(WAVEFORMATEX)) {
                    ::CloseHandle(hFileType);
                    return -1;
                }
                //tH[}bgTag`FbN
                if (m_waveFormat.wFormatTag == 85 || m_waveFormat.wFormatTag == 80) {
                    //MPEG Audio f[^
                } else if (m_waveFormat.wFormatTag == 1) {
                    //Aoblf[^
                    fType += 64;
                    ::CloseHandle(hFileType);
                    return fType;
                } else {
                    //̑̌`
                    ::CloseHandle(hFileType);
                    return -1;
                }
                ::SetFilePointer(hFileType, -(long)sizeof(WAVEFORMATEX), NULL, FILE_CURRENT);
            }

            if (::memcmp(common.chunkID, "data", 4) == 0) {
                //data`N
                break;
            }
            else {
                //̃`N܂œǂݔ΂
                ::SetFilePointer(hFileType, common.chunkSize, NULL, FILE_CURRENT);
            }
        }
        result = ReadFile(hFileType, buf, 4, &bytesRead, NULL);
        if (!result || bytesRead != 4) {
            ::CloseHandle(hFileType);
            return -1;
        }
        if (memcmp(buf, "ID3", 3) == 0) {
            fType += 4;
        }
    }
    else if (memcmp(buf, "ID3", 3) == 0) {
        fType += 4;
    }
    else if (buf[0] == 0x00) {
        fType += 8;
    }

    SetFilePointer(hFileType, -(long)128, NULL, FILE_END);
    result = ReadFile(hFileType, buf, 128, &bytesRead, NULL);
    if (!result || bytesRead != 128) {
        ::CloseHandle(hFileType);
        return -1;
    }
    if (memcmp(buf, "TAG", 3) == 0) {
        fType += 16;
    }

    //LYRICS3݃`FbN
    DWORD lysize;
    if (chkLYRICS3(hFileType, &lysize)) {
        fType += 32;
    }

    ::CloseHandle(hFileType);
    return fType;

}

BOOL input_cutID3v2(const char* pszName, const wchar_t* pszNameW)
{
    HANDLE          hFile;
    BYTE            buf[1024];
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    RiffChunk       riff;
    CommonChunk     common;
    long            tagSync = 0;
    long            tagSize;
    DWORD           fSize;
    DWORD           dSize;
    DWORD           backDataSize;
    DWORD           siSize;
    DWORD           siSync;
    int             riffFlag = 0;
    int             adjSize = 0;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, OPEN_EXISTING, &hFile) == FALSE)
		return FALSE;

    //RIFF`AID3 Tag v2`FbN
    SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFile, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        ::CloseHandle(hFile);
        return FALSE;
    }
    if (memcmp(buf, "RIFF", 4) == 0) {
        riffFlag = 1;
        //t@C̓ɖ߂ARIFF`Nǂݍ
        ::SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
        result = ::ReadFile(hFile, &riff, sizeof(riff), &bytesRead, NULL);
        if (!result || bytesRead != sizeof(riff)) {
            ::CloseHandle(hFile);
            return FALSE;
        }

        tagSync = sizeof(riff);

        //data`NT
        for (;;) {
            result = ::ReadFile(hFile, &common, sizeof(common), &bytesRead, NULL);
            if (!result || bytesRead != sizeof(common)) {
                ::CloseHandle(hFile);
                return FALSE;
            }

            if (::memcmp(common.chunkID, "fmt ", 4) == 0) {
                //MpegȊÕtH[}bg`FbN
                ::ReadFile(hFile, &m_waveFormat, sizeof(WAVEFORMATEX), &bytesRead, NULL);
                if (!result || bytesRead != sizeof(WAVEFORMATEX)) {
                    ::CloseHandle(hFile);
                    return FALSE;
                }

                //tH[}bgTag`FbN
                if (!(m_waveFormat.wFormatTag == 85 || m_waveFormat.wFormatTag == 80)) {
                    //MPEG Audio f[^ȊǑ`
                    ::CloseHandle(hFile);
                    return FALSE;
                }
                ::SetFilePointer(hFile, -(long)sizeof(WAVEFORMATEX), NULL, FILE_CURRENT);
            }

            if (::memcmp(common.chunkID, "data", 4) == 0) {
                //data`N
                break;
            }
            else {
                //̃`N܂œǂݔ΂
                tagSync = ::SetFilePointer(hFile, common.chunkSize, NULL, FILE_CURRENT);
            }
        }

        //`NTCY`FbN
        dSize = common.chunkSize;

        tagSync += bytesRead;
        result = ReadFile(hFile, buf, 4, &bytesRead, NULL);
        if (!result || bytesRead != 4) {
            ::CloseHandle(hFile);
            return FALSE;
        }
        if (memcmp(buf, "ID3", 3) != 0) {
            ::CloseHandle(hFile);
            return FALSE;
        }
    }
    else if (memcmp(buf, "ID3", 3) != 0) {
        ::CloseHandle(hFile);
        return FALSE;
    }

    //ID3 Tag v2 TCY`FbN
    tagSize = getID3v2Size(hFile, tagSync);
    if ( tagSize == -1 ) {
        ::CloseHandle(hFile);
        return FALSE;
    }

    //TCYs`FbN
    ::SetFilePointer(hFile, tagSync + tagSize, NULL, FILE_BEGIN);
    result = ::ReadFile(hFile, buf, sizeof(buf), &bytesRead, NULL);
    if (!result || bytesRead == 0) {
        ::CloseHandle(hFile);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }

    //RIFFT(^OʒuRIFFʒu܂Ŋg)
    int sync;
    for ( sync = 0; sync < sizeof(buf) - 3; sync++ ) {
        if ( buf[sync] == 'R' ) {
            if (memcmp(&buf[sync], "RIFF", 4) == 0) {
                tagSize += sync;
                sync = -1;
                break;
            }
        }
    }

    //wb_T(^OTCY𓯊wb_ʒu܂Ŋg)
    if ( sync >= 0 ){
        if (!::mp3FindSync(buf, bytesRead, &sync)) {
            ::CloseHandle(hFile);
            _setError(ERR_FRAME_HEADER_NOT_FOUND);
            return FALSE;
        }
        tagSize += sync;
    }

    if ( (tagSize % 2) == 1 ) adjSize = 1;
    fSize = GetFileSize(hFile, NULL) - tagSize;
    if ( riffFlag == 1 ) {
        dSize -= tagSize;
        backDataSize = fSize - tagSync - 1;
        siSize = backDataSize - dSize;
        siSync = tagSync + tagSize + dSize;
    } else {
        dSize = fSize;
        backDataSize = dSize;
        siSize = 0;
        siSync = 0;
    }

    //dataړ
    result = moveFileData(hFile, tagSync + tagSize, dSize, tagSync);
    if (!result){
        ::CloseHandle(hFile);
        return FALSE;
    }

    //listړ
    if ( siSize != 0 ) {
        if ( adjSize == 1 ) {
            buf[0] = 0;
            ::SetFilePointer(hFile, siSync, NULL, FILE_BEGIN);
            result = ::WriteFile(hFile, buf, 1, &bytesWrite, NULL);
            if (!result || bytesWrite != 1) {
                _setError(ERR_MP3_FILE_WRITE);
                ::CloseHandle(hFile);
                return FALSE;
            }
            siSync++;
        }
        result = moveFileData(hFile, siSync, siSize, tagSync + dSize);
        if (!result){
            ::CloseHandle(hFile);
            return FALSE;
        }
    }
    dSize += adjSize;
    fSize += adjSize;

    ::SetFilePointer(hFile, fSize, NULL, FILE_BEGIN);
    ::SetEndOfFile(hFile);

    //sizeZbg
    if ( riffFlag == 1 ) {
        ::memset(&buf, 0, sizeof(buf));
        buf[0] = (BYTE)( dSize & 0x000000ff);
        buf[1] = (BYTE)((dSize & 0x0000ffff) >> 8);
        buf[2] = (BYTE)((dSize & 0x00ffffff) >> 16);
        buf[3] = (BYTE)((dSize & 0xffffffff) >> 24);

        SetFilePointer(hFile, tagSync - 4, NULL, FILE_BEGIN);
        result = ::WriteFile(hFile, buf, 4, &bytesWrite, NULL);
        if (!result || bytesWrite != 4) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFile);
            return FALSE;
        }

        ::memset(&buf, 0, sizeof(buf));
        buf[0] = (BYTE)( (fSize - 8) & 0x000000ff);
        buf[1] = (BYTE)(((fSize - 8) & 0x0000ffff) >> 8);
        buf[2] = (BYTE)(((fSize - 8) & 0x00ffffff) >> 16);
        buf[3] = (BYTE)(((fSize - 8) & 0xffffffff) >> 24);

        SetFilePointer(hFile, 4, NULL, FILE_BEGIN);
        result = ::WriteFile(hFile, buf, 4, &bytesWrite, NULL);
        if (!result || bytesWrite != 4) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFile);
            return FALSE;
        }
    }

    ::CloseHandle(hFile);
    return TRUE;

}

BOOL moveFileData(HANDLE hFile,long dataSync, DWORD dataSize, long movePoint)
{
    BYTE*           pBuf;
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;

    pBuf = (BYTE*)::malloc(dataSize);
    if (pBuf == NULL){
        _setError(ERR_MALLOC);
        ::CloseHandle(hFile);
        return FALSE;
    }

    SetFilePointer(hFile, dataSync, NULL, FILE_BEGIN);
    result = ::ReadFile(hFile, pBuf, dataSize, &bytesRead, NULL);
    if (!result || bytesRead != dataSize) {
        _setError(ERR_MP3_FILE_READ);
        ::free(pBuf);
        return FALSE;
    }

    ::SetFilePointer(hFile, movePoint, NULL, FILE_BEGIN);
    result = ::WriteFile(hFile, pBuf, dataSize, &bytesWrite, NULL);
    if (!result || bytesWrite != dataSize) {
        _setError(ERR_MP3_FILE_WRITE);
        ::free(pBuf);
        return FALSE;
    }

    ::free(pBuf);

    return TRUE;
}

BOOL input_cutMacBinary(const char* pszName, const wchar_t* pszNameW)
{
    BYTE*           fBuf;
    HANDLE          hFileMac;               //^OҏWt@C
    BYTE            buf[1024];
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    DWORD           fSize;
    DWORD           target = 0;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, OPEN_EXISTING, &hFileMac) == FALSE)
		return FALSE;

    //}bNoCi`FbN
    SetFilePointer(hFileMac, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileMac, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileMac);
        return FALSE;
    }
    if (buf[0] != 0x00) {
        ::CloseHandle(hFileMac);
        return TRUE;
    }

    fSize = GetFileSize(hFileMac, NULL) - 128;

    fBuf = (BYTE*)::malloc(fSize);
    if (fBuf == NULL){
        _setError(ERR_MALLOC);
        ::CloseHandle(hFileMac);
        return FALSE;
    }

    SetFilePointer(hFileMac, 128, NULL, FILE_BEGIN);
    result = ::ReadFile(hFileMac, fBuf, fSize, &bytesRead, NULL);
    if (!result || bytesRead != fSize) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileMac);
        return FALSE;
    }

    for (target = fSize - 1; target > 0 && fBuf[target - 1] == 0; target --) {}

    ::SetFilePointer(hFileMac, 0, NULL, FILE_BEGIN);
    result = ::WriteFile(hFileMac, fBuf, target, &bytesWrite, NULL);
    if (!result || bytesWrite != target) {
        _setError(ERR_MP3_FILE_WRITE);
        ::CloseHandle(hFileMac);
        return FALSE;
    }

    ::SetEndOfFile(hFileMac);
    ::CloseHandle(hFileMac);
    ::free(fBuf);
    return TRUE;

}

BOOL input_changeWav(const char* pszName, const wchar_t* pszNameW)
{
    BOOL            Result;
    HANDLE          hFileMP3;               //^OҏWt@C
    BYTE            buf[1024];
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    DWORD           fSize;
    DWORD           dSize;
    DWORD           siSize;
    DWORD           target = 0;
    BYTE            tag[128];
    int             tagFlag = 0;
    int             lyFlag = 0;
    BYTE*           lyBuf;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, OPEN_EXISTING, &hFileMP3) == FALSE)
		return FALSE;

    fSize = ::GetFileSize(hFileMP3, NULL);
    dSize = fSize;
    siSize = 298 - 8;

    //RIFF`AID3 Tag v2`FbN
    SetFilePointer(hFileMP3, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileMP3, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileMP3);
        return FALSE;
    }
    if (memcmp(buf, "RIFF", 4) == 0) {
        LIST_INFO_EX pListInfo;
        //SItB[h擾
        if ( !getListInfoEX(hFileMP3, &pListInfo) ){
            _setError(ERR_NON_RIFF);
            ::CloseHandle(hFileMP3);
            return FALSE;
        }   
        ::CloseHandle(hFileMP3);

        //MP3ϊ
        if ( !input_changeMp3(pszName, pszNameW) ){
            _setError(ERR_NON_RIFF);
            return FALSE;
        }   

        //WAVϊ
        if ( !input_changeWav(pszName, pszNameW) ){
            _setError(ERR_NON_RIFF);
            return FALSE;
        }   

        //SItB[hݒ
        if ( !input_setListInfoEX(pszName, pszNameW, &pListInfo) ){
            _setError(ERR_NON_RIFF);
            return FALSE;
        }
        return TRUE;
    }
/*
    else if (memcmp(buf, "ID3", 3) == 0) {
        ::CloseHandle(hFileMP3);
        return FALSE;
    }
*/
    else if (buf[0] == 0x00) {
        _setError(ERR_MAC_BIN);
        ::CloseHandle(hFileMP3);
        return FALSE;
    }

    SetFilePointer(hFileMP3, -(long)128, NULL, FILE_END);
    result = ReadFile(hFileMP3, tag, 128, &bytesRead, NULL);
    if (!result || bytesRead != 128) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileMP3);
        return FALSE;
    }
    if (memcmp(tag, "TAG", 3) == 0) {
        dSize -= 128;
        //siSize += 8 + 128;
        fSize -= 128;
        tagFlag = 1;
    }

    //LYRICS3݃`FbN
    DWORD lysize = 0;           //TCY
    DWORD lyfsize;          //tB[hTCY
    DWORD lysp = 0;         //X^[gʒu
    if (chkLYRICS3(hFileMP3, &lysize)) {
        if (m_Lyrics3InsertPoint == 0 ) {
            siSize = siSize + lysize - 10;
        } else if ( m_Lyrics3InsertPoint == 1 ) {
            siSize = siSize + lysize;
        }
        lyfsize = lysize;
        dSize -= lysize;
        fSize -= lysize;
        lyFlag = 1;

        if ( m_Lyrics3InsertPoint != 2 ) {
            //LYRICS3 `FbN
            if ((lysize % 2) == 1) {
                lyfsize++;
                siSize++;
                lysp = 1;
                lyBuf = (BYTE*)::malloc(lyfsize);
                lyBuf[0] = 0x00;
            } else {
                lyBuf = (BYTE*)::malloc(lysize);
            }

            if (lyBuf == NULL){
                _setError(ERR_MALLOC);
                ::CloseHandle(hFileMP3);
                ::free(lyBuf);
                return FALSE;
            }

            //|C^ړ
            if ( tagFlag == 1 ){
                SetFilePointer(hFileMP3, -(long)(128+lysize), NULL, FILE_END);
            } else {
                SetFilePointer(hFileMP3, -(long)(lysize), NULL, FILE_END);
            }

            result = ReadFile(hFileMP3, &lyBuf[lysp], lysize, &bytesRead, NULL);
            if (!result || bytesRead != lysize) {
                _setError(ERR_MP3_FILE_READ);
                ::CloseHandle(hFileMP3);
                ::free(lyBuf);
                return FALSE;
            }
            lysize = lyfsize;
        }
    }

    fSize = fSize + 12 + 38 + 4 + siSize + 12;  //fSize + Riff + frm + data + siSize + List

    ::memset(&buf, 0, sizeof(buf));
    ::memcpy(&buf[0], "RIFF", 4);
    buf[4] = (BYTE)((fSize - 8) & 0x000000ff);
    buf[5] = (BYTE)(((fSize - 8) & 0x0000ffff) >> 8);
    buf[6] = (BYTE)(((fSize - 8) & 0x00ffffff) >> 16);
    buf[7] = (BYTE)(((fSize - 8) & 0xffffffff) >> 24);
    ::memcpy(&buf[8], "WAVE", 4);
    ::memcpy(&buf[12], "fmt ", 4);
    target = 16;
    buf[target] = 0x1e;
    buf[++target] = 0x00;
    buf[++target] = 0x00;
    buf[++target] = 0x00;
    buf[++target] = 0x55;
    buf[++target] = 0x00;
    buf[++target] = 0x02;
    buf[++target] = 0x00;
    buf[++target] = 0x44;
    buf[++target] = 0xac;
    buf[++target] = 0x00;
    buf[++target] = 0x00;
    buf[++target] = 0x80;
    buf[++target] = 0x3e;
    buf[++target] = 0x00;
    buf[++target] = 0x00;
    buf[++target] = 0x01;
    buf[++target] = 0x00;
    buf[++target] = 0x00;
    buf[++target] = 0x00;
    buf[++target] = 0x0c;
    buf[++target] = 0x00;
    buf[++target] = 0x01;
    buf[++target] = 0x00;
    buf[++target] = 0x02;
    buf[++target] = 0x00;
    buf[++target] = 0x00;
    buf[++target] = 0x00;
    buf[++target] = 0xa1;
    buf[++target] = 0x01;
    buf[++target] = 0x01;
    buf[++target] = 0x00;
    buf[++target] = 0x71;
    buf[++target] = 0x05;
    target++;
    ::memcpy(&buf[target], "data", 4);
    target += 4;
    buf[target] = (BYTE)(dSize & 0x000000ff);
    buf[++target] = (BYTE)((dSize & 0x0000ffff) >> 8);
    buf[++target] = (BYTE)((dSize & 0x00ffffff) >> 16);
    buf[++target] = (BYTE)((dSize & 0xffffffff) >> 24);
    target++;

    Result = insertFileDataBegin(hFileMP3, (BYTE*)buf, target);

    ::memset(&buf, 0, sizeof(buf));
    ::memcpy(&buf[0], "LIST", 4);
    buf[4] = (BYTE)(siSize & 0x000000ff);
    buf[5] = (BYTE)((siSize & 0x0000ffff) >> 8);
    buf[6] = (BYTE)((siSize & 0x00ffffff) >> 16);
    buf[7] = (BYTE)((siSize & 0xffffffff) >> 24);
    ::memcpy(&buf[8], "INFO", 4);

    target = 12;
    ::memcpy(&buf[target], "INAM", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "IART", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "IPRD", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ICMT", 4);
    buf[target + 4] = 1;
    
    target += 10;
    ::memcpy(&buf[target], "IGNR", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ICRD", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ICOP", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "IENG", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ISRC", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ISFT", 4);
    buf[target + 4] = 0x0b;
    ::memcpy(&buf[target + 8], "VBMP3", 11);

    target += 20;
    ::memcpy(&buf[target], "IKEY", 4);
    buf[target + 4] = 1;
    
    target += 10;
    ::memcpy(&buf[target], "ITCH", 4);
    buf[target + 4] = 1;
    
    target += 10;
    ::memcpy(&buf[target], "ICMS", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ILYC", 4);

    if ( lyFlag == 0 || m_Lyrics3InsertPoint == 2 ) {
        buf[target + 4] = 1;

        target += 10;
        ::memcpy(&buf[target], "IID3", 4);
        buf[target + 4] = 0x80;

        target += 8;

        if ( tagFlag == 0 ) {
            int Cnt;
            ::memcpy(&buf[target], "TAG", 3);
            for (Cnt = 3; Cnt < 127; Cnt++){
                ::memcpy(&buf[target+Cnt], " ", 1);
            }
            buf[target+127] = 0xFA;
            SetFilePointer(hFileMP3, -(long)(lysize - lysp), NULL, FILE_END);
        } else {
            ::memcpy(&buf[target], tag, 128);
            SetFilePointer(hFileMP3, -(long)(128 + lysize - lysp), NULL, FILE_END);
        }

        target += 128;
        result = ::WriteFile(hFileMP3, &buf, target, &bytesWrite, NULL);
        if (!result || bytesWrite != target) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileMP3);
            if ( lyFlag == 1 ) ::free(lyBuf);
            return FALSE;
        }
        ::SetEndOfFile(hFileMP3);

    } else {
        if ( m_Lyrics3InsertPoint == 1 ) {
            buf[target + 4] = 1;

            target += 10;
            ::memcpy(&buf[target], "IID3", 4);
        }

        buf[target + 4] = (BYTE)((lysize + 128) & 0x000000ff);
        buf[target + 5] = (BYTE)(((lysize + 128) & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)(((lysize + 128) & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)(((lysize + 128) & 0xffffffff) >> 24);

        target += 8;

        long seekPoint = lysize;
        if ( tagFlag == 1 ) {
            seekPoint += 128;
        }
        SetFilePointer(hFileMP3, -(long)(seekPoint - lysp), NULL, FILE_END);

        //SItB[h }
        result = ::WriteFile(hFileMP3, &buf, target, &bytesWrite, NULL);
        if (!result || bytesWrite != target) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileMP3);
            if ( lyFlag == 1 ) ::free(lyBuf);
            return FALSE;
        }

        //LYRICS3 }
        if ( lysize != 0 ) {
            result = ::WriteFile(hFileMP3, lyBuf, lysize, &bytesWrite, NULL);
            if (!result || bytesWrite != lysize) {
                _setError(ERR_MP3_FILE_WRITE);
                ::CloseHandle(hFileMP3);
                if ( lyFlag == 1 ) ::free(lyBuf);
                return FALSE;
            }
        }

        //ID3Tag v1 }
        if ( tagFlag == 0 ) {
            int Cnt;
            ::memcpy(&buf[0], "TAG", 3);
            for (Cnt = 3; Cnt < 127; Cnt++){
                ::memcpy(&buf[Cnt], " ", 1);
            }
            buf[127] = 0xFA;
        } else {
            ::memcpy(&buf[0], tag, 128);
        }
        result = ::WriteFile(hFileMP3, &buf, 128, &bytesWrite, NULL);
        if (!result || bytesWrite != 128) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileMP3);
            if ( lyFlag == 1 ) ::free(lyBuf);
            return FALSE;
        }

    }

    ::CloseHandle(hFileMP3);
    if ( lyFlag == 1 ) ::free(lyBuf);
    return Result;

}


BOOL input_changeMp3(const char* pszName, const wchar_t* pszNameW)
{
    HANDLE          hFileList;              //^OҏWt@C
    BYTE*           fBuf;
    BYTE            buf[1024];
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    DWORD           fSync;                  //wb_̈ʒu
    DWORD           fLast;                  //f[^`N̍Ō
    DWORD           dSize;
    DWORD           target = 0;
    BYTE            tag[128];
    int             tagFlag = 0;
    RiffChunk       riff;
    CommonChunk     common;
    int             lyFlag = 0;
    BYTE*           lyBuf;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, OPEN_EXISTING, &hFileList) == FALSE)
		return FALSE;

    fSync = 0;
    dSize = GetFileSize(hFileList, NULL);

    //RIFF`AID3 Tag v2`FbN
    SetFilePointer(hFileList, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileList, buf, 4, &bytesRead, NULL);
    if (memcmp(buf, "RIFF", 4) != 0) {
        _setError(ERR_NON_RIFF);
        ::CloseHandle(hFileList);
        return FALSE;
    } else {
        //t@C̓ɖ߂ARIFF`Nǂݍ
        ::SetFilePointer(hFileList, 0, NULL, FILE_BEGIN);
        result = ::ReadFile(hFileList, &riff, sizeof(riff), &bytesRead, NULL);
        if (!result || bytesRead != sizeof(riff)) {
            _setError(ERR_MP3_FILE_READ);
            ::CloseHandle(hFileList);
            return FALSE;
        }
        fSync = sizeof(riff);
        //data`NT
        for (;;) {
            result = ::ReadFile(hFileList, &common, sizeof(common), &bytesRead, NULL);
            if (!result || bytesRead != sizeof(common)) {
                _setError(ERR_MP3_FILE_READ);
                ::CloseHandle(hFileList);
                return FALSE;
            }

            if (::memcmp(common.chunkID, "data", 4) == 0) {
                //data`N
                break;
            }
            else {
                //̃`N܂œǂݔ΂
                fSync = ::SetFilePointer(hFileList, common.chunkSize, NULL, FILE_CURRENT);
            }
        }

        //`NTCY`FbN
        if (common.chunkSize < dSize) {
            dSize = common.chunkSize;
        }

        fSync += bytesRead;
        fLast = fSync + dSize;
    }

    fBuf = (BYTE*)::malloc(dSize);
    if (fBuf == NULL){
        ::CloseHandle(hFileList);
        ::free(fBuf);
        _setError(ERR_MALLOC);
        return FALSE;
    }

    //ID3Tag v1 `FbN
    SetFilePointer(hFileList, -(long)128, NULL, FILE_END);
    result = ReadFile(hFileList, tag, 128, &bytesRead, NULL);
    if (!result || bytesRead != 128) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileList);
        ::free(fBuf);
        return FALSE;
    }
    if (memcmp(tag, "TAG", 3) == 0) {
        tagFlag = 1;
    }

    //LYRICS3݃`FbN
    DWORD lysize = 0;           //TCY
    if (chkLYRICS3(hFileList, &lysize)) {
        lyFlag = 1;

        lyBuf = (BYTE*)::malloc(lysize);
        if (lyBuf == NULL){
            _setError(ERR_MALLOC);
            ::CloseHandle(hFileList);
            ::free(lyBuf);
            ::free(fBuf);
            return FALSE;
        }

        if ( tagFlag == 1 ){
            SetFilePointer(hFileList, -(long)(128+lysize), NULL, FILE_END);
        } else {
            SetFilePointer(hFileList, -(long)(lysize), NULL, FILE_END);
        }

        result = ReadFile(hFileList, lyBuf, lysize, &bytesRead, NULL);
        if (!result || bytesRead != lysize) {
            _setError(ERR_MP3_FILE_READ);
            ::CloseHandle(hFileList);
            ::free(lyBuf);
            return FALSE;
        }

    }

    SetFilePointer(hFileList, fSync, NULL, FILE_BEGIN);
    result = ::ReadFile(hFileList, fBuf, dSize, &bytesRead, NULL);
    if (!result || bytesRead != dSize) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileList);
        if ( lyFlag == 1 ) ::free(lyBuf);
        ::free(fBuf);
        return FALSE;
    }

    ::SetFilePointer(hFileList, 0, NULL, FILE_BEGIN);
    result = ::WriteFile(hFileList, fBuf, dSize, &bytesWrite, NULL);
    if (!result || bytesWrite != dSize) {
        _setError(ERR_MP3_FILE_WRITE);
        ::CloseHandle(hFileList);
        if ( lyFlag == 1 ) ::free(lyBuf);
        ::free(fBuf);
        return FALSE;
    }

    if ( lyFlag == 1 ) {
        result = ::WriteFile(hFileList, lyBuf, lysize, &bytesWrite, NULL);
        if (!result || bytesWrite != lysize) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileList);
            if ( lyFlag == 1 ) ::free(lyBuf);
            ::free(fBuf);
            return FALSE;
        }
    }

    if ( tagFlag == 1 ) {
        result = ::WriteFile(hFileList, &tag, 128, &bytesWrite, NULL);
        if (!result || bytesWrite != 128) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileList);
            if ( lyFlag == 1 ) ::free(lyBuf);
            ::free(fBuf);
            return FALSE;
        }
    }

    ::SetEndOfFile(hFileList);
    ::CloseHandle(hFileList);
    if ( lyFlag == 1 ) ::free(lyBuf);
    ::free(fBuf);
    return TRUE;
}


BOOL setListInfo(const char* pszName, const wchar_t* pszNameW, LIST_INFO* pListInfo)
{
    HANDLE          hFileList;              //^OҏWt@C
    BYTE            buf[102400];
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    DWORD           fSync;                  //wb_̈ʒu
    DWORD           fLast;                  //f[^`N̍Ō
    DWORD           fSize;
    DWORD           dSize;
    DWORD           listSize;
    DWORD           target = 0;
    BYTE            tag[128];
    int             tagFlag = 0;
    RiffChunk       riff;
    CommonChunk     common;
    int             lyFlag = 0;
    BYTE*           lyBuf;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, OPEN_EXISTING, &hFileList) == FALSE)
		return FALSE;

	fSync = 0;
    dSize = GetFileSize(hFileList, NULL);

    //RIFF`AID3 Tag v2`FbN
    SetFilePointer(hFileList, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileList, buf, 4, &bytesRead, NULL);
    if (memcmp(buf, "RIFF", 4) != 0) {
        ::CloseHandle(hFileList);
        _setError(ERR_NON_RIFF);
        return FALSE;
    } else {
        //t@C̓ɖ߂ARIFF`Nǂݍ
        ::SetFilePointer(hFileList, 0, NULL, FILE_BEGIN);
        result = ::ReadFile(hFileList, &riff, sizeof(riff), &bytesRead, NULL);
        if (!result || bytesRead != sizeof(riff)) {
            ::CloseHandle(hFileList);
            _setError(ERR_MP3_FILE_READ);
            return FALSE;
        }
        fSync = sizeof(riff);
        //data`NT
        for (;;) {
            result = ::ReadFile(hFileList, &common, sizeof(common), &bytesRead, NULL);
            if (!result || bytesRead != sizeof(common)) {
                _setError(ERR_MP3_FILE_READ);
                ::CloseHandle(hFileList);
                return FALSE;
            }

            if (::memcmp(common.chunkID, "data", 4) == 0) {
                //data`N
                break;
            }
            else {
                //̃`N܂œǂݔ΂
                fSync = ::SetFilePointer(hFileList, common.chunkSize, NULL, FILE_CURRENT);
            }
        }

        //`NTCY`FbN
        if (common.chunkSize < dSize) {
            dSize = common.chunkSize;
        }

        fSync += bytesRead;
        fLast = fSync + dSize;
    }

    //ID3Tag v1`FbN
    SetFilePointer(hFileList, -(long)128, NULL, FILE_END);
    result = ReadFile(hFileList, tag, 128, &bytesRead, NULL);
    if (!result || bytesRead != 128) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileList);
        return FALSE;
    }
    if (memcmp(tag, "TAG", 3) == 0) {
        tagFlag = 1;
    }

    //LYRICS3݃`FbN
    DWORD lysize = 0;           //TCY
    DWORD lyfsize;          //tB[hTCY
    DWORD lysp = 0;         //X^[gʒu
    if (chkLYRICS3(hFileList, &lysize)) {
        lyFlag = 1;
        lyfsize = lysize;

        //LYRICS3 `FbN
        if ((lysize % 2) == 1) {
            lyfsize++;
            lysp = 1;
            lyBuf = (BYTE*)::malloc(lyfsize);
            lyBuf[0] = 0x00;
        } else {
            lyBuf = (BYTE*)::malloc(lysize);
        }

        if (lyBuf == NULL){
            _setError(ERR_MALLOC);
            ::CloseHandle(hFileList);
            ::free(lyBuf);
            return FALSE;
        }

        //|C^ړ
        if ( tagFlag == 1 ){
            SetFilePointer(hFileList, -(long)(128+lysize), NULL, FILE_END);
        } else {
            SetFilePointer(hFileList, -(long)(lysize), NULL, FILE_END);
        }

        result = ReadFile(hFileList, &lyBuf[lysp], lysize, &bytesRead, NULL);
        if (!result || bytesRead != lysize) {
            _setError(ERR_MP3_FILE_READ);
            ::CloseHandle(hFileList);
            ::free(lyBuf);
            return FALSE;
        }
        lysize = lyfsize;
    }

    //f[^TCY΍
    if ( (dSize % 2) == 1 ){
        buf[0] = 0x00;
        SetFilePointer(hFileList, fLast, NULL, FILE_BEGIN);
        result = ::WriteFile(hFileList, &buf, 1, &bytesWrite, NULL);
        if (!result || bytesWrite != 1) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileList);
            ::free(lyBuf);
            return FALSE;
        }
        fLast++;
    }

    ::memset(&buf, 0, sizeof(buf));

    target = 12;
    listSize = ::strlen(pListInfo->INAM) + 1;
    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->INAM[listSize - 1] == ' ' ||
                                                     pListInfo->INAM[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "INAM", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->INAM, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->IART[listSize - 1] == ' ' ||
                                                     pListInfo->IART[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IART", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IART, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->IPRD[listSize - 1] == ' ' ||
                                                     pListInfo->IPRD[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IPRD", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IPRD, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->ICMT[listSize - 1] == ' ' || 
                                                     pListInfo->ICMT[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ICMT", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ICMT, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->IGNR[listSize - 1] == ' ' || 
                                                     pListInfo->IGNR[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IGNR", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IGNR, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->ICRD[listSize - 1] == ' ' ||
                                                     pListInfo->ICRD[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ICRD", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ICRD, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->ICOP[listSize - 1] == ' ' ||
                                                     pListInfo->ICOP[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ICOP", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ICOP, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->IENG[listSize - 1] == ' ' ||
                                                     pListInfo->IENG[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IENG", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IENG, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->ISRC[listSize - 1] == ' ' ||
                                                     pListInfo->ISRC[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ISRC", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ISRC, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->ISFT[listSize - 1] == ' ' ||
                                                     pListInfo->ISFT[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ISFT", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ISFT, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->IKEY[listSize - 1] == ' ' ||
                                                     pListInfo->IKEY[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IKEY", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IKEY, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->ITCH[listSize - 1] == ' ' ||
                                                     pListInfo->ITCH[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ITCH", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ITCH, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->ICMS[listSize - 1] == ' ' ||
                                                     pListInfo->ICMS[listSize - 1] == '\0') ; listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ICMS", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ICMS, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = INFO_NAME_SIZE; listSize > 0 && (pListInfo->ILYC[listSize - 1] == ' ' ||
                                                     pListInfo->ILYC[listSize - 1] == '\0') ; listSize --) {}
    listSize++;
    ::memcpy(&buf[target], "ILYC", 4);

    if ( lyFlag == 0 || m_Lyrics3InsertPoint == 2 ) {
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ILYC, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;

        ::memcpy(&buf[target], "IID3", 4);
        buf[target + 4] = 0x80;

        target += 8;

        if ( tagFlag == 0 ) {
            int Cnt;
            ::memcpy(&buf[target], "TAG", 3);
            for (Cnt = 3; Cnt < 127; Cnt++){
                ::memcpy(&buf[target+Cnt], " ", 1);
            }
            buf[target+127] = 0xFA;
        } else {
            ::memcpy(&buf[target], tag, 128);
        }

        target += 128;
    } else {
        if ( m_Lyrics3InsertPoint == 1 ) {
            buf[target + 4] = (BYTE)(listSize & 0x000000ff);
            buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
            buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
            buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
            ::memcpy(&buf[target + 8], pListInfo->ILYC, listSize - 1);
            target += 8 + listSize;
            if ((listSize % 2) == 1 ) target += 1;

            ::memcpy(&buf[target], "IID3", 4);
        }

        buf[target + 4] = (BYTE)((lysize + 128) & 0x000000ff);
        buf[target + 5] = (BYTE)(((lysize + 128) & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)(((lysize + 128) & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)(((lysize + 128) & 0xffffffff) >> 24);

        target += 8 + lysize + 128;
    }

    ::memcpy(&buf[0], "LIST", 4);
    buf[4] = (BYTE)((target - 8) & 0x000000ff);
    buf[5] = (BYTE)(((target - 8) & 0x0000ffff) >> 8);
    buf[6] = (BYTE)(((target - 8) & 0x00ffffff) >> 16);
    buf[7] = (BYTE)(((target - 8) & 0xffffffff) >> 24);
    ::memcpy(&buf[8], "INFO", 4);

    SetFilePointer(hFileList, fLast, NULL, FILE_BEGIN);
    result = ::WriteFile(hFileList, &buf, target, &bytesWrite, NULL);
    if (!result || bytesWrite != target) {
        _setError(ERR_MP3_FILE_WRITE);
        ::CloseHandle(hFileList);
        ::free(lyBuf);
        return FALSE;
    }
    ::SetEndOfFile(hFileList);

    if ( lyFlag != 0 && m_Lyrics3InsertPoint != 2 ) {
        SetFilePointer(hFileList, -(long)(lysize + 128), NULL, FILE_END);
        result = ::WriteFile(hFileList, lyBuf, lysize, &bytesWrite, NULL);
        if (!result || bytesWrite != lysize) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileList);
            ::free(lyBuf);
            return FALSE;
        }

        if ( tagFlag == 0 ) {
            int Cnt;
            ::memcpy(&buf[0], "TAG", 3);
            for (Cnt = 3; Cnt < 127; Cnt++){
                ::memcpy(&buf[target+Cnt], " ", 1);
            }
            buf[127] = 0xFA;
        } else {
            ::memcpy(&buf[0], tag, 128);
        }

        result = ::WriteFile(hFileList, &buf, 128, &bytesWrite, NULL);
        if (!result || bytesWrite != 128) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileList);
            ::free(lyBuf);
            return FALSE;
        }

        ::SetEndOfFile(hFileList);
    }

    fSize = fLast + target;

    ::memset(&buf, 0, sizeof(buf));
    buf[0] = (BYTE)((fSize - 8) & 0x000000ff);
    buf[1] = (BYTE)(((fSize - 8) & 0x0000ffff) >> 8);
    buf[2] = (BYTE)(((fSize - 8) & 0x00ffffff) >> 16);
    buf[3] = (BYTE)(((fSize - 8) & 0xffffffff) >> 24);

    SetFilePointer(hFileList, 4, NULL, FILE_BEGIN);
    result = ::WriteFile(hFileList, &buf, 4, &bytesWrite, NULL);
    if (!result || bytesWrite != 4) {
        _setError(ERR_MP3_FILE_WRITE);
        ::CloseHandle(hFileList);
        ::free(lyBuf);
        return FALSE;
    }

    ::CloseHandle(hFileList);
    ::free(lyBuf);
    return TRUE;

}

BOOL input_changeRmp(const char* pszName, const wchar_t* pszNameW)
{
    BOOL            Result;
    HANDLE          hFileMP3;               //^OҏWt@C
    BYTE            buf[1024];
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    DWORD           fSize;
    DWORD           dSize;
    DWORD           siSize;
    DWORD           target = 0;
    BYTE            tag[128];
    int             tagFlag = 0;
    int             lyFlag = 0;
    BYTE*           lyBuf;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, OPEN_EXISTING, &hFileMP3) == FALSE)
		return FALSE;

	fSize = ::GetFileSize(hFileMP3, NULL);
    dSize = fSize;
    siSize = 298 - 8;

    //RIFF`AID3 Tag v2`FbN
    SetFilePointer(hFileMP3, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileMP3, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileMP3);
        return FALSE;
    }
    if (memcmp(buf, "RIFF", 4) == 0) {
        LIST_INFO_EX pListInfo;
        //SItB[h擾
        if ( !getListInfoEX(hFileMP3, &pListInfo) ){
            _setError(ERR_NON_RIFF);
            ::CloseHandle(hFileMP3);
            return FALSE;
        }   
        ::CloseHandle(hFileMP3);

        //MP3ϊ
        if ( !input_changeMp3(pszName, pszNameW) ){
            _setError(ERR_NON_RIFF);
            return FALSE;
        }   

        //RMPϊ
        if ( !input_changeRmp(pszName, pszNameW) ){
            _setError(ERR_NON_RIFF);
            return FALSE;
        }   

        //SItB[hݒ
        if ( !input_setListInfoEX(pszName, pszNameW, &pListInfo) ){
            _setError(ERR_NON_RIFF);
            return FALSE;
        }   
        return TRUE;
    }
/*
    else if (memcmp(buf, "ID3", 3) == 0) {
        ::CloseHandle(hFileMP3);
        return FALSE;
    }
*/
    else if (buf[0] == 0x00) {
        _setError(ERR_MAC_BIN);
        ::CloseHandle(hFileMP3);
        return FALSE;
    }

    SetFilePointer(hFileMP3, -(long)128, NULL, FILE_END);
    result = ReadFile(hFileMP3, tag, 128, &bytesRead, NULL);
    if (!result || bytesRead != 128) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileMP3);
        return FALSE;
    }
    if (memcmp(tag, "TAG", 3) == 0) {
        dSize -= 128;
        //siSize += 8 + 128;
        fSize -= 128;
        tagFlag = 1;
    }

    //LYRICS3݃`FbN
    DWORD lysize = 0;           //TCY
    DWORD lyfsize;          //tB[hTCY
    DWORD lysp = 0;         //X^[gʒu
    if (chkLYRICS3(hFileMP3, &lysize)) {
        if (m_Lyrics3InsertPoint == 0 ) {
            siSize = siSize + lysize - 10;
        } else if ( m_Lyrics3InsertPoint == 1 ) {
            siSize = siSize + lysize;
        }
        lyfsize = lysize;
        dSize -= lysize;
        fSize -= lysize;
        lyFlag = 1;

        if ( m_Lyrics3InsertPoint != 2 ) {
            //LYRICS3 `FbN
            if ((lysize % 2) == 1) {
                lyfsize++;
                siSize++;
                lysp = 1;
                lyBuf = (BYTE*)::malloc(lyfsize);
                lyBuf[0] = 0x00;
            } else {
                lyBuf = (BYTE*)::malloc(lysize);
            }

            if (lyBuf == NULL){
                _setError(ERR_MALLOC);
                ::CloseHandle(hFileMP3);
                ::free(lyBuf);
                return FALSE;
            }

            //|C^ړ
            if ( tagFlag == 1 ){
                SetFilePointer(hFileMP3, -(long)(128+lysize), NULL, FILE_END);
            } else {
                SetFilePointer(hFileMP3, -(long)(lysize), NULL, FILE_END);
            }

            result = ReadFile(hFileMP3, &lyBuf[lysp], lysize, &bytesRead, NULL);
            if (!result || bytesRead != lysize) {
                _setError(ERR_MP3_FILE_READ);
                ::CloseHandle(hFileMP3);
                ::free(lyBuf);
                return FALSE;
            }
            lysize = lyfsize;
        }
    }

    fSize = fSize + 12 + 4 + siSize + 12;  //fSize + Riff + data + siSize + List

    ::memset(&buf, 0, sizeof(buf));
    ::memcpy(&buf[0], "RIFF", 4);
    buf[4] = (BYTE)((fSize - 8) & 0x000000ff);
    buf[5] = (BYTE)(((fSize - 8) & 0x0000ffff) >> 8);
    buf[6] = (BYTE)(((fSize - 8) & 0x00ffffff) >> 16);
    buf[7] = (BYTE)(((fSize - 8) & 0xffffffff) >> 24);
    ::memcpy(&buf[8], "RMP3", 4);
    ::memcpy(&buf[12], "data", 4);
    buf[16] = (BYTE)(dSize & 0x000000ff);
    buf[17] = (BYTE)((dSize & 0x0000ffff) >> 8);
    buf[18] = (BYTE)((dSize & 0x00ffffff) >> 16);
    buf[19] = (BYTE)((dSize & 0xffffffff) >> 24);

    Result = insertFileDataBegin(hFileMP3, (BYTE*)buf, 20);

    ::memset(&buf, 0, sizeof(buf));
    ::memcpy(&buf[0], "LIST", 4);
    buf[4] = (BYTE)(siSize & 0x000000ff);
    buf[5] = (BYTE)((siSize & 0x0000ffff) >> 8);
    buf[6] = (BYTE)((siSize & 0x00ffffff) >> 16);
    buf[7] = (BYTE)((siSize & 0xffffffff) >> 24);
    ::memcpy(&buf[8], "INFO", 4);

    target = 12;
    ::memcpy(&buf[target], "INAM", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "IART", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "IPRD", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ICMT", 4);
    buf[target + 4] = 1;
    
    target += 10;
    ::memcpy(&buf[target], "IGNR", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ICRD", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ICOP", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "IENG", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ISRC", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ISFT", 4);
    buf[target + 4] = 0x0b;
    ::memcpy(&buf[target + 8], "VBMP3", 11);

    target += 20;
    ::memcpy(&buf[target], "IKEY", 4);
    buf[target + 4] = 1;
    
    target += 10;
    ::memcpy(&buf[target], "ITCH", 4);
    buf[target + 4] = 1;
    
    target += 10;
    ::memcpy(&buf[target], "ICMS", 4);
    buf[target + 4] = 1;

    target += 10;
    ::memcpy(&buf[target], "ILYC", 4);

    if ( lyFlag == 0 || m_Lyrics3InsertPoint == 2 ) {
        buf[target + 4] = 1;

        target += 10;
        ::memcpy(&buf[target], "IID3", 4);
        buf[target + 4] = 0x80;

        target += 8;

        if ( tagFlag == 0 ) {
            int Cnt;
            ::memcpy(&buf[target], "TAG", 3);
            for (Cnt = 3; Cnt < 127; Cnt++){
                ::memcpy(&buf[target+Cnt], " ", 1);
            }
            buf[target+127] = 0xFA;
            SetFilePointer(hFileMP3, -(long)(lysize - lysp), NULL, FILE_END);
        } else {
            ::memcpy(&buf[target], tag, 128);
            SetFilePointer(hFileMP3, -(long)(128 + lysize - lysp), NULL, FILE_END);
        }

        target += 128;
        result = ::WriteFile(hFileMP3, &buf, target, &bytesWrite, NULL);
        if (!result || bytesWrite != target) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileMP3);
            if ( lyFlag == 1 ) ::free(lyBuf);
            return FALSE;
        }
        ::SetEndOfFile(hFileMP3);

    } else {
        if ( m_Lyrics3InsertPoint == 1 ) {
            buf[target + 4] = 1;

            target += 10;
            ::memcpy(&buf[target], "IID3", 4);
        }

        buf[target + 4] = (BYTE)((lysize + 128) & 0x000000ff);
        buf[target + 5] = (BYTE)(((lysize + 128) & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)(((lysize + 128) & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)(((lysize + 128) & 0xffffffff) >> 24);

        target += 8;

        long seekPoint = lysize - lysp;
        if ( tagFlag == 1 ) {
            seekPoint += 128;
        }
        SetFilePointer(hFileMP3, -seekPoint, NULL, FILE_END);

        //SItB[h }
        result = ::WriteFile(hFileMP3, &buf, target, &bytesWrite, NULL);
        if (!result || bytesWrite != target) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileMP3);
            if ( lyFlag == 1 ) ::free(lyBuf);
            return FALSE;
        }

        //LYRICS3 }
        if ( lysize != 0 ) {
            result = ::WriteFile(hFileMP3, lyBuf, lysize, &bytesWrite, NULL);
            if (!result || bytesWrite != lysize) {
                _setError(ERR_MP3_FILE_WRITE);
                ::CloseHandle(hFileMP3);
                if ( lyFlag == 1 ) ::free(lyBuf);
                return FALSE;
            }
        }

        //ID3Tag v1 }
        if ( tagFlag == 0 ) {
            int Cnt;
            ::memcpy(&buf[0], "TAG", 3);
            for (Cnt = 3; Cnt < 127; Cnt++){
                ::memcpy(&buf[Cnt], " ", 1);
            }
            buf[127] = 0xFA;
        } else {
            ::memcpy(&buf[0], tag, 128);
        }
        result = ::WriteFile(hFileMP3, &buf, 128, &bytesWrite, NULL);
        if (!result || bytesWrite != 128) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(hFileMP3);
            if ( lyFlag == 1 ) ::free(lyBuf);
            return FALSE;
        }
    }

    ::CloseHandle(hFileMP3);
    if ( lyFlag == 1 ) ::free(lyBuf);
    return Result;

}

BOOL insertFileDataBegin(HANDLE hFileHandle, BYTE* lpData, DWORD dSize)
{
    BYTE*           fBuf;
    BOOL            result;
    DWORD           fSize;
    DWORD           bytesRead;
    DWORD           bytesWrite;

    //t@CI[v
    if (hFileHandle == INVALID_HANDLE_VALUE) {
        _setError(ERR_MP3_FILE_OPEN);
        return FALSE;
    }

    fSize = ::GetFileSize(hFileHandle, NULL);
    fBuf = (BYTE*)::malloc(fSize);
    if (fBuf == NULL){
        _setError(ERR_MALLOC);
        return FALSE;
    }
    
    SetFilePointer(hFileHandle, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileHandle, fBuf, fSize, &bytesRead, NULL);
    if (!result || bytesRead != fSize) {
        //::CloseHandle(hFileHandle);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }

    ::SetFilePointer(hFileHandle, 0, NULL, FILE_BEGIN);
    result = ::WriteFile(hFileHandle, lpData, dSize, &bytesWrite, NULL);
    if (!result || bytesWrite != dSize) {
        //::CloseHandle(hFileHandle);
        _setError(ERR_MP3_FILE_WRITE);
        return FALSE;
    }

    result = ::WriteFile(hFileHandle, fBuf, fSize, &bytesWrite, NULL);
    if (!result || bytesWrite != fSize) {
        //::CloseHandle(hFileHandle);
        _setError(ERR_MP3_FILE_WRITE);
        return FALSE;
    }

    ::SetEndOfFile(hFileHandle);
    //::CloseHandle(hFileHandle);
    ::free(fBuf);
    return TRUE;

}

BOOL getListInfo(LIST_INFO* pListInfo)
{
    //t@CI[v
    if (m_hFile == INVALID_HANDLE_VALUE) {
        _setError(ERR_MP3_FILE_OPEN);
        return FALSE;
    }

    ::memset(pListInfo, 0, sizeof(LIST_INFO));

    //XgZbg
    ::memcpy(pListInfo->IART,  listInfo.IART,  sizeof(listInfo.IART));
    ::memcpy(pListInfo->ICMS,  listInfo.ICMS,  sizeof(listInfo.ICMS));
    ::memcpy(pListInfo->ICMT,  listInfo.ICMT,  sizeof(listInfo.ICMT));
    ::memcpy(pListInfo->ICOP,  listInfo.ICOP,  sizeof(listInfo.ICOP));
    ::memcpy(pListInfo->ICRD,  listInfo.ICRD,  sizeof(listInfo.ICRD));
    ::memcpy(pListInfo->IENG,  listInfo.IENG,  sizeof(listInfo.IENG));
    ::memcpy(pListInfo->IGNR,  listInfo.IGNR,  sizeof(listInfo.IGNR));
    ::memcpy(pListInfo->IKEY,  listInfo.IKEY,  sizeof(listInfo.IKEY));
    ::memcpy(pListInfo->ILYC,  listInfo.ILYC,  sizeof(listInfo.ILYC));
    ::memcpy(pListInfo->INAM,  listInfo.INAM,  sizeof(listInfo.INAM));
    ::memcpy(pListInfo->IPRD,  listInfo.IPRD,  sizeof(listInfo.IPRD));
    ::memcpy(pListInfo->ISFT,  listInfo.ISFT,  sizeof(listInfo.ISFT));
    ::memcpy(pListInfo->ISRC,  listInfo.ISRC,  sizeof(listInfo.ISRC));
    ::memcpy(pListInfo->ITCH,  listInfo.ITCH,  sizeof(listInfo.ITCH));
    
    return TRUE;
}

BOOL getFileTagInfo(const char* pszName, const wchar_t* pszNameW, TAG_INFO* pTagInfo)
{
    HANDLE          hFileMpeg;              //^OҏWt@C

    ::memset(pTagInfo, 0, sizeof(TAG_INFO));

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE, OPEN_EXISTING, &hFileMpeg) == FALSE)
		return FALSE;

    //ID3 Tagv1`FbN
    ::readID3v1(hFileMpeg, pTagInfo);

    ::CloseHandle(hFileMpeg);
    return TRUE;
}

BOOL getFileTagInfoEX(const char* pszName, const wchar_t* pszNameW, TAG_INFO_11* pTagInfo)
{
    HANDLE          hFileMpeg;              //^OҏWt@C

    ::memset(pTagInfo, 0, sizeof(TAG_INFO_11));

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE, OPEN_EXISTING, &hFileMpeg) == FALSE)
		return FALSE;

    //ID3 Tagv1`FbN
    ::readID3v11(hFileMpeg, pTagInfo);

    ::CloseHandle(hFileMpeg);
    return TRUE;
}

BOOL getFileInfo(const char* pszName, const wchar_t* pszNameW, TAG_INFO* pTagInfo, MPEG_INFO* pMpegInfo)
{
    HANDLE          hFileMpeg;              //^OҏWt@C
    BOOL            result;
    int             fSync;                  //wb_̈ʒu
    DWORD           dSize;                  //MPEG Audiõf[^TCY
    BYTE            buf[1024];
    DWORD           bytesRead;
    MPEG_DECODE_INFO dInfo;
    LIST_INFO       ListInfo;
    DWORD           fSize;
    int             padding;

    //
    ::memset(pTagInfo, 0, sizeof(TAG_INFO));
    ::memset(pMpegInfo, 0, sizeof(MPEG_INFO));

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE, OPEN_EXISTING, &hFileMpeg) == FALSE)
		return FALSE;

    fSync = 0;
    dSize = GetFileSize(hFileMpeg, NULL);
    fSize = dSize;

    SetFilePointer(hFileMpeg, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileMpeg, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileMpeg);
        return FALSE;
    }

    //RIFF`AID3 Tag v2`FbN
    if (memcmp(buf, "RIFF", 4) == 0) {
        ::readRIFF(hFileMpeg, &ListInfo, &fSync, &dSize);
        if (RiffStatus != 0) {
            ::CloseHandle(hFileMpeg);
            _setError(ERR_UNKNOWN_FILE);
            return FALSE;
        } else {
            if (readID3v2(hFileMpeg, &fSync)) {
                dSize -= fSync;
            }
        }
    }
    else if (memcmp(buf, "ID3", 3) == 0) {
        if (readID3v2(hFileMpeg, &fSync)) {
            dSize -= fSync;
        }
    }
    //Tag f[^
    else if (buf[0] == 0x00) {
            fSync = 128;
    }
    else {
        fSync = 0;
    }

    //wb_擾
    ::SetFilePointer(hFileMpeg, fSync, NULL, FILE_BEGIN);
    result = ::ReadFile(hFileMpeg, buf, sizeof(buf), &bytesRead, NULL);
    if (!result || bytesRead == 0) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }
    //wb_T
    int sync;
    if (!::mp3FindSync(buf, bytesRead, &sync)) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_FRAME_HEADER_NOT_FOUND);
        return FALSE;
    }
    fSync += sync;

    //wb_ʒu擾
    ::SetFilePointer(hFileMpeg, fSync, NULL, FILE_BEGIN);
    result = ::ReadFile(hFileMpeg, buf, sizeof(buf), &bytesRead, NULL);
    if (!result || bytesRead == 0) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }
    
    //wb_擾
    if (!::mp3GetDecodeInfo(buf, bytesRead, &dInfo, 0)) {
        _setError(ERR_FRAME_HEADER_READ);
        ::CloseHandle(hFileMpeg);
        return FALSE;
    }

    //ID3 Tagv1`FbN
    if (readID3v1(hFileMpeg, pTagInfo)) {
        if (fSize == dSize) 
            dSize -= 128;
    }

    //LYRICS3݃`FbN
    DWORD lysize;
    if (chkLYRICS3(hFileMpeg, &lysize)) {
        dSize -= lysize;
    }

    //MPEGRs[
    pMpegInfo->version = dInfo.header.version;
    pMpegInfo->layer = dInfo.header.layer;
    pMpegInfo->crcDisable = (dInfo.header.error_prot) ? 0 : 1;
    pMpegInfo->mode = dInfo.header.mode;
    pMpegInfo->extension = dInfo.header.mode_ext;
    pMpegInfo->copyright = dInfo.header.copyright;
    pMpegInfo->original = dInfo.header.original;
    pMpegInfo->emphasis = dInfo.header.emphasis;
    pMpegInfo->channels = dInfo.channels;
    pMpegInfo->bitRate = dInfo.bitRate / 1000;
    pMpegInfo->fileSize = fSize;
    pMpegInfo->samplingRate = dInfo.frequency;
    padding = dInfo.header.padding;

    if (dInfo.frames == 0) {
        dInfo.frames = dSize / dInfo.minInputSize;
    }
    pMpegInfo->flames = dInfo.frames;
    int samples;
    int bytes = dInfo.frames * dInfo.outputSize;
    if ( bytes > ALLOW_DATA_SIZE ) {
        samples = (long)((double)bytes / (double)dInfo.bitsPerSample / (double)dInfo.channels * (double)8.0);
    } else {
        samples = (long)((double)bytes * (double)8.0 / (double)dInfo.bitsPerSample / (double)dInfo.channels);
    }

    pMpegInfo->totalSec = samples / dInfo.frequency;

    ::CloseHandle(hFileMpeg);
    return TRUE;

}

BOOL getFileInfoMtr(const char* pszName, const wchar_t* pszNameW, TAG_INFO* pTagInfo, MPEG_INFO* pMpegInfo, LIST_INFO* pListInfo,int* pFileType, FILE_TIME *pFileTime)
{
    HANDLE          hFileMpeg;              //^OҏWt@C
    BOOL            result;
    int             fSync;                  //wb_̈ʒu
    DWORD           dSize;                  //MPEG Audiõf[^TCY
    BYTE            buf[1024];
    DWORD           bytesRead;
    MPEG_DECODE_INFO dInfo;
    DWORD           fSize;
    int             padding;
    BY_HANDLE_FILE_INFORMATION lpFileInformation;
    FILETIME        ftWork;
    SYSTEMTIME      stCreationTime;
    SYSTEMTIME      stLastAccessTime;
    SYSTEMTIME      stLastWriteTime;

    
    //
    ::memset(pTagInfo, 0, sizeof(TAG_INFO));
    ::memset(pMpegInfo, 0, sizeof(MPEG_INFO));
    ::memset(pListInfo, 0, sizeof(LIST_INFO));
    m_FileType = 0;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE, OPEN_EXISTING, &hFileMpeg) == FALSE)
		return FALSE;

    //t@C擾
    ::GetFileInformationByHandle(hFileMpeg, &lpFileInformation);
    FileTimeToLocalFileTime(&lpFileInformation.ftCreationTime, &ftWork);
    FileTimeToSystemTime(&ftWork, &stCreationTime);
    pFileTime->CreateDate = stCreationTime.wYear * 10000 + stCreationTime.wMonth * 100 + stCreationTime.wDay;
    pFileTime->CreateTime = stCreationTime.wHour * 10000 + stCreationTime.wMinute * 100 + stCreationTime.wSecond;
    FileTimeToLocalFileTime(&lpFileInformation.ftLastAccessTime, &ftWork);
    FileTimeToSystemTime(&ftWork, &stLastAccessTime);
    pFileTime->LastAccessDate = stLastAccessTime.wYear * 10000 + stLastAccessTime.wMonth * 100 + stLastAccessTime.wDay;
    pFileTime->LastAccessTime = stLastAccessTime.wHour * 10000 + stLastAccessTime.wMinute * 100 + stLastAccessTime.wSecond;
    FileTimeToLocalFileTime(&lpFileInformation.ftLastWriteTime, &ftWork);
    FileTimeToSystemTime(&ftWork, &stLastWriteTime);
    pFileTime->LastWriteDate = stLastWriteTime.wYear * 10000 + stLastWriteTime.wMonth * 100 + stLastWriteTime.wDay;
    pFileTime->LastWriteTime = stLastWriteTime.wHour * 10000 + stLastWriteTime.wMinute * 100 + stLastWriteTime.wSecond;

    fSync = 0;
    dSize = GetFileSize(hFileMpeg, NULL);
    fSize = dSize;

    SetFilePointer(hFileMpeg, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileMpeg, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }

    //RIFF`AID3 Tag v2`FbN
    if (memcmp(buf, "RIFF", 4) == 0) {
        ::readRIFF(hFileMpeg, pListInfo, &fSync, &dSize);
        if (RiffStatus != 0) {
            ::CloseHandle(hFileMpeg);
            _setError(ERR_UNKNOWN_FILE);
            return FALSE;
        } else {
            if (readID3v2(hFileMpeg, &fSync)) {
                dSize -= fSync;
            }
        }
    }
    else if (memcmp(buf, "ID3", 3) == 0) {
        if (readID3v2(hFileMpeg, &fSync)) {
            dSize -= fSync;
        }
    }
    //Tag f[^
    else if (buf[0] == 0x00) {
            m_FileType += 8;
            fSync = 128;
    }
    else {
        fSync = 0;
    }

    //wb_擾
    ::SetFilePointer(hFileMpeg, fSync, NULL, FILE_BEGIN);
    result = ::ReadFile(hFileMpeg, buf, sizeof(buf), &bytesRead, NULL);
    if (!result || bytesRead == 0) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }
    //wb_T
    int sync;
    if (!::mp3FindSync(buf, bytesRead, &sync)) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_FRAME_HEADER_NOT_FOUND);
        return FALSE;
    }
    fSync += sync;

    //wb_ʒu擾
    ::SetFilePointer(hFileMpeg, fSync, NULL, FILE_BEGIN);
    result = ::ReadFile(hFileMpeg, buf, sizeof(buf), &bytesRead, NULL);
    if (!result || bytesRead == 0) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }

    //wb_擾
    if (!::mp3GetDecodeInfo(buf, bytesRead, &dInfo, 0)) {
        _setError(ERR_FRAME_HEADER_READ);
        ::CloseHandle(hFileMpeg);
        return FALSE;
    }

    //ID3 Tagv1`FbN
    if (readID3v1(hFileMpeg, pTagInfo)) {
        if (fSize == dSize) 
            //m_FileType += 16;
            dSize -= 128;
    }

    //LYRICS3݃`FbN
    DWORD lysize;
    if (chkLYRICS3(hFileMpeg, &lysize)) {
        dSize -= lysize;
    }

    *pFileType = m_FileType;

    //MPEGRs[
    pMpegInfo->version = dInfo.header.version;
    pMpegInfo->layer = dInfo.header.layer;
    pMpegInfo->crcDisable = (dInfo.header.error_prot) ? 0 : 1;
    pMpegInfo->mode = dInfo.header.mode;
    pMpegInfo->extension = dInfo.header.mode_ext;
    pMpegInfo->copyright = dInfo.header.copyright;
    pMpegInfo->original = dInfo.header.original;
    pMpegInfo->emphasis = dInfo.header.emphasis;
    pMpegInfo->channels = dInfo.channels;
    pMpegInfo->bitRate = dInfo.bitRate / 1000;
    pMpegInfo->fileSize = fSize;
    pMpegInfo->samplingRate = dInfo.frequency;
    padding = dInfo.header.padding;

    if (dInfo.frames == 0) {
        dInfo.frames = dSize / dInfo.minInputSize;
    }
    pMpegInfo->flames = dInfo.frames;
    int samples;
    int bytes = dInfo.frames * dInfo.outputSize;
    if ( bytes > ALLOW_DATA_SIZE ) {
        samples = (long)((double)bytes / (double)dInfo.bitsPerSample / (double)dInfo.channels * (double)8.0);
    } else {
        samples = (long)((double)bytes * (double)8.0 / (double)dInfo.bitsPerSample / (double)dInfo.channels);
    }

    pMpegInfo->totalSec = samples / dInfo.frequency;

    ::CloseHandle(hFileMpeg);
    return TRUE;
}


BOOL getFileInfo2(const char* pszName, const wchar_t* pszNameW, TAG_INFO* pTagInfo, MPEG_INFO* pMpegInfo, LIST_INFO* pListInfo)
{
    HANDLE          hFileMpeg;              //^OҏWt@C
    BOOL            result;
    int             fSync;                  //wb_̈ʒu
    DWORD           dSize;                  //MPEG Audiõf[^TCY
    BYTE            buf[1024];
    DWORD           bytesRead;
    MPEG_DECODE_INFO dInfo;
    DWORD           fSize;
    int             padding;

    //
    ::memset(pTagInfo, 0, sizeof(TAG_INFO));
    ::memset(pMpegInfo, 0, sizeof(MPEG_INFO));
    ::memset(pListInfo, 0, sizeof(LIST_INFO));

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE, OPEN_EXISTING, &hFileMpeg) == FALSE)
		return FALSE;

    fSync = 0;
    dSize = GetFileSize(hFileMpeg, NULL);
    fSize = dSize;

    SetFilePointer(hFileMpeg, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileMpeg, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }

    //RIFF`AID3 Tag v2`FbN
    if (memcmp(buf, "RIFF", 4) == 0) {
        ::readRIFF(hFileMpeg, pListInfo, &fSync, &dSize);
        if (RiffStatus != 0 ) {
            ::CloseHandle(hFileMpeg);
            _setError(ERR_UNKNOWN_FILE);
            return FALSE;
        } else {
            if (readID3v2(hFileMpeg, &fSync)) {
                dSize -= fSync;
            }
        }
    }
    else if (memcmp(buf, "ID3", 3) == 0) {
        if (readID3v2(hFileMpeg, &fSync)) {
            dSize -= fSync;
        }
    }
    //Tag f[^
    else if (buf[0] == 0x00) {
            fSync = 128;
    }
    else {
        fSync = 0;
    }
    //wb_擾
    ::SetFilePointer(hFileMpeg, fSync, NULL, FILE_BEGIN);
    result = ::ReadFile(hFileMpeg, buf, sizeof(buf), &bytesRead, NULL);
    if (!result || bytesRead == 0) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }

    //wb_T
    int sync;
    if (!::mp3FindSync(buf, bytesRead, &sync)) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_FRAME_HEADER_NOT_FOUND);
        return FALSE;
    }
    fSync += sync;

    //wb_ʒu擾
    ::SetFilePointer(hFileMpeg, fSync, NULL, FILE_BEGIN);
    result = ::ReadFile(hFileMpeg, buf, sizeof(buf), &bytesRead, NULL);
    if (!result || bytesRead == 0) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }

    //wb_擾
    if (!::mp3GetDecodeInfo(buf, bytesRead, &dInfo, 0)) {
        ::CloseHandle(hFileMpeg);
        _setError(ERR_FRAME_HEADER_READ);
        return FALSE;
    }

    //ID3 Tagv1`FbN
    if (readID3v1(hFileMpeg, pTagInfo)) {
        if (fSize == dSize) 
            dSize -= 128;
    }

    //LYRICS3݃`FbN
    DWORD lysize;
    if (chkLYRICS3(hFileMpeg, &lysize)) {
        dSize -= lysize;
    }

    //MPEGRs[
    pMpegInfo->version = dInfo.header.version;
    pMpegInfo->layer = dInfo.header.layer;
    pMpegInfo->crcDisable = (dInfo.header.error_prot) ? 0 : 1;
    pMpegInfo->mode = dInfo.header.mode;
    pMpegInfo->extension = dInfo.header.mode_ext;
    pMpegInfo->copyright = dInfo.header.copyright;
    pMpegInfo->original = dInfo.header.original;
    pMpegInfo->emphasis = dInfo.header.emphasis;
    pMpegInfo->channels = dInfo.channels;
    pMpegInfo->bitRate = dInfo.bitRate / 1000;
    pMpegInfo->fileSize = fSize;
    pMpegInfo->samplingRate = dInfo.frequency;
    padding = dInfo.header.padding;

    if (dInfo.frames == 0) {
        dInfo.frames = dSize / dInfo.minInputSize;
    }

    pMpegInfo->flames = dInfo.frames;
    int samples;
    int bytes = dInfo.frames * dInfo.outputSize;
    if ( bytes > ALLOW_DATA_SIZE ) {
        samples = (long)((double)bytes / (double)dInfo.bitsPerSample / (double)dInfo.channels * (double)8.0);
    } else {
        samples = (long)((double)bytes * (double)8.0 / (double)dInfo.bitsPerSample / (double)dInfo.channels);
    }

    pMpegInfo->totalSec = samples / dInfo.frequency;

    ::CloseHandle(hFileMpeg);
    return TRUE;

}

BOOL setTagInfo(const char* pszName, const wchar_t* pszNameW, TAG_INFO* pTagInfo,int tagSet, int tagAdd)
{
    HANDLE          hFileTag;               //^OҏWt@C
    ID3Tagv1        tag;
    ID3Tagv1        wtag;
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    BYTE            buf[1024];
    LIST_INFO       ListInfo;
    int             fSync;                  //wb_̈ʒu
    DWORD           dSize;                  //MPEG Audiõf[^TCY
    int             siFlag = 0;
    DWORD           target = 0;
    DWORD           fSize;
    DWORD           lSize;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, OPEN_EXISTING, &hFileTag) == FALSE)
		return FALSE;

    ::memcpy(wtag.tag, "TAG",  3);
    ::memcpy(wtag.albumName,  pTagInfo->szAlbumName,  30);
    ::memcpy(wtag.artistName,  pTagInfo->szArtistName,  30);
    ::memcpy(wtag.comment,  pTagInfo->szComment,  30);
    ::memcpy(wtag.trackName,  pTagInfo->szTrackName,  30);
    ::memcpy(wtag.year,  pTagInfo->szYear,  4);
    if ( pTagInfo->genre < 0 || pTagInfo->genre > 255 ) {
        pTagInfo->genre = 200;
    }
    wtag.genre = pTagInfo->genre;

    //RIFF``FbN
    SetFilePointer(hFileTag, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileTag, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileTag);
        return FALSE;
    }
    if (memcmp(buf, "RIFF", 4) == 0) {
        ::readRIFF(hFileTag, &ListInfo, &fSync, &dSize);
        if ( m_listChunkPoint > 0 ) {
            siFlag = 1;
        } else {
            siFlag = 2;
        }
    }

    //t@Č납128oCgǂݍ
    ::SetFilePointer(hFileTag, -(long)sizeof(tag), NULL, FILE_END);
    result = ::ReadFile(hFileTag, &tag, sizeof(tag), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(tag)) {
        ::CloseHandle(hFileTag);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }
    if (::memcmp(tag.tag, "TAG", 3) != 0 && tagAdd == 0 ) {
        //ID3Tag v1 ^Oǉ֎~
        ::CloseHandle(hFileTag);
        _setError(ERR_MP3_FILE_WRITE);
        return FALSE;
    } else if (::memcmp(tag.tag, "TAG", 3) != 0 && tagAdd == 1 ) {
        //ID3Tag v1 ^Oǉ
        if ( siFlag == 0 ) {
            //ʏMP3t@C
            //t@C̍Ōɏ
            ::SetFilePointer(hFileTag, 0, NULL, FILE_END);
            result = ::WriteFile(hFileTag, &wtag, sizeof(wtag), &bytesWrite, NULL);
            if (!result || bytesWrite != sizeof(wtag)) {
                ::CloseHandle(hFileTag);
                _setError(ERR_MP3_FILE_WRITE);
                return FALSE;
            }
        } else if ( siFlag == 1 ) {
            //SItB[h邪IID3tB[hȂ
            //IID3tB[hǉ
            ::memcpy(&buf[target], "IID3", 4);
            buf[target + 4] = 0x80;
            target += 8;
            ::memcpy(&buf[target], &wtag, 128);
            target += 128;
            SetFilePointer(hFileTag, 0, NULL, FILE_END);
            result = ::WriteFile(hFileTag, &buf, target, &bytesWrite, NULL);
            if (!result || bytesWrite != target) {
                ::CloseHandle(hFileTag);
                _setError(ERR_MP3_FILE_WRITE);
                return FALSE;
            }
            ::SetEndOfFile(hFileTag);
            fSize = ::GetFileSize(m_hFile, NULL) - 8;
            lSize = m_listChunkSize + target;

            //LIST`NTCYύX
            buf[0] = (BYTE)(fSize & 0x000000ff);
            buf[1] = (BYTE)((fSize & 0x0000ffff) >> 8);
            buf[2] = (BYTE)((fSize & 0x00ffffff) >> 16);
            buf[3] = (BYTE)((fSize & 0xffffffff) >> 24);
            SetFilePointer(hFileTag, m_listChunkPoint + 4, NULL, FILE_BEGIN);
            result = ::WriteFile(hFileTag, &buf, 4, &bytesWrite, NULL);
            if (!result || bytesWrite != 4) {
                ::CloseHandle(hFileTag);
                _setError(ERR_MP3_FILE_WRITE);
                return FALSE;
            }

            //DATA`NTCYύX
            buf[0] = (BYTE)(lSize & 0x000000ff);
            buf[1] = (BYTE)((lSize & 0x0000ffff) >> 8);
            buf[2] = (BYTE)((lSize & 0x00ffffff) >> 16);
            buf[3] = (BYTE)((lSize & 0xffffffff) >> 24);
            SetFilePointer(hFileTag,  4, NULL, FILE_BEGIN);
            result = ::WriteFile(hFileTag, &buf, 4, &bytesWrite, NULL);
            if (!result || bytesWrite != 4) {
                ::CloseHandle(hFileTag);
                _setError(ERR_MP3_FILE_WRITE);
                return FALSE;
            }

        } else if ( siFlag == 2 ) {
            //SItB[hȂ
            //LIST񏉊
            ::memset(&ListInfo, ' ', sizeof(LIST_INFO));

            //LIST`Nǉ
            ::CloseHandle(hFileTag);
            if (!setListInfo(pszName, pszNameW, &ListInfo)) {
                return FALSE;
            }

            //t@CI[v
            hFileTag = ::CreateFile(pszName, GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ,
                    NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
            if (hFileTag == INVALID_HANDLE_VALUE) {
                _setError(ERR_MP3_FILE_OPEN);
                return FALSE;
            }

            //t@Č납128oCgɏ
            ::SetFilePointer(hFileTag, -(long)sizeof(wtag), NULL, FILE_END);
            result = ::WriteFile(hFileTag, &wtag, sizeof(wtag), &bytesWrite, NULL);
            if (!result || bytesWrite != sizeof(wtag)) {
                ::CloseHandle(hFileTag);
                _setError(ERR_MP3_FILE_WRITE);
                return FALSE;
            }
        }
    } else {
        //ID3Tag v1 L
        //t@Č납128oCgɏ
        ::SetFilePointer(hFileTag, -(long)sizeof(wtag), NULL, FILE_END);
        result = ::WriteFile(hFileTag, &wtag, sizeof(wtag), &bytesWrite, NULL);
        if (!result || bytesWrite != sizeof(wtag)) {
            ::CloseHandle(hFileTag);
            _setError(ERR_MP3_FILE_WRITE);
            return FALSE;
        }
    }

    if ( tagSet == 1 ) {
        //^OZbg
        ::strcpyID3v1((char*)tagInfo.szTrackName, pTagInfo->szTrackName);
        ::strcpyID3v1((char*)tagInfo.szArtistName, pTagInfo->szArtistName);
        ::strcpyID3v1((char*)tagInfo.szAlbumName, pTagInfo->szAlbumName);
        ::memcpy(tagInfo.szYear, pTagInfo->szYear, 5);
        ::strcpyID3v1((char*)tagInfo.szComment, pTagInfo->szComment);
        tagInfo.genre = pTagInfo->genre;
    }

    ::CloseHandle(hFileTag);
    return TRUE;
}

BOOL setTagInfoEX(const char* pszName, const wchar_t* pszNameW, TAG_INFO_11* pTagInfo,int tagSet, int tagAdd)
{
    HANDLE          hFileTag;               //^OҏWt@C
    ID3Tagv1        tag;
    ID3Tagv1        wtag;
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    BYTE            buf[1024];
    LIST_INFO       ListInfo;
    int             fSync;                  //wb_̈ʒu
    DWORD           dSize;                  //MPEG Audiõf[^TCY
    int             siFlag = 0;
    DWORD           target = 0;
    DWORD           fSize;
    DWORD           lSize;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ, OPEN_EXISTING, &hFileTag) == FALSE)
		return FALSE;

	::memcpy(wtag.tag, "TAG",  3);
    ::memcpy(wtag.albumName,  pTagInfo->szAlbumName,  30);
    ::memcpy(wtag.artistName,  pTagInfo->szArtistName,  30);

    if ( pTagInfo->trackNo >= 0 && pTagInfo->trackNo <= 255 ) {
        pTagInfo->szComment[28] = 0;
        pTagInfo->szComment[29] = pTagInfo->trackNo;
    }

    ::memcpy(wtag.comment,  pTagInfo->szComment,  30);
    ::memcpy(wtag.trackName,  pTagInfo->szTrackName,  30);
    ::memcpy(wtag.year,  pTagInfo->szYear,  4);
    if ( pTagInfo->genre < 0 || pTagInfo->genre > 255 ) {
        pTagInfo->genre = 200;
    }
    wtag.genre = pTagInfo->genre;

    //RIFF``FbN
    SetFilePointer(hFileTag, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileTag, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileTag);
        return FALSE;
    }
    if (memcmp(buf, "RIFF", 4) == 0) {
        ::readRIFF(hFileTag, &ListInfo, &fSync, &dSize);
        if ( m_listChunkPoint > 0 ) {
            siFlag = 1;
        } else {
            siFlag = 2;
        }
    }

    //t@Č납128oCgǂݍ
    ::SetFilePointer(hFileTag, -(long)sizeof(tag), NULL, FILE_END);
    result = ::ReadFile(hFileTag, &tag, sizeof(tag), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(tag)) {
        ::CloseHandle(hFileTag);
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }
    if (::memcmp(tag.tag, "TAG", 3) != 0 && tagAdd == 0 ) {
        //ID3Tag v1 ^Oǉ֎~
        ::CloseHandle(hFileTag);
        _setError(ERR_MP3_FILE_WRITE);
        return FALSE;
    } else if (::memcmp(tag.tag, "TAG", 3) != 0 && tagAdd == 1 ) {
        //ID3Tag v1 ^Oǉ
        if ( siFlag == 0 ) {
            //ʏMP3t@C
            //t@C̍Ōɏ
            ::SetFilePointer(hFileTag, 0, NULL, FILE_END);
            result = ::WriteFile(hFileTag, &wtag, sizeof(wtag), &bytesWrite, NULL);
            if (!result || bytesWrite != sizeof(wtag)) {
                ::CloseHandle(hFileTag);
                _setError(ERR_MP3_FILE_WRITE);
                return FALSE;
            }
        } else if ( siFlag == 1 ) {
            //SItB[h邪IID3tB[hȂ
            //IID3tB[hǉ
            ::memcpy(&buf[target], "IID3", 4);
            buf[target + 4] = 0x80;
            target += 8;
            ::memcpy(&buf[target], &wtag, 128);
            target += 128;
            SetFilePointer(hFileTag, 0, NULL, FILE_END);
            result = ::WriteFile(hFileTag, &buf, target, &bytesWrite, NULL);
            if (!result || bytesWrite != target) {
                ::CloseHandle(hFileTag);
                _setError(ERR_MP3_FILE_WRITE);
                return FALSE;
            }
            ::SetEndOfFile(hFileTag);
            fSize = ::GetFileSize(m_hFile, NULL) - 8;
            lSize = m_listChunkSize + target;

            //LIST`NTCYύX
            buf[0] = (BYTE)(fSize & 0x000000ff);
            buf[1] = (BYTE)((fSize & 0x0000ffff) >> 8);
            buf[2] = (BYTE)((fSize & 0x00ffffff) >> 16);
            buf[3] = (BYTE)((fSize & 0xffffffff) >> 24);
            SetFilePointer(hFileTag, m_listChunkPoint + 4, NULL, FILE_BEGIN);
            result = ::WriteFile(hFileTag, &buf, 4, &bytesWrite, NULL);
            if (!result || bytesWrite != 4) {
                ::CloseHandle(hFileTag);
                _setError(ERR_MP3_FILE_WRITE);
                return FALSE;
            }

            //DATA`NTCYύX
            buf[0] = (BYTE)(lSize & 0x000000ff);
            buf[1] = (BYTE)((lSize & 0x0000ffff) >> 8);
            buf[2] = (BYTE)((lSize & 0x00ffffff) >> 16);
            buf[3] = (BYTE)((lSize & 0xffffffff) >> 24);
            SetFilePointer(hFileTag,  4, NULL, FILE_BEGIN);
            result = ::WriteFile(hFileTag, &buf, 4, &bytesWrite, NULL);
            if (!result || bytesWrite != 4) {
                ::CloseHandle(hFileTag);
                _setError(ERR_MP3_FILE_WRITE);
                return FALSE;
            }

        } else if ( siFlag == 2 ) {
            //SItB[hȂ
            //LIST񏉊
            ::memset(&ListInfo, ' ', sizeof(LIST_INFO));

            //LIST`Nǉ
            ::CloseHandle(hFileTag);
            if (!setListInfo(pszName, pszNameW, &ListInfo)) {
                return FALSE;
            }

            //t@CI[v
            hFileTag = ::CreateFile(pszName, GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ,
                    NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
            if (hFileTag == INVALID_HANDLE_VALUE) {
                _setError(ERR_MP3_FILE_OPEN);
                return FALSE;
            }

            //t@Č납128oCgɏ
            ::SetFilePointer(hFileTag, -(long)sizeof(wtag), NULL, FILE_END);
            result = ::WriteFile(hFileTag, &wtag, sizeof(wtag), &bytesWrite, NULL);
            if (!result || bytesWrite != sizeof(wtag)) {
                ::CloseHandle(hFileTag);
                _setError(ERR_MP3_FILE_WRITE);
                return FALSE;
            }
        }
    } else {
        //ID3Tag v1 L
        //t@Č납128oCgɏ
        ::SetFilePointer(hFileTag, -(long)sizeof(wtag), NULL, FILE_END);
        result = ::WriteFile(hFileTag, &wtag, sizeof(wtag), &bytesWrite, NULL);
        if (!result || bytesWrite != sizeof(wtag)) {
            ::CloseHandle(hFileTag);
            _setError(ERR_MP3_FILE_WRITE);
            return FALSE;
        }
    }

    if ( tagSet == 1 ) {
        //^OZbg
        ::strcpyID3v1((char*)tagInfo.szTrackName, pTagInfo->szTrackName);
        ::strcpyID3v1((char*)tagInfo.szArtistName, pTagInfo->szArtistName);
        ::strcpyID3v1((char*)tagInfo.szAlbumName, pTagInfo->szAlbumName);
        ::memcpy(tagInfo.szYear, pTagInfo->szYear, 5);
        ::strcpyID3v1((char*)tagInfo.szComment, pTagInfo->szComment);
        tagInfo.genre = pTagInfo->genre;
    }

    ::CloseHandle(hFileTag);
    return TRUE;
}

BOOL input_GetGenre(TAG_INFO* pTagInfo)
{
    int gno;

    gno = pTagInfo->genre;

    ::memset(pTagInfo, 0, sizeof(TAG_INFO));

    pTagInfo->genre = gno;
    if ( gno < 0 || gno > 255 ) {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }

    ::memcpy(pTagInfo->szGanreName,  genre_tbl[gno],  strlen(genre_tbl[gno]));

    return TRUE;
}

BOOL input_SetVbmp3Option(VBMP3_OPTION* pVbmp3Option)
{

    if (m_state != STATE_STOP) {
        //~̂ݏ
        _setError(ERR_NOT_STATE_STOP);
        return FALSE;
    }

    //IvVl`FbN
    if (pVbmp3Option->inputBlock <= 0) {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }

    if (pVbmp3Option->outputBlock <= 0) {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }

    if (pVbmp3Option->inputSleep < 0) {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }

    if (pVbmp3Option->outputSleep < 0) {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }

    m_inputBlock    = pVbmp3Option->inputBlock;
    m_outputBlock_in    = pVbmp3Option->outputBlock;
    m_outputBlock_in_tmp = m_outputBlock_in;
    m_inputSleep    = pVbmp3Option->inputSleep;
    //m_outputSleep = pVbmp3Option->outputSleep;

    output_SetVbmp3Option(pVbmp3Option);

    return TRUE;

}

void input_GetVbmp3Option(VBMP3_OPTION* pVbmp3Option){
    pVbmp3Option->inputBlock = m_inputBlock;
    pVbmp3Option->outputBlock = m_outputBlock_in;
    pVbmp3Option->inputSleep = m_inputSleep;
    //pVbmp3Option->outputSleep = m_outputSleep;
    output_GetVbmp3Option(pVbmp3Option);
    return;
}

BOOL input_SetDecodeOption(DEC_OPTION* pDecOption)
{
    double curRed;
    double Red;

    if (m_state != STATE_STOP) {
        //~̂ݏ
        _setError(ERR_NOT_STATE_STOP);
        return FALSE;
    }

    //IvVl`FbN
    if (pDecOption->reduction != 0 &&
        pDecOption->reduction != 1 &&
        pDecOption->reduction != 2) {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }

    if (pDecOption->convert != 0 &&
        pDecOption->convert != 1) {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }

    if (pDecOption->freqLimit <= 0) {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }

    decodeOption.reduction = pDecOption->reduction;
    decodeOption.convert = pDecOption->convert;
    decodeOption.freqLimit = pDecOption->freqLimit;

    if ( pDecOption->reduction == 0 ) {
        curRed = 4;
    } else if ( pDecOption->reduction == 1 ) {
        curRed = 2;
    } else {
        curRed = 1;
    }

    if ( m_reduction == 0 ) {
        Red = 4;
    } else if ( m_reduction == 1 ) {
        Red = 2;
    } else {
        Red = 1;
    }
    m_playSpeed = Red / curRed;

    return ::mp3SetDecodeOption(&decodeOption);
}

void input_GetDecodeOption(DEC_OPTION* pDecOption)
{
    mp3GetDecodeOption(&decodeOption);

    pDecOption->reduction = decodeOption.reduction;
    pDecOption->convert = decodeOption.convert;
    pDecOption->freqLimit = decodeOption.freqLimit;

    return;
}

int getTime()
{
    if (m_decInfo.frequency == 0 || m_decInfo.frequency == 0) return 0;

    if ( decFlag == 1 ){
        return input_sampleToSec(m_firstSample + input_bytesToSample(decodePos));
    }

    double playMs;
    int playSamples;
    playSamples = output_getCurrentSample();
    //{tΉ
    playMs = (((double)playSamples + (double)m_firstSample) * (double)1000 / (double)m_decInfo.frequency) / (double)1000.0;
    playMs = playMs * m_playSpeed;
    m_playSec = (int)playMs;
    if (m_timeFlag && m_playSec > m_totalSec) m_playSec = m_totalSec;
    return m_playSec;
}

BOOL getTagInfo(TAG_INFO* pTagInfo)
{
    //t@CI[v
    if (m_hFile == INVALID_HANDLE_VALUE) {
        _setError(ERR_MP3_FILE_NOT_OPEN);
        return FALSE;
    }

    ::memset(pTagInfo, 0, sizeof(TAG_INFO));

    //^OZbg
    ::strcpyID3v1(pTagInfo->szTrackName,  (char*)tagInfo.szTrackName);
    ::strcpyID3v1(pTagInfo->szArtistName, (char*)tagInfo.szArtistName);
    ::strcpyID3v1(pTagInfo->szAlbumName, (char*)tagInfo.szAlbumName);
    ::memcpy(pTagInfo->szYear,  tagInfo.szYear,  5);
    ::strcpyID3v1(pTagInfo->szComment, (char*)tagInfo.szComment);
    pTagInfo->genre = tagInfo.genre;
    ::memcpy(pTagInfo->szGanreName,  genre_tbl[tagInfo.genre],  strlen(genre_tbl[tagInfo.genre]));
    
    return TRUE;
}

BOOL getMpegInfo(MPEG_INFO* pMpegInfo)
{
    int             padding;

    //t@CI[v
    if (m_hFile == INVALID_HANDLE_VALUE) {
        _setError(ERR_MP3_FILE_NOT_OPEN);
        return FALSE;
    }

    pMpegInfo->version = m_decInfo.header.version;
    pMpegInfo->layer = m_decInfo.header.layer;
    pMpegInfo->crcDisable = (m_decInfo.header.error_prot) ? 0 : 1;
    pMpegInfo->mode = m_decInfo.header.mode;
    pMpegInfo->extension = m_decInfo.header.mode_ext;
    pMpegInfo->copyright = m_decInfo.header.copyright;
    pMpegInfo->original = m_decInfo.header.original;
    pMpegInfo->emphasis = m_decInfo.header.emphasis;
    pMpegInfo->channels = m_decInfo.channels;
    pMpegInfo->bitRate = m_decInfo.bitRate / 1000;
    pMpegInfo->fileSize = fileSize;
    pMpegInfo->samplingRate = m_decInfo.frequency;
    padding = m_decInfo.header.padding;

    if (m_decInfo.frames == 0) {
        m_decInfo.frames = dataSize / m_decInfo.minInputSize;
    }
    pMpegInfo->flames = m_decInfo.frames;
    int samples = input_bytesToSample(m_decInfo.frames * m_decInfo.outputSize);
    pMpegInfo->totalSec = input_sampleToSec(samples);

    return TRUE;
}


BOOL initDec()
{
    //fR[hGW̏
    ::mp3DecodeInit();

    //OS`FbN
    OSVERSIONINFO   obsInfo;
    obsInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
    ::GetVersionEx(&obsInfo);
    if ( obsInfo.dwPlatformId == VER_PLATFORM_WIN32_NT ) {
        m_platForm = 1;
    } else {
        m_platForm = 0;
    }
    
    m_pProc         = _NullProc;
    m_hFile         = INVALID_HANDLE_VALUE;

    m_hThread       = NULL;
    m_threadID      = 0;
    m_state         = STATE_STOP;
    m_inputBuf      = NULL;
    m_outputBuf_in  = NULL;

    callBackFlag    = FALSE;

    initOutput();
    initEnc();
    mp3SetEqualizer(NULL);
    return TRUE;
}

BOOL freeDec()
{
    //::Sleep(50);//폜 by Kobarinî߂ɕKvHj
    callBackFlag = FALSE;
    m_pProc = _NullProc;

    input_stopCallBackTimer();

    if ( m_playFlag ) input_stop();
    input_close();

    if ( m_playFlag ) output_stop();
    if ( m_playFlag ) output_close();
    freeOutput();
    freeEnc();

    ::free(m_inputBuf);
    ::free(m_outputBuf_in);
    //::Sleep(50);//폜 by Kobarinî߂ɕKvHj
    return TRUE;
}

BOOL input_open(const char* pszName, const wchar_t* pszNameW, InputInfo* pInfo)
{
    //::Sleep(100);//폜 by Kobarinî߂ɕKvHj

	if (pszName != nullptr)
	{
		//t@CI[v`FbN
		if (!isVar(pszName)) {
			//͂Ȃꍇ͏I
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}
	}
	else
	{
		//t@CI[v`FbN
		if (!isVarW(pszNameW)) {
			//͂Ȃꍇ͏I
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}
	}

    //t@CN[Y
    ::input_close();
    m_waveFlag = FALSE;

    //ݒ
    regRead();

	if (pszName != nullptr)
	{
		//t@CI[v
		m_hFile = ::CreateFile(pszName, GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (m_hFile == INVALID_HANDLE_VALUE) {
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}
		::strcpy(m_openFileName, pszName);
		m_openFileNameW[0] = 0;
	}
	else
	{
		//t@CI[v
		m_hFile = ::CreateFileW(pszNameW, GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (m_hFile == INVALID_HANDLE_VALUE) {
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}
		::wcscpy(m_openFileNameW, pszNameW);
		m_openFileName[0] = 0;
	}
		


    //^Oǂ
    m_firstSync = 0;
    dataSize = ::GetFileSize(m_hFile, NULL);
    fileSize = dataSize;

    ::memset(pInfo, 0, sizeof(InputInfo));

    ::memset(tagInfo.szAlbumName, 0, sizeof(tagInfo.szAlbumName));
    ::memset(tagInfo.szArtistName, 0, sizeof(tagInfo.szArtistName));
    ::memset(tagInfo.szTrackName, 0, sizeof(tagInfo.szTrackName));
    ::memset(tagInfo.szYear, 0, sizeof(tagInfo.szYear));
    tagInfo.genre = 0;
    ::memset(tagInfo.szGanreName, 0, sizeof(tagInfo.szGanreName));
    ::memset(tagInfo.szComment, 0, sizeof(tagInfo.szComment));

    ::memset(listInfo.IART, 0, sizeof(listInfo.IART));
    ::memset(listInfo.ICMS, 0, sizeof(listInfo.ICMS));
    ::memset(listInfo.ICMT, 0, sizeof(listInfo.ICMT));
    ::memset(listInfo.ICOP, 0, sizeof(listInfo.ICOP));
    ::memset(listInfo.ICRD, 0, sizeof(listInfo.ICRD));
    ::memset(listInfo.IENG, 0, sizeof(listInfo.IENG));
    ::memset(listInfo.IGNR, 0, sizeof(listInfo.IGNR));
    ::memset(listInfo.IKEY, 0, sizeof(listInfo.IKEY));
    ::memset(listInfo.ILYC, 0, sizeof(listInfo.ILYC));
    ::memset(listInfo.INAM, 0, sizeof(listInfo.INAM));
    ::memset(listInfo.IPRD, 0, sizeof(listInfo.IPRD));
    ::memset(listInfo.ISFT, 0, sizeof(listInfo.ISFT));
    ::memset(listInfo.ISRC, 0, sizeof(listInfo.ISRC));
    ::memset(listInfo.ITCH, 0, sizeof(listInfo.ITCH));

    if (readID3v1(m_hFile, &tagInfo)) {
        dataSize -= 128;
        ::strcpyID3v1(pInfo->szTrackName,  (char*)tagInfo.szTrackName);
        ::strcpyID3v1(pInfo->szArtistName, (char*)tagInfo.szArtistName);
    }

    //LYRICS3݃`FbN
    DWORD lysize;
    if (chkLYRICS3(m_hFile, &lysize)) {
        dataSize -= lysize;
        if ( m_Lyrics3UseFlag ) {
            input_readLyrics3(m_hFile);
        }
    }

/*
    if (readID3v2(m_hFile, &m_firstSync)) {
        dataSize -= m_firstSync;
    } else {
        ::readRIFF(m_hFile, &listInfo, &m_firstSync, &dataSize);
        if (RiffStatus == -1) {
            //MPEG,WAVEȊO
            ::input_close();
            return FALSE;
        } else if (RiffStatus == 1) {
            //WAVEt@C̏ꍇ
            m_decInfo.bitsPerSample = m_waveFormat.wBitsPerSample;
            m_decInfo.channels = m_waveFormat.nChannels;
            m_decInfo.dataSize = dataSize;
            m_decInfo.frequency = m_waveFormat.nSamplesPerSec;
            m_decInfo.bitRate = m_waveFormat.wBitsPerSample;        //̒l
            m_bitRate = m_decInfo.bitRate;                          //̒l
            m_playSpeed = 1;
            m_firstSample = 0;
            m_decInfo.outputSize = DEFAULT_BUFFER_SIZE;
            m_decInfo.frames = (int)(m_decInfo.dataSize / m_decInfo.outputSize);

            ::strcpy(pInfo->szTrackName,  "WAVE_FORMAT_PCM");
            ::strcpy(pInfo->szArtistName, "WAVE_FORMAT_PCM");
            pInfo->bitRate = m_waveFormat.wBitsPerSample;
            pInfo->channels = m_waveFormat.nChannels;
            pInfo->samplingRate = m_waveFormat.nSamplesPerSec;
            pInfo->totalSec = input_sampleToSec(input_bytesToSample(dataSize));
            m_totalSec = pInfo->totalSec;
            m_winampTotalSec = pInfo->totalSec;
            m_waveFlag= TRUE;
            return TRUE;
        }
    }
*/
    ::readRIFF(m_hFile, &listInfo, &m_firstSync, &dataSize);
    if (RiffStatus == -1) {
        //MPEG,WAVEȊO
        ::input_close();
        _setError(ERR_UNKNOWN_FILE);
        return FALSE;
    } else if (RiffStatus == 1) {
        //WAVEt@C̏ꍇ
        m_decInfo.bitsPerSample = m_waveFormat.wBitsPerSample;
        m_decInfo.channels = m_waveFormat.nChannels;
        m_decInfo.dataSize = dataSize;
        m_decInfo.frequency = m_waveFormat.nSamplesPerSec;
        m_decInfo.bitRate = m_waveFormat.wBitsPerSample;        //̒l
        m_bitRate = m_decInfo.bitRate;                          //̒l
        m_playSpeed = 1;
        m_firstSample = 0;
        m_decInfo.outputSize = DEFAULT_BUFFER_SIZE;
        if ( m_decInfo.bitRate == 8 ) m_decInfo.outputSize = m_decInfo.outputSize / 4;
        m_decInfo.frames = (int)(m_decInfo.dataSize / m_decInfo.outputSize);

        ::strcpy(pInfo->szTrackName,  "WAVE_FORMAT_PCM");
        ::strcpy(pInfo->szArtistName, "WAVE_FORMAT_PCM");
        pInfo->bitRate = m_waveFormat.wBitsPerSample;
        pInfo->channels = m_waveFormat.nChannels;
        pInfo->samplingRate = m_waveFormat.nSamplesPerSec;
        int samples = input_bytesToSample(dataSize);
        m_totalMs = input_sampleToMs(samples);
        pInfo->totalSec = m_totalMs / 1000;
        /* pInfo->totalSec = input_sampleToSec(samples); */
        m_totalSec = pInfo->totalSec;
        //winamp΍
        m_winampTotalMs = getWinampMs(pInfo->samplingRate, samples);
        m_winampTotalSec = m_winampTotalMs / 1000;
        //m_winampTotalSec = pInfo->totalSec;
        m_waveFlag= TRUE;
        return TRUE;
    }

    if (readID3v2(m_hFile, &m_firstSync)) {
        dataSize -= m_firstSync;
    }


    //wb_T
    BOOL    result;
    DWORD   bytesRead;
    BYTE    buf[10240];
    int     sync;

    //Tag f[^
    if (m_firstSync == 0) {
        ::SetFilePointer(m_hFile, m_firstSync, NULL, FILE_BEGIN);
        result = ::ReadFile(m_hFile, buf, 4, &bytesRead, NULL);
        if (!result || bytesRead != 4) {
            ::input_close();
            _setError(ERR_MP3_FILE_READ);
            return FALSE;
        }
        if (buf[0] == 0x00){
            m_firstSync = 128;
        }
    }

    //wb_ʒu擾
    ::SetFilePointer(m_hFile, m_firstSync, NULL, FILE_BEGIN);
    result = ::ReadFile(m_hFile, buf, sizeof(buf), &bytesRead, NULL);
    if (!result || bytesRead == 0) {
        _setError(ERR_MP3_FILE_READ);
        ::input_close();
        return FALSE;
    }

    //wb_ʒuړ
    if (!::mp3FindSync(buf, bytesRead, &sync)) {
        ::input_close();
        _setError(ERR_FRAME_HEADER_NOT_FOUND);
        return FALSE;
    }

    m_firstSync += sync;

    //wb_ʒu擾
    ::SetFilePointer(m_hFile, m_firstSync, NULL, FILE_BEGIN);
    result = ::ReadFile(m_hFile, buf, sizeof(buf), &bytesRead, NULL);
    if (!result || bytesRead == 0) {
        ::input_close();
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }

    //fR[h擾
    if (!::mp3GetDecodeInfo(buf, bytesRead, &m_decInfo, 1)) {
    //if (!::mp3GetDecodeInfo(&buf[m_firstSync], bytesRead - sync, &m_decInfo)) {
        ::input_close();
        _setError(ERR_FRAME_HEADER_READ);
        return FALSE;
    }

    m_firstSample = 0;

    pInfo->channels = m_decInfo.channels;
    pInfo->bitRate = m_decInfo.bitRate / 1000;
    m_bitRate = pInfo->bitRate;
    if ( m_bitRate == 0 ) {
        m_openFileVBR = TRUE;
    } else {
        m_openFileVBR = FALSE;
    }
    pInfo->samplingRate = m_decInfo.frequency;

    if (m_frameReadFlag == 1 ) {
        m_decInfo.frames = input_getTotalFrames(pszName, pszNameW);
    } else {
        if (m_decInfo.frames == 0) {
            m_decInfo.frames = dataSize / m_decInfo.minInputSize;
        }
    }
    int samples = input_bytesToSample(m_decInfo.frames * m_decInfo.outputSize);
    m_totalMs = input_sampleToMs(samples);
    pInfo->totalSec = m_totalMs / 1000;
    /* pInfo->totalSec = input_sampleToSec(samples); */
    m_totalSec = pInfo->totalSec;
    
    //winamp΍
    m_winampTotalMs = getWinampMs(pInfo->samplingRate, samples);
    /*
    m_winampTotalSec = m_winampTotalMs / 1000;
    */
    double msPerFrame = ms_table[m_decInfo.header.layer-1][m_decInfo.header.fr_index];
    m_winampTotalSec = (m_decInfo.frames * (int)(msPerFrame) / 1000);

    //ĐxύXΉ
    mp3GetDecodeOption(&decodeOption);
    m_reduction = decodeOption.reduction;
    m_playSpeed = 1;

	return TRUE;
}

long getWinampMs(long samplingRate, long samples)
{
    double UnitSamples = (double)1152.0;                                        //PʃTv
    double TPFn = (double)UnitSamples * (double)1000.0 / (double)samplingRate;  //Time Per Frame
    long TPFw = (long)TPFn;                                                     //Time Per Frame(Winamp)
    double TimeRatio = (double)((double)TPFw / (double)TPFn);                   //Ԕ
    return (long)((double)TimeRatio * (double)input_sampleToMs(samples));       //Winamp
}

long getNomalMs(long samplingRate, long samples)
{
    double UnitSamples = (double)1152.0;                                        //PʃTv
    double TPFn = (double)UnitSamples * (double)1000.0 / (double)samplingRate;  //Time Per Frame
    long TPFw = (long)TPFn;                                                     //Time Per Frame(Winamp)
    double TimeRatio = (double)((double)TPFn / (double)TPFw);                   //Ԕ
    return (long)((double)TimeRatio * (double)input_sampleToMs(samples));       //Winamp
}

long getFreeRateMs(double TimeRatio, long samples)
{
    return (long)((double)TimeRatio * (double)input_sampleToMs(samples));       //Winamp
}

BOOL input_close()
{
    //::Sleep(100);//폜 by Kobarinî߂ɕKvHj
    if (m_hFile == INVALID_HANDLE_VALUE) {
        _setError(ERR_MP3_FILE_NOT_OPEN);
        return FALSE;
    }

    if (m_state != STATE_STOP) {
        input_stop();
    }

    ::CloseHandle(m_hFile);
    m_hFile = INVALID_HANDLE_VALUE;

    return TRUE;
}

int input_read(void* pData, int length)
{
    int bytesRead;
    if (!::ReadFile(m_hFile, pData, length, (DWORD*)&bytesRead, NULL)) {
        return 0;//exception
    }
    return bytesRead;
}

BOOL input_setCallback(INPUT_MSG_PROC pProc)
{
    if (m_state != STATE_STOP) {
        //~ȊO͐ݒs
        _setError(ERR_NOT_STATE_STOP);
        return FALSE;
    }
    m_pProc = pProc;
    callBackFlag = TRUE;
    return TRUE;
}

BOOL input_startCallback()
{
    callBackFlag = TRUE;
    return TRUE;
}

BOOL input_stopCallback()
{
    callBackFlag = FALSE;
    return TRUE;
}

int input_getState()
{
    if ( m_seekFlag == TRUE || m_reloadFlag == TRUE ) {
        return m_stateWork;
    } else {
        return m_state;
    }
}

BOOL input_play()
{
    /*Sleep(100);
    if (m_state != STATE_STOP || m_hThread != NULL ) {
        //~ȊO͎ss
        _setError(ERR_NOT_STATE_STOP);
        return FALSE;
    }*/
    //C by Kobarin
    if(m_state != STATE_STOP){
        //~ȊO͎ss
        _setError(ERR_NOT_STATE_STOP);
        return FALSE;
    }
    else{
        input_stop();//Xbh̊J
    }
    //܂
    decFlag = 0;
    m_fadeOutFlag = false;
    m_playFlag = TRUE;
    m_readWaveDataSize = 0;

    //fR[h
    //decodePos     = 0;        //Ԏ擾p

    ::SetFilePointer(m_hFile, m_firstSync, NULL, FILE_BEGIN);
    if ( m_waveFlag == FALSE ){
        m_inputSize     = m_decInfo.maxInputSize * m_inputBlock;
        m_inputBuf      = (byte*)::realloc(m_inputBuf, m_inputSize);
        m_inputPtr      = m_inputBuf;
        m_inputBytes    = input_read(m_inputPtr, m_inputSize);
        if (m_inputBytes == 0) {
            _setError(ERR_INVALID_VALUE);
            return FALSE;
        }
    }

    //o̓foCXJ
    OutputInfo  outputInfo;
    outputInfo.channels         = m_decInfo.channels;
    outputInfo.bitsPerSample    = m_decInfo.bitsPerSample;
    outputInfo.frequency        = m_decInfo.frequency;

    if ( m_waveFlag == FALSE ){
        outputInfo.bufferSize       = m_decInfo.outputSize * m_outputBlock_in;
    } else {
        m_outWaveBufSize = m_decInfo.outputSize * m_outputBlock_in_wav;
        m_inWaveBufSize = m_decInfo.outputSize;
        outputInfo.bufferSize       = m_outWaveBufSize;
    }

    if (!output_open(NULL, outputInfo)) {
        _setError(ERR_OPEN_OUT_DEVICE);
        return FALSE;
    }

    //m_outputBuf_in= (BYTE*)::realloc(m_outputBuf_in, m_decInfo.outputSize * m_outputBlock_in);
    m_outputBuf_in= (BYTE*)::realloc(m_outputBuf_in, outputInfo.bufferSize);

    if ( m_waveFlag == FALSE ){
        if (!::mp3DecodeStart(m_inputPtr, m_inputBytes)) {
            output_close();
            _setError(ERR_DECODE);
            return FALSE;
        }
        m_inputPtr += m_decInfo.skipSize;
        m_inputBytes -= m_decInfo.skipSize;
    }

    //fR[hpXbhJn
    /*
    m_hThread = (HANDLE)::_beginthreadex(NULL, 0, 
            (THREADFUNC)input_ThreadProc, NULL, 0, (unsigned int*)&m_threadID);
    if (m_hThread == NULL) {
        output_close();
        _setError(ERR_DECODE_THREAD);
        return FALSE;
    }
    //ԃ`FbN
    int Cnt;
    for ( Cnt = 0; Cnt < 100; Cnt++){
        ::Sleep(10);
        if ( m_state == STATE_PLAY ) {
            break;
        } else if ( Cnt == 99 ){
            _setError(ERR_PLAY);
            return FALSE;
        }
    }
    */
    //C by Kobarin
    HANDLE hStartEvent = ::CreateEvent(NULL, FALSE, FALSE, NULL);
    m_hThread = (HANDLE)::_beginthreadex(NULL, 0, 
            input_ThreadProc, hStartEvent, 0, (unsigned int*)&m_threadID);
    if (m_hThread == NULL) {
        CloseHandle(hStartEvent);
        output_close();
        _setError(ERR_DECODE_THREAD);
        return FALSE;
    }
    WaitForSingleObject(hStartEvent, INFINITE);
    CloseHandle(hStartEvent);
    //܂
    return TRUE;
}

BOOL input_decodeWave(const char* pszWaveName, const wchar_t* pszWaveNameW)
{
    BYTE            buf[1024];
    BOOL            result;
    DWORD           bytesWrite;

    if (m_state != STATE_STOP) {
        //~ȊO͎ss
        _setError(ERR_NOT_STATE_STOP);
        return FALSE;
    }

    decFlag = 1;

	if (pszWaveName != nullptr)
	{
		//t@CI[v
		m_hWaveFile = ::CreateFile(pszWaveName, GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ,
			NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if (m_hWaveFile == INVALID_HANDLE_VALUE) {
			_setError(ERR_WAV_FILE_OPEN);
			return FALSE;
		}
	}
	else
	{
		//t@CI[v
		m_hWaveFile = ::CreateFileW(pszWaveNameW, GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ,
			NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if (m_hWaveFile == INVALID_HANDLE_VALUE) {
			_setError(ERR_WAV_FILE_OPEN);
			return FALSE;
		}
	}

    SetFilePointer(m_hWaveFile, 0, NULL, FILE_BEGIN);
    ::strcpy((char*)buf,"RIFF");
    result = ::WriteFile(m_hWaveFile, &buf, 4, &bytesWrite, NULL);
    ::memset(&buf, 0, 4);
    result = ::WriteFile(m_hWaveFile, &buf, 4, &bytesWrite, NULL);
    ::strcpy((char*)buf,"WAVEfmt ");
    result = ::WriteFile(m_hWaveFile, &buf, 8, &bytesWrite, NULL);

    //`NTCY
    buf[0] = 0x10;
    buf[1] = 0x00;
    buf[2] = 0x00;
    buf[3] = 0x00;
    result = ::WriteFile(m_hWaveFile, &buf, 4, &bytesWrite, NULL);

    //tH[}bghc
    buf[0] = 0x01;
    buf[1] = 0x00;
    result = ::WriteFile(m_hWaveFile, &buf, 2, &bytesWrite, NULL);

    //`l
    buf[0] = m_decInfo.channels;
    buf[1] = 0x00;
    result = ::WriteFile(m_hWaveFile, &buf, 2, &bytesWrite, NULL);

    //TvO[g
    DWORD nSamplesPerSec = m_decInfo.frequency;
    result = ::WriteFile(m_hWaveFile, &nSamplesPerSec, 4, &bytesWrite, NULL);

    //][g
    WORD  nBlockAlign = (m_decInfo.channels * m_decInfo.bitsPerSample / 8);
    DWORD nAvgBytesPerSec = (nBlockAlign * m_decInfo.frequency);
    result = ::WriteFile(m_hWaveFile, &nAvgBytesPerSec, 4, &bytesWrite, NULL);

    //ubNTCY
    result = ::WriteFile(m_hWaveFile, &nBlockAlign, 2, &bytesWrite, NULL);

    //ʎqrbg
    buf[0] = m_decInfo.bitsPerSample;
    buf[1] = 0x00;
    result = ::WriteFile(m_hWaveFile, &buf, 2, &bytesWrite, NULL);

    ::strcpy((char*)buf,"data");
    result = ::WriteFile(m_hWaveFile, &buf, 4, &bytesWrite, NULL);
    ::memset(&buf, 0, 4);
    result = ::WriteFile(m_hWaveFile, &buf, 4, &bytesWrite, NULL);

    //fR[h
    //decodePos     = 0;        //Ԏ擾p
    ::SetFilePointer(m_hFile, m_firstSync, NULL, FILE_BEGIN);
    m_inputSize     = m_decInfo.maxInputSize * m_inputBlock;
    m_inputBuf      = (byte*)::realloc(m_inputBuf, m_inputSize);
    m_inputPtr      = m_inputBuf;
    m_inputBytes    = input_read(m_inputPtr, m_inputSize);

    if (m_inputBytes == 0) {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }

    //o̓foCXJ
/*
    OutputInfo  outputInfo;
    outputInfo.channels         = m_decInfo.channels;
    outputInfo.bitsPerSample    = m_decInfo.bitsPerSample;
    outputInfo.frequency        = m_decInfo.frequency;
    outputInfo.bufferSize       = m_decInfo.maxOutputSize * m_outputBlock;

    if (!output_open(NULL, outputInfo)) {
        return FALSE;
    }
*/
    m_outputBuf_in= (BYTE*)::realloc(m_outputBuf_in, m_decInfo.outputSize);

    if (!::mp3DecodeStart(m_inputPtr, m_inputBytes)) {
        output_close();
        _setError(ERR_DECODE);
        return FALSE;
    }

    m_inputPtr += m_decInfo.skipSize;
    m_inputBytes -= m_decInfo.skipSize;

    //fR[hpXbhJn
    /*
    m_hThread = (HANDLE)::_beginthreadex(NULL, 0, 
            (THREADFUNC)input_ThreadProc, NULL, 0, (unsigned int*)&m_threadID);
    if (m_hThread == NULL) {
        output_close();
        _setError(ERR_DECODE_THREAD);
        return FALSE;
    }
    //ԃ`FbN
    int Cnt;
    for ( Cnt = 0; Cnt < 100; Cnt++){
        ::Sleep(10);
        if ( m_state == STATE_PLAY ) {
            break;
        } else if ( Cnt == 99 ){
            _setError(ERR_PLAY);
            return FALSE;
        }
    }
    */
    //C by Kobarin
    HANDLE hStartEvent = ::CreateEvent(NULL, FALSE, FALSE, NULL);
    m_hThread = (HANDLE)::_beginthreadex(NULL, 0, 
            input_ThreadProc, hStartEvent, 0, (unsigned int*)&m_threadID);
    if (m_hThread == NULL) {
        CloseHandle(hStartEvent);
        output_close();
        _setError(ERR_DECODE_THREAD);
        return FALSE;
    }
    WaitForSingleObject(hStartEvent, INFINITE);
    CloseHandle(hStartEvent);
    //܂
    return TRUE;
}

BOOL input_playDecodeWave(const char* pszWaveName, const wchar_t* pszWaveNameW)
{
    BYTE            buf[1024];
    BOOL            result;
    DWORD           bytesWrite;

    if (m_state != STATE_STOP) {
        //~ȊO͎ss
        _setError(ERR_NOT_STATE_STOP);
        return FALSE;
    }

    //WAVEt@C
    if ( m_waveFlag == TRUE ){
        return FALSE;
    }

    decFlag = 2;
    m_fadeOutFlag = false;
    m_playFlag = TRUE;
    m_readWaveDataSize = 0;

    ::SetFilePointer(m_hFile, m_firstSync, NULL, FILE_BEGIN);
    m_inputSize     = m_decInfo.maxInputSize * m_inputBlock;
    m_inputBuf      = (byte*)::realloc(m_inputBuf, m_inputSize);
    m_inputPtr      = m_inputBuf;
    m_inputBytes    = input_read(m_inputPtr, m_inputSize);
    if (m_inputBytes == 0) {
        _setError(ERR_INVALID_VALUE);
        return FALSE;
    }

    //o̓foCXJ
    OutputInfo  outputInfo;
    outputInfo.channels         = m_decInfo.channels;
    outputInfo.bitsPerSample    = m_decInfo.bitsPerSample;
    outputInfo.frequency        = m_decInfo.frequency;

    outputInfo.bufferSize       = m_decInfo.outputSize * m_outputBlock_in;

    if (!output_open(NULL, outputInfo)) {
        _setError(ERR_OPEN_OUT_DEVICE);
        return FALSE;
    }

    m_outputBuf_in= (BYTE*)::realloc(m_outputBuf_in, outputInfo.bufferSize);

    if (!::mp3DecodeStart(m_inputPtr, m_inputBytes)) {
        output_close();
        _setError(ERR_DECODE);
        return FALSE;
    }
    m_inputPtr += m_decInfo.skipSize;
    m_inputBytes -= m_decInfo.skipSize;

	if (pszWaveName != nullptr)
	{
		//t@CI[v
		m_hWaveFile = ::CreateFile(pszWaveName, GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ,
			NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if (m_hWaveFile == INVALID_HANDLE_VALUE) {
			_setError(ERR_WAV_FILE_OPEN);
			return FALSE;
		}
	}
	else
	{
		//t@CI[v
		m_hWaveFile = ::CreateFileW(pszWaveNameW, GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ,
			NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		if (m_hWaveFile == INVALID_HANDLE_VALUE) {
			_setError(ERR_WAV_FILE_OPEN);
			return FALSE;
		}
	}

    SetFilePointer(m_hWaveFile, 0, NULL, FILE_BEGIN);
    ::strcpy((char*)buf,"RIFF");
    result = ::WriteFile(m_hWaveFile, &buf, 4, &bytesWrite, NULL);
    ::memset(&buf, 0, 4);
    result = ::WriteFile(m_hWaveFile, &buf, 4, &bytesWrite, NULL);
    ::strcpy((char*)buf,"WAVEfmt ");
    result = ::WriteFile(m_hWaveFile, &buf, 8, &bytesWrite, NULL);

    //`NTCY
    buf[0] = 0x10;
    buf[1] = 0x00;
    buf[2] = 0x00;
    buf[3] = 0x00;
    result = ::WriteFile(m_hWaveFile, &buf, 4, &bytesWrite, NULL);

    //tH[}bghc
    buf[0] = 0x01;
    buf[1] = 0x00;
    result = ::WriteFile(m_hWaveFile, &buf, 2, &bytesWrite, NULL);

    //`l
    buf[0] = m_decInfo.channels;
    buf[1] = 0x00;
    result = ::WriteFile(m_hWaveFile, &buf, 2, &bytesWrite, NULL);

    //TvO[g
    DWORD nSamplesPerSec = m_decInfo.frequency;
    result = ::WriteFile(m_hWaveFile, &nSamplesPerSec, 4, &bytesWrite, NULL);

    //][g
    WORD  nBlockAlign = (m_decInfo.channels * m_decInfo.bitsPerSample / 8);
    DWORD nAvgBytesPerSec = (nBlockAlign * m_decInfo.frequency);
    result = ::WriteFile(m_hWaveFile, &nAvgBytesPerSec, 4, &bytesWrite, NULL);

    //ubNTCY
    result = ::WriteFile(m_hWaveFile, &nBlockAlign, 2, &bytesWrite, NULL);

    //ʎqrbg
    buf[0] = m_decInfo.bitsPerSample;
    buf[1] = 0x00;
    result = ::WriteFile(m_hWaveFile, &buf, 2, &bytesWrite, NULL);

    ::strcpy((char*)buf,"data");
    result = ::WriteFile(m_hWaveFile, &buf, 4, &bytesWrite, NULL);
    ::memset(&buf, 0, 4);
    result = ::WriteFile(m_hWaveFile, &buf, 4, &bytesWrite, NULL);

    //fR[hpXbhJn
    /*
    m_hThread = (HANDLE)::_beginthreadex(NULL, 0, 
            (THREADFUNC)input_ThreadProc, NULL, 0, (unsigned int*)&m_threadID);
    if (m_hThread == NULL) {
        output_close();
        _setError(ERR_DECODE_THREAD);
        return FALSE;
    }
    //ԃ`FbN
    int Cnt;
    for ( Cnt = 0; Cnt < 100; Cnt++){
        ::Sleep(10);
        if ( m_state == STATE_PLAY ) {
            break;
        } else if ( Cnt == 99 ){
            _setError(ERR_PLAY);
            return FALSE;
        }
    }
    */
    //C by Kobarin
    HANDLE hStartEvent = ::CreateEvent(NULL, FALSE, FALSE, NULL);
    m_hThread = (HANDLE)::_beginthreadex(NULL, 0, 
            input_ThreadProc, hStartEvent, 0, (unsigned int*)&m_threadID);
    if (m_hThread == NULL) {
        CloseHandle(hStartEvent);
        output_close();
        _setError(ERR_DECODE_THREAD);
        return FALSE;
    }
    WaitForSingleObject(hStartEvent, INFINITE);
    CloseHandle(hStartEvent);
    //܂
    return TRUE;
}

BOOL input_stop()
{
    /*if (m_state == STATE_STOP) {
        //~͏Ȃ
        _setError(ERR_STATE_STOP);
        return TRUE;
    }*/
    
    /*if ( m_threadID == 0 ) {
        output_stop();
    //  output_close();
    }
    if (::PostThreadMessage(m_threadID, EV_STOP, 0, 0)) {
        ::WaitForSingleObject(m_hThread, 500);
    }*/
    //C by Kobarin
    if(!m_hThread)return TRUE;
    if(m_threadID == ::GetCurrentThreadId()){
        //fR[hXbhŌĂяoĂ͂Ȃ
        //R[obN֐LȏꍇɌĂ΂\
        ::PostThreadMessage(m_threadID, EV_STOP, 0, 0);
        MessageBox(NULL, "R[obN֐ vbmp3_stop() ĂяoĂ"
                         "܂", "G[", MB_OK);
        return FALSE;
    }
    ::PostThreadMessage(m_threadID, EV_STOP, 0, 0);
    output_stop();
    WaitForSingleObject(m_hThread, INFINITE);
    ::CloseHandle(m_hThread);
    m_hThread = NULL;        
    m_threadID = 0;          
    //܂
    m_playSec = 0;
    m_playSecCur = 0;
    m_playMs = 0;
    m_firstSample = 0;

    //ԃ`FbN
    /*int Cnt;
    for ( Cnt = 0; Cnt < 100; Cnt++){
        if ( m_state == STATE_STOP ) {
            break;
        } else if ( Cnt == 99 ){
            _setError(ERR_STOP);
            return FALSE;
        }
        ::Sleep(10);
    }*/
    //폜 by Kobarin
    return TRUE;
}

BOOL input_pause()
{
    int w_state = m_state;
    if (w_state == STATE_PAUSE ||
            w_state == STATE_STOP) {
        //ꎞ~A~͏Ȃ
        _setError(ERR_NOT_STATE_PLAY);
        return TRUE;
    }

    /*if ( m_threadID == 0 ) {
        input_actionPause();
        return TRUE;
    } else {
        return input_sendEvent(EV_PAUSE, 0);
    }*/
    //C by Kobarin
    if(m_threadID == ::GetCurrentThreadId()){
        //fR[hXbhŌĂяoĂ͂Ȃ
        //R[obN֐LȏꍇɌĂ΂\
        MessageBox(NULL, "R[obN֐ vbmp3_pause() ĂяoĂ"
                         "܂", "G[", MB_OK);
        return FALSE;
    }
    return input_sendEvent(EV_PAUSE, 0);
}

BOOL input_restart()
{
    if (m_state != STATE_PAUSE) {
        //ꎞ~ȊO͏Ȃ
        return TRUE;
    }

    /*if ( m_threadID == 0 ) {
        input_actionRestart();
        return TRUE;
    } else {
        return input_sendEvent(EV_RESTART, 0);
    }*/
    //C by Kobarin
    if(m_threadID == ::GetCurrentThreadId()){
        //fR[hXbhŌĂяoĂ͂Ȃ
        //R[obN֐LȏꍇɌĂ΂\
        MessageBox(NULL, "R[obN֐ vbmp3_restart() ĂяoĂ"
                         "܂", "G[", MB_OK);
        return FALSE;
    }
    return input_sendEvent(EV_RESTART, 0);
}

void input_setEqualizer(int* pTable)
{
    /*if (m_threadID) {
        input_sendEvent(EV_EQUALIZER, (int)pTable);
    }
    else {
        mp3SetEqualizer(pTable);
    }*/
    //C by Kobarin
    if(m_threadID == ::GetCurrentThreadId()){
        //fR[hXbhŌĂяoĂ͂Ȃ
        //R[obN֐LȏꍇɌĂ΂\
        MessageBox(NULL, "R[obN֐ vbmp3_setEqualizer() ĂяoĂ"
                         "܂", "G[", MB_OK);
        return;
    }
    input_sendEvent(EV_EQUALIZER, (int)pTable);
}


void input_seekByFrame(int frame)
{
    m_seekFlag = TRUE;
    int sample = input_bytesToSample(frame * m_decInfo.outputSize);
    input_seekBySample(sample);
}

void input_seekBySample(int sample)
{
    int w_state = m_state;

    if ( m_seekPlayFlag == FALSE && m_state == STATE_STOP ) return;

    m_seekFlag = TRUE;
    m_stateWork = w_state;
    /*if ( m_threadID == 0 && m_state != STATE_STOP) {
        input_stop();
    }
    input_play();
    if ( w_state == STATE_PAUSE )
        input_pause();
    input_sendEvent(EV_SEEK, sample);
    output_softVolumeMax();
    */
    //C by Kobarin
    if(m_threadID == ::GetCurrentThreadId()){
        //fR[hXbhŌĂяoĂ͂Ȃ
        //R[obN֐LȏꍇɌĂ΂\
        MessageBox(NULL, "R[obN֐ŃV[N֐ĂяoĂ"
                         "܂", "G[", MB_OK);
        return;
    }
    if(!m_hThread){
        input_play();
    }
seek_start: // Lenn ǉ(̍s) 2002.09.17.
    PostThreadMessage(m_threadID, EV_SUSPEND, 0, 0);
    output_stop();
    if ( w_state == STATE_PAUSE )
        input_pause();
    input_sendEvent(EV_SEEK, sample);
    output_softVolumeMax();
    //Sleep(1000);
    PostThreadMessage(m_threadID, EV_RESUME, 0, 0);
    //܂

    //ԃ`FbN
    /*int Cnt;
    for ( Cnt = 0; Cnt < 100; Cnt++){
        ::Sleep(10);
        if ( m_state == w_state ) {
            break;
        } else if ( Cnt == 99 ){
            _setError(ERR_PLAY);
        }
    }
    */
    //폜 by Kobarin
    //ǉ by Kobarin
    if(m_seekFlag){
        //V[NɎsiłɃfR[hXbhIĂj
        m_seekFlag = FALSE;
#if 1   // Lenn ǉ 2002.09.17. Ȃ̏It߂ŃV[Nƒ~oȌC
        DWORD exitCode;
        if(GetExitCodeThread(m_hThread, &exitCode)) {
            if(exitCode != STILL_ACTIVE) {
                m_seekFlag = TRUE;
                input_stop();
                input_play();
                goto seek_start;
            }
        }
#endif  // Lenn ܂
    }
}

void input_seekBySec(int sec)
{
    m_seekFlag = TRUE;
    int sample = input_secToSample((int)(sec / m_playSpeed));
    input_seekBySample(sample);
}

int input_bytesToSample(int bytes)
{
    if ( m_decInfo.bitsPerSample != 0 && m_decInfo.channels != 0){
        if ( bytes > ALLOW_DATA_SIZE ) {
            return (long)((double)bytes / (double)m_decInfo.bitsPerSample / (double)m_decInfo.channels * (double)8.0);
        } else {
            return (long)((double)bytes * (double)8.0 / (double)m_decInfo.bitsPerSample / (double)m_decInfo.channels);
        }
    } else {
        return 0;
    }
}

int input_secToSample(int sec)
{
    return (sec * m_decInfo.frequency);
}

int input_msToSample(int sec)
{
    return (long)((double)sec  * (double)m_decInfo.frequency / (double)1000.0);
}

int input_sampleToSec(int sample)
{
    if ( m_decInfo.frequency == 0 ) return 0;
    return (sample / m_decInfo.frequency);
}

long input_sampleToMs(int sample)
{
    if ( m_decInfo.frequency == 0 ) return 0;
    return (long)((double)sample * (double)1000.0 / (double)m_decInfo.frequency);
}

BOOL input_skipNextSync()
{
    int sync;
    m_inputPtr   ++;
    m_inputBytes --;
    for (;;) {
        if (::mp3FindSync(m_inputPtr, m_inputBytes, &sync)) {
            m_inputPtr += sync;
            m_inputBytes -= sync;
            return true;
        }
        ::memmove(m_inputBuf, m_inputPtr, m_inputBytes);
        int bytesRead = input_read(&m_inputBuf[m_inputBytes], m_inputSize - m_inputBytes);
        if (bytesRead == 0) {
            _setError(ERR_MP3_FILE_READ);
            return false;//no more data
        }
        m_inputPtr = m_inputBuf;
        m_inputBytes += bytesRead;
    }
}

BOOL input_setVolume(int left, int right)
{
    return output_setVolume(left, right);
}

BOOL input_getVolume(int* left, int* right)
{
    return output_getVolume(left, right);
}

int input_getWaveOutSupport()
{
    int support = output_getWaveOutSupport();
    if (support & WAVECAPS_LRVOLUME) {
        //En[h{[ݒ\
        return 2;
    }
    else if (support & WAVECAPS_VOLUME) {
        //n[h{[ݒ\iEsj
        return 1;
    }
    else {
        //n[h{[ݒs\
        return 0;
    }
}


void input_setSoftVolume(int left, int right)
{
    output_setSoftVolume(left, right);
}

void input_getSoftVolume(int* left, int* right)
{
    output_getSoftVolume(left, right);
}


#define FFT_BAND    256
#define WAVE_BAND   256

void input_getSpectrum(int* pSpecL, int* pSpecR)
{
    if ( m_state == STATE_STOP ) {
        ::memset(pSpecL, 0, FFT_BAND*sizeof(int));
        ::memset(pSpecR, 0, FFT_BAND*sizeof(int));
        return;
    }
    output_getSpectrum(output_getCurrentSample(), pSpecL, pSpecR);
    return;
}

void input_getWave(int* pWaveL, int* pWaveR)
{
    if ( m_state == STATE_STOP ) {
        ::memset(pWaveL, 0, WAVE_BAND*sizeof(int));
        ::memset(pWaveR, 0, WAVE_BAND*sizeof(int));
        return;
    }
    int sample = output_getCurrentSample();
    if ( sample ) output_getWave(sample, pWaveL, pWaveR);
    return;
}

BOOL input_sendEvent(int code, int param)
{
    HANDLE hEvent = ::CreateEvent(NULL, false, false, NULL);
    if (hEvent == NULL) {
        _setError(ERR_CREATE_EVENT);
        return FALSE;
    }
    if (::PostThreadMessage(m_threadID, code, (WPARAM)hEvent, param)) {
        //::WaitForSingleObject(hEvent, 500);
        //C by Kobarin
        //fR[hXbhI邩Cxgʒm܂ő҂
        HANDLE hEvents[2] = {hEvent, m_hThread};
        ::WaitForMultipleObjects(2, hEvents, FALSE, INFINITE);
        //܂
    }
    ::CloseHandle(hEvent);
    return TRUE;
}

//C by Kobarin
//int input_ThreadProc()
unsigned __stdcall input_ThreadProc(void *param)
{
    //return input_decode();
    return input_decode(param);
}

//C by Kobarin
//int input_decode()
unsigned input_decode(void *p)
{
    HANDLE hEvent = (HANDLE)p;//ǉ by Kobarin
    MPEG_DECODE_PARAM param;
    MSG     msg;
    DWORD   readBytes;
    int     oldSec;
    int     waitLCount = 0;
    int     waitRCount = 0;

    BOOL            result;
    DWORD           bytesWrite;

    oldSec = 0;
    m_state = STATE_PLAY;
    m_playSec = 0;
    m_playSecCur = 0;
    m_playMs = 0;
    m_playSamples = 0;
    m_firstSample = 0;
    decodePos = 0;
    param.outputBuf = m_outputBuf_in;
    ::PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE);//ǉ by Kobarin
    ::SetEvent(hEvent);//ǉ by Kobarin

decode_start://ǉ by Kobarini܂Ȃ(^^;j

    if ( m_seekFlag != TRUE && m_reloadFlag != TRUE ) {
        if ( callBackFlag == TRUE )
            m_pProc(INPUT_MSG_PLAY, m_playSec);
    }
    
    for (;;) {
        for (;;) {
            if ( m_waveFlag == FALSE ){
                param.inputBuf  = m_inputPtr;
                param.inputSize = m_inputBytes;

                if (!::mp3DecodeFrame(&param)) {
                    int err = ::mp3GetLastError();
                    if (err == MP3_ERROR_OUT_OF_BUFFER) {
                        break;//read next buffer
                    }
                    //invalid frame !!
                    if ( callBackFlag == TRUE )
                            m_pProc(INPUT_MSG_ERROR, 0);
                    if (!input_skipNextSync()) {
                        goto exit;
                    }
                    continue;
                }

                m_bitRate = param.bitRate;
                m_inputPtr   += param.inputSize;
                m_inputBytes -= param.inputSize;
            } else {
                //WAVEt@C
#if 0   // Lenn C 2002.09.22. ɎG邱ƂoȌC
                if ( (int)dataSize >= m_readWaveDataSize ) {
                    param.outputSize = m_inWaveBufSize;
                    m_readWaveDataSize = m_readWaveDataSize + param.outputSize;
                    if ( (int)dataSize < m_readWaveDataSize ) param.outputSize = m_readWaveDataSize - dataSize;
#else   // Lenn ܂ō폜Aȉǉ
                if ( (int)dataSize > m_readWaveDataSize ) {
                    if(m_readWaveDataSize + m_inWaveBufSize > dataSize)
                        param.outputSize = dataSize - m_readWaveDataSize;
                    else
                        param.outputSize = m_inWaveBufSize;
                    m_readWaveDataSize = m_readWaveDataSize + param.outputSize;
#endif  // Lenn ܂
                    if ( m_bitRate == 8 ) {
                        ::memset(param.outputBuf, 128, param.outputSize);
                    } else {
                        ::memset(param.outputBuf, 0, param.outputSize);
                    }
                    readBytes = input_read(param.outputBuf, param.outputSize);
                    if (readBytes == 0) {
                        goto exit;//no more data
                    }
#if 1   // Lenn ǉ 2002.09.22. ɎG邱ƂoȌC(Ô)
                    else if(readBytes < param.outputSize) {
                        param.outputSize = readBytes;
                    }
#else
                } // GfB^Œʂ̑ΉƂ̂ߒǉ
#endif  // Lenn ܂
                } else {
                    goto exit;
                }
            }
            //fR[h or WAVEt@Co
            if (decFlag == 0 || decFlag == 2) {
                if (output_write(param.outputBuf, param.outputSize)){
                    m_playSamples = output_getCurrentSample();
                    m_playSec = (int)(input_sampleToSec(m_playSamples + m_firstSample) * m_playSpeed);
                    if (m_playSec != oldSec) {
                        if ( m_seekFlag != TRUE && m_reloadFlag != TRUE ) {
                            if ( callBackFlag == TRUE )
                                m_pProc(INPUT_MSG_PLAY, m_playSec);
                        }
                        oldSec = m_playSec;
                    }
                }
            }
            if (decFlag == 1 || decFlag == 2) {
                decodePos += param.outputSize;
                result = ::WriteFile(m_hWaveFile, param.outputBuf, param.outputSize, &bytesWrite, NULL);
                m_playSamples = output_getCurrentSample();
                m_playSec = (int)(input_sampleToSec(m_playSamples + m_firstSample) * m_playSpeed);
                if (m_playSec != oldSec) {
                    if ( m_seekFlag != TRUE && m_reloadFlag != TRUE ) {
                        if ( callBackFlag == TRUE )
                            m_pProc(INPUT_MSG_PLAY, m_playSec);
                    }
                    oldSec = m_playSec;
                }
            }

            //tF[hAEg
            if ( m_fadeOutFlag ){
                waitLCount++;
                waitRCount++;
                if ( (m_lvol > 50 && waitLCount == 2) ||
                     (m_lvol > 10 && waitLCount == 3) ||
                     (m_lvol > 5  && waitLCount == 5) ||
                     (m_lvol > 0  && waitLCount == 10) ) {
                    m_lvol--;
                    waitLCount=0;
                }

                if ( (m_rvol > 50 && waitRCount == 2) ||
                     (m_rvol > 10 && waitRCount == 3) ||
                     (m_rvol > 5  && waitRCount == 5) ||
                     (m_rvol > 0  && waitRCount == 10) ) {
                    m_rvol--;
                    waitRCount=0;
                }
                output_setVolume(m_lvol, m_rvol);

                if (m_lvol == 0 && m_rvol == 0) {
                    m_lvol=m_olvol;
                    m_rvol=m_orvol;
                    m_fadeOutFlag = false;
                    output_stop();
                    goto exit;
                }
            }
            
            if (::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
                input_handleEvent(msg);
            }
            while (m_state == STATE_PAUSE) {
                if (::GetMessage(&msg, NULL, 0, 0)) {
                    input_handleEvent(msg);
               }
            }
            if (m_state == STATE_STOP) {
                goto exit;
            }
        }
        if ( m_waveFlag == FALSE ){
            ::memmove(m_inputBuf, m_inputPtr, m_inputBytes);
            readBytes = input_read(&m_inputBuf[m_inputBytes], m_inputSize - m_inputBytes);
            if (readBytes == 0) break;//no more data
            m_inputPtr = m_inputBuf;
            m_inputBytes += readBytes;
            ::Sleep(m_inputSleep);
        }
    }
    
exit:

    /*
    m_threadID = 0;
    while (::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
        HANDLE hEvent = (HANDLE)msg.wParam;
        if (hEvent != NULL) {
            ::SetEvent(hEvent);
        }
    }
    ::CloseHandle(m_hThread);
    m_hThread = NULL;        
    */
    //폜 by Kobarin
    if (m_state != STATE_STOP) {
        //ǉ by Kobarin
        while(output_wait(50) == WAIT_TIMEOUT){
            if (::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) {
                input_handleEvent(msg);
                if(msg.message == EV_SEEK){
                    goto decode_start;
                }
            }
#if 1   // Lenn ǉ 2002.09.17. Ȃ̏It߂ŃR[obN֐Ă΂ȂoȌC
            if ( m_seekFlag != TRUE && m_reloadFlag != TRUE ) {
                if ( callBackFlag == TRUE ) {
                    m_playSamples = output_getCurrentSample();
                    m_playSec = (int)(input_sampleToSec(m_playSamples + m_firstSample) * m_playSpeed);
                    if ( m_timeFlag && m_playSec > m_totalSec )
                        m_playSec = m_totalSec;
                    if (m_playSec != oldSec) {
                        m_pProc(INPUT_MSG_PLAY, m_playSec);
                        oldSec = m_playSec;
                    }
                }
            }
#endif  // Lenn ܂
        }
        //܂
        input_actionStop(TRUE);
        if ( m_seekFlag != TRUE && m_reloadFlag != TRUE ) {
            if ( callBackFlag == TRUE )
                m_pProc(INPUT_MSG_PLAYDONE, 0);
        }
    }
    m_playSec = 0;
    m_playSecCur = 0;
    m_playMs = 0;
    m_playSamples = 0;
    m_firstSample = 0;
    if (decFlag == 1 || decFlag == 2) {
        ::SetEndOfFile(m_hWaveFile);
        DWORD cSize = ::GetFileSize(m_hWaveFile, NULL) - 8;
        SetFilePointer(m_hWaveFile, 4, NULL, FILE_BEGIN);
        result = ::WriteFile(m_hWaveFile, &cSize, 4, &bytesWrite, NULL);

        DWORD dSize = cSize - 36;
        SetFilePointer(m_hWaveFile, 40, NULL, FILE_BEGIN);
        result = ::WriteFile(m_hWaveFile, &dSize, 4, &bytesWrite, NULL);

        ::CloseHandle(m_hWaveFile);
    }
    return 0;
}

void input_handleEvent(const MSG& msg)
{
    switch (msg.message) {
    case EV_STOP:
        input_actionStop(FALSE);
        break;
    case EV_PAUSE:
        input_actionPause();
        m_state = STATE_PAUSE;
        break;
    case EV_RESTART:
        input_actionRestart();
        m_state = STATE_PLAY;
        break;
    case EV_SEEK:
        input_actionSeek(msg.lParam);
        break;
    case EV_EQUALIZER:
        mp3SetEqualizer((int*)msg.lParam);
        break;
    case EV_SUSPEND:{
            HANDLE hEvent = (HANDLE)msg.wParam;
            if(hEvent) {
                ::SetEvent(hEvent);
            }
            MSG msg2;
            while(::GetMessage(&msg2, NULL, 0, 0) > 0){
                if(msg2.message == EV_RESUME){
                    break;
                }
                input_handleEvent(msg2);
            }
            return;
        }
    }
    HANDLE hEvent = (HANDLE)msg.wParam;
    if (hEvent != NULL) {
        ::SetEvent(hEvent);
    }

}

void input_actionStop(BOOL wait)
{
    if (!wait) {
        //~
        output_stop();
    }
    output_close();
    if (wait) {
        //
        output_stop();
    }
    m_state = STATE_STOP;
    if ( m_seekFlag != TRUE && m_reloadFlag != TRUE ) {
        if ( callBackFlag == TRUE && !wait)
            m_pProc(INPUT_MSG_STOP, 0);
    }
}

void input_actionPause()
{
    if (m_state == STATE_STOP) {
        return;//ignore
    }
    if (m_state == STATE_PLAY) {
        output_pause();
    }
    m_state = STATE_PAUSE;
    if ( m_seekFlag != TRUE && m_reloadFlag != TRUE ) {
        if ( callBackFlag == TRUE )
            m_pProc(INPUT_MSG_PAUSE, 0);
    }
}

void input_actionRestart()
{
    if (m_state == STATE_PAUSE) {
        output_restart();
    }
    m_state = STATE_PLAY;
    if ( m_seekFlag != TRUE && m_reloadFlag != TRUE ) {
        if ( callBackFlag == TRUE ){
            m_pProc(INPUT_MSG_PLAY, m_playSec);
        }
    }

}

void input_actionSeek(int seekSample)
{
    MPEG_DECODE_PARAM param;
    int bytesRead, outputBytes;
    int fSync = m_firstSync;

    outputBytes = 0;
    if ( m_waveFlag == FALSE && m_openFileVBR == FALSE && m_quickSeek == TRUE ){
        fSync = input_getSampleToSeekPoint(seekSample, &outputBytes);
        m_firstSample = input_bytesToSample(outputBytes);
    } else {
        fSync = m_firstSync;
    }

    output_stop();
    //::SetFilePointer(m_hFile, m_firstSync, NULL, FILE_BEGIN);
    ::SetFilePointer(m_hFile, fSync, NULL, FILE_BEGIN);

    m_playSecCur = 0;
    m_playMs = 0;
    m_inputBytes = 0;
    m_playSamples = 0;
    param.outputBuf = m_outputBuf_in;
    m_readWaveDataSize = 0;

    if ( m_waveFlag == FALSE){
        ::mp3MuteStart(&param);
        for (;;) {
            ::memmove(m_inputBuf, m_inputPtr, m_inputBytes);
            bytesRead = input_read(&m_inputBuf[m_inputBytes], m_inputSize - m_inputBytes);
            if (bytesRead == 0) break;//no more data
            m_inputPtr = m_inputBuf;
            m_inputBytes += bytesRead;

            for (;;) {
                param.inputBuf  = m_inputPtr;
                param.inputSize = m_inputBytes;
                if (!::mp3DecodeFrame(&param)) {
                    int err = ::mp3GetLastError();
                    if (err == MP3_ERROR_OUT_OF_BUFFER) {
                        break;//read next buffer
                    }
                    //invalid frame !!
                    if ( callBackFlag == TRUE )
                        m_pProc(INPUT_MSG_ERROR, 0);
                    if (!input_skipNextSync()) {
                        goto exit;
                    }
                    continue;
                }
                m_inputPtr   += param.inputSize;
                m_inputBytes -= param.inputSize;
                outputBytes += param.outputSize;
                m_firstSample = input_bytesToSample(outputBytes);
                if (m_firstSample >= seekSample) {
                    ::mp3DecodeStart(param.inputBuf, param.inputSize);//ǉ by Kobarin
                    ZeroMemory(param.outputBuf, param.outputSize);//ǉ by Kobarin
                    goto exit;
                }
            }
        }
    } else {
        m_firstSample = seekSample;
        decodePos = m_firstSample;
        outputBytes = output_sampleToBytes(m_firstSample);
        ::SetFilePointer(m_hFile, outputBytes, NULL, FILE_CURRENT);
        m_readWaveDataSize = m_readWaveDataSize + outputBytes;
    }

exit:
    if ( m_waveFlag == FALSE ) 
        ::mp3MuteEnd(&param);
    m_seekFlag = FALSE;
//  if ( callBackFlag == TRUE )
//      m_pProc(INPUT_MSG_PLAY, m_playSec);

}



/******************** ID3Tagv1.cpp ************************/
BOOL readID3v1(HANDLE FileHandle, TAG_INFO* lpTagInfo)
{
    ID3Tagv1    tag;
    BOOL        result;
    DWORD       bytesRead;

    //t@Č납128oCgǂݍ
    ::SetFilePointer(FileHandle, -(long)sizeof(tag), NULL, FILE_END);
    result = ::ReadFile(FileHandle, &tag, sizeof(tag), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(tag)) {
        return FALSE;
    }
    if (::memcmp(tag.tag, "TAG", 3) != 0) {
        return FALSE;
    }
    m_FileType += 16;

    //^Oۑ
    ::strcpyID3v1(lpTagInfo->szTrackName,  (char*)tag.trackName);
    ::strcpyID3v1(lpTagInfo->szArtistName, (char*)tag.artistName);
    ::strcpyID3v1(lpTagInfo->szAlbumName, (char*)tag.albumName);
    ::strcpyID3v1(lpTagInfo->szComment, (char*)tag.comment);
    ::memcpy(lpTagInfo->szYear,  tag.year,  4);
    lpTagInfo->genre = tag.genre;
    ::memcpy(lpTagInfo->szGanreName,  genre_tbl[tag.genre],  strlen(genre_tbl[tag.genre]));
    return TRUE;
}

BOOL readID3v11(HANDLE FileHandle, TAG_INFO_11* lpTagInfo)
{
    ID3Tagv1    tag;
    BOOL        result;
    DWORD       bytesRead;

    //t@Č납128oCgǂݍ
    ::SetFilePointer(FileHandle, -(long)sizeof(tag), NULL, FILE_END);
    result = ::ReadFile(FileHandle, &tag, sizeof(tag), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(tag)) {
        return FALSE;
    }
    if (::memcmp(tag.tag, "TAG", 3) != 0) {
        return FALSE;
    }
    m_FileType += 16;

    //^Oۑ
    ::strcpyID3v1(lpTagInfo->szTrackName,  (char*)tag.trackName);
    ::strcpyID3v1(lpTagInfo->szArtistName, (char*)tag.artistName);
    ::strcpyID3v1(lpTagInfo->szAlbumName, (char*)tag.albumName);
    ::strcpyID3v1(lpTagInfo->szComment, (char*)tag.comment);
#if 0   // Lenn C 2002.09.16. v1.0(trackNo=-1)  setTagInfoEx()
        // Ă񂾂Ƃ getFileTagInfoEx()  trackNo=0 ƂȂoȌC
    if ( tag.comment[28] == 0 ) {
#else   // Lenn ܂ō폜Aȉǉ
    if(( tag.comment[28] == 0 )&&( tag.comment[29] != 0 )) {
#endif  // Lenn ܂
        lpTagInfo->trackNo = tag.comment[29];
        tag.comment[28] = ' ';
        tag.comment[29] = ' ';
        ::strcpyID3v1(lpTagInfo->szComment, (char*)tag.comment);
    } else {
        lpTagInfo->trackNo = -1;
        ::strcpyID3v1(lpTagInfo->szComment, (char*)tag.comment);
    }

/*
    ::strcpyID3v1(lpTagInfo->szComment, (char*)tag.comment);
    if ( lpTagInfo->szComment[28] == 0 ) {
        lpTagInfo->trackNo = (BYTE)lpTagInfo->szComment[29];
        lpTagInfo->szComment[29] = 0;
    } else {
        lpTagInfo->trackNo = -1;
    }
*/

    ::memcpy(lpTagInfo->szYear,  tag.year,  4);
    lpTagInfo->genre = tag.genre;
    ::memcpy(lpTagInfo->szGanreName,  genre_tbl[tag.genre],  strlen(genre_tbl[tag.genre]));
    return TRUE;
}

//ɂXy[X؂
void strcpyID3v1(char* pszDest, const char* pszSrc)
{
    ::memcpy(pszDest,  pszSrc,  30);
    char* p1 = pszDest;
    char* p2 = pszDest + 29;
    for (; p2 > p1 && *p2 == ' '; p2 --) {}
    *(p2 + 1) = 0;
}


/******************** ID3Tagv2.cpp ************************/
BOOL readID3v2(HANDLE FileHandle,int* fSync)
{
    ID3Tagv2    tag;
    BOOL        result;
    DWORD       bytesRead;
    BYTE        temp[4];
    int         v3size;

    //t@C̓ɖ߂A^Õwb_ǂݍ
    ::SetFilePointer(FileHandle, *fSync, NULL, FILE_BEGIN);
    result = ::ReadFile(FileHandle, &tag, sizeof(tag), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(tag)) {
        return FALSE;
    }
    if (::memcmp(tag.id3, "ID3", 3) != 0) {
        return FALSE;
    }
    m_FileType += 4;

    //28rbg̐lϊ
    temp[3] =   tag.size[3]             | ((tag.size[2] & 1) << 7);
    temp[2] = ((tag.size[2] >> 1) & 63) | ((tag.size[1] & 3) << 6);
    temp[1] = ((tag.size[1] >> 2) & 31) | ((tag.size[0] & 7) << 5);
    temp[0] = ((tag.size[0] >> 3) & 15);
    v3size = temp[3] | ((DWORD)temp[2] << 8) | 
            ((DWORD)temp[1] << 16) | ((DWORD)temp[0] << 24) + 10;
    *fSync = *fSync + v3size;
    return TRUE;
}

long getID3v2Size(HANDLE FileHandle,int fSync){
    ID3Tagv2    tag;
    BOOL        result;
    DWORD       bytesRead;
    BYTE        temp[4];
    long        tagSize;

    //t@C̓ɖ߂A^Õwb_ǂݍ
    ::SetFilePointer(FileHandle, fSync, NULL, FILE_BEGIN);
    result = ::ReadFile(FileHandle, &tag, sizeof(tag), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(tag)) {
        return -1;
    }
    if (::memcmp(tag.id3, "ID3", 3) != 0) {
        return -1;
    }

    //28rbg̐lϊ
    temp[3] =   tag.size[3]             | ((tag.size[2] & 1) << 7);
    temp[2] = ((tag.size[2] >> 1) & 63) | ((tag.size[1] & 3) << 6);
    temp[1] = ((tag.size[1] >> 2) & 31) | ((tag.size[0] & 7) << 5);
    temp[0] = ((tag.size[0] >> 3) & 15);
    tagSize = temp[3] | ((DWORD)temp[2] << 8) | 
            ((DWORD)temp[1] << 16) | ((DWORD)temp[0] << 24) + 10;
    return tagSize;
}

/******************** RIFF.cpp ************************/
BOOL readRIFF(HANDLE FileHandle, LIST_INFO* lplistInfo, int* fSync, DWORD* dSize)
{
    RiffChunk   riff;
    CommonChunk common;
    BOOL        result;
    DWORD       bytesRead;
    DWORD       wkDataSize;
    DWORD       wkDataPoint;

    RiffStatus = 0;
    m_listChunkPoint = 0;
    m_listChunkSize = 0;

    //t@C̓ɖ߂ARIFF`Nǂݍ
    ::SetFilePointer(FileHandle, 0, NULL, FILE_BEGIN);
    result = ::ReadFile(FileHandle, &riff, sizeof(riff), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(riff)) {
        return FALSE;
    }
    if (::memcmp(riff.chunkID, "RIFF", 4) != 0) {
        return FALSE;
    }

    //FileType
    if (memcmp(riff.formatType, "WAVE", 4) == 0) {
        m_FileType += 1;
    }
    if (memcmp(riff.formatType, "RMP3", 4) == 0) {
        m_FileType += 2;
    }

    *fSync = sizeof(riff);

    //data`NT
    for (;;) {
        result = ::ReadFile(FileHandle, &common, sizeof(common), &bytesRead, NULL);
        if (!result || bytesRead != sizeof(common)) {
            return FALSE;
        }

        if (::memcmp(common.chunkID, "fmt ", 4) == 0) {
            //MpegȊÕtH[}bg`FbN
            //::ReadFile(FileHandle, &m_waveFormat, common.chunkSize, &bytesRead, NULL);
            ::ReadFile(FileHandle, &m_waveFormat, sizeof(WAVEFORMATEX), &bytesRead, NULL);
            if (!result || bytesRead != sizeof(WAVEFORMATEX)) {
                RiffStatus = -1;
                return FALSE;
            }
            //tH[}bgTag`FbN
            if (m_waveFormat.wFormatTag == 85 || m_waveFormat.wFormatTag == 80) {
                //MPEG Audio f[^
                //common.chunkSize = 0;  //`Nǂݔ΂Ȃ
            } else if (m_waveFormat.wFormatTag == 1) {
                //Aoblf[^
                RiffStatus = 1;
                //common.chunkSize = 0;  //`Nǂݔ΂Ȃ
            } else {
                //̑̌`
                RiffStatus = -1;
                return FALSE; 
            }
            ::SetFilePointer(FileHandle, -(long)sizeof(WAVEFORMATEX), NULL, FILE_CURRENT);
/*
            if (common.chunkSize != 0x1E && common.chunkSize != 0x20) {
                if (common.chunkSize == 0x10 || common.chunkSize == 0x12) {
                    ::ReadFile(FileHandle, &m_waveFormat, common.chunkSize, &bytesRead, NULL);
                    if (!result || bytesRead != common.chunkSize) {
                        RiffStatus = -1;
                        return FALSE;
                    }
                    RiffStatus = 1;
                    common.chunkSize = 0;  //`Nǂݔ΂Ȃ
                    //return TRUE;
                } else {
                    RiffStatus = -1;
                    return FALSE; 
                }
            }
*/
        }
        
        if (::memcmp(common.chunkID, "data", 4) == 0) {
            //data`N
            break;
        }
        else {
            //̃`N܂œǂݔ΂
            *fSync = ::SetFilePointer(FileHandle, common.chunkSize, NULL, FILE_CURRENT);
        }
    }

    //`NTCY`FbN
    wkDataSize = *dSize;
    if (common.chunkSize < wkDataSize) {
        *dSize = common.chunkSize;
    }

    *fSync += bytesRead;

    wkDataPoint = *fSync + common.chunkSize;

    //RIFF MP3 LIST`NT
    if ( (common.chunkSize % 2) == 1 ) common.chunkSize++;
    if (::SetFilePointer(FileHandle, common.chunkSize, NULL, FILE_CURRENT) == 0xFFFFFFFF && GetLastError() != NO_ERROR ){
        return TRUE;
    };
    wkDataPoint += common.chunkSize;
    for (;;) {
        result = ::ReadFile(FileHandle, &common, sizeof(common), &bytesRead, NULL);
        wkDataPoint += sizeof(common);
        if (!result || bytesRead != sizeof(common)) {
            return TRUE;
        }
        if (memcmp(common.chunkID, "LIST", 4) == 0) {
            m_listChunkPoint = wkDataPoint;
            m_listChunkSize = common.chunkSize;
            //LIST`N
            break;
        }
        else {
            //̃`N܂œǂݔ΂
            ::SetFilePointer(FileHandle, common.chunkSize, NULL, FILE_CURRENT);
            wkDataPoint += common.chunkSize;
        }
    }
    BYTE* pRMPList = (BYTE*)::malloc(common.chunkSize);
    if (pRMPList == NULL) {
        return TRUE;
    }
    BYTE listID[4];
    result = ::ReadFile(FileHandle, listID, 4, &bytesRead, NULL);
    wkDataPoint += 4;
    if (!result || bytesRead != 4) {
        ::free(pRMPList);
        return TRUE;
    }
    if (::memcmp(listID, "INFO", 4) != 0) {
        //INFOȊO͖
        ::free(pRMPList);
        return TRUE;
    }
    RMPInfo* pRMPInfo = (RMPInfo*)::malloc(sizeof(RMPInfo));
    if (pRMPInfo == NULL) {
        ::free(pRMPList);
        return TRUE;
    }
    DWORD remains = common.chunkSize - 4;
    DWORD baseSize = sizeof(RMPInfo) - sizeof(pRMPInfo->data);
    int dumy;
    while (remains >= sizeof(RMPInfo)) {
        result = ::ReadFile(FileHandle, pRMPInfo, baseSize, &bytesRead, NULL);
        wkDataPoint += baseSize;
        if (!result || bytesRead != baseSize) {
            break;
        }
        dumy = 0;
        if ((pRMPInfo->length % 2) == 1 ) dumy = 1;

        pRMPInfo = (RMPInfo*)::realloc(pRMPInfo, baseSize + pRMPInfo->length + dumy);
        if (pRMPInfo == NULL) {
            break;
        }
        result = ::ReadFile(FileHandle, pRMPInfo->data, pRMPInfo->length + dumy, &bytesRead, NULL);
        wkDataPoint += pRMPInfo->length + dumy;
        if (!result || bytesRead != pRMPInfo->length + dumy) {
            break;
        }
        loadRMPInfo(lplistInfo, pRMPInfo);
    
        remains -= baseSize + pRMPInfo->length + dumy;
    }
    ::free(pRMPList);
    ::free(pRMPInfo);
    return TRUE;
}


#define CHECK(x)    (::memcmp(pRMPInfo->infoID, x, 4) == 0)
#define SET(x, y)   strcpyRMP(x, (char*)pRMPInfo->data, pRMPInfo->length, y)

void loadRMPInfo(LIST_INFO* lplistInfo, RMPInfo* pRMPInfo)
{
    if (CHECK("INAM")) {        //track name
        SET(lplistInfo->INAM, INFO_NAME_SIZE);
    }
    else if (CHECK("IART")) {   //artist name
        SET(lplistInfo->IART, INFO_NAME_SIZE);
    }
    else if (CHECK("IPRD")) {
        SET(lplistInfo->IPRD, INFO_NAME_SIZE);
    }
    else if (CHECK("ICMT")) {
        SET(lplistInfo->ICMT, INFO_NAME_SIZE);
    }
    else if (CHECK("ICRD")) {
        SET(lplistInfo->ICRD, INFO_NAME_SIZE);
    }
    else if (CHECK("IGNR")) {
        SET(lplistInfo->IGNR, INFO_NAME_SIZE);
    }
    else if (CHECK("ICOP")) {
        SET(lplistInfo->ICOP, INFO_NAME_SIZE);
    }
    else if (CHECK("IENG")) {
        SET(lplistInfo->IENG, INFO_NAME_SIZE);
    }
    else if (CHECK("ISRC")) {
        SET(lplistInfo->ISRC, INFO_NAME_SIZE);
    }
    else if (CHECK("ISFT")) {
        SET(lplistInfo->ISFT, INFO_NAME_SIZE);
    }
    else if (CHECK("IKEY")) {
        SET(lplistInfo->IKEY, INFO_NAME_SIZE);
    }
    else if (CHECK("ITCH")) {
        SET(lplistInfo->ITCH, INFO_NAME_SIZE);
    }
    else if (CHECK("ILYC")) {
        SET(lplistInfo->ILYC, INFO_NAME_SIZE);
    }
    else if (CHECK("ICMS")) {
        SET(lplistInfo->ICMS, INFO_NAME_SIZE);
    }
}

void strcpyRMP(char* pszDest, const char* pszSrc, int length, int maxSize)
{
    if (length >= maxSize) {
        length = maxSize - 1;
        ::memcpy(pszDest, pszSrc, length);
        pszDest[maxSize - 1] = 0;
    }
    else {
        ::memcpy(pszDest, pszSrc, length);
        pszDest[length] = 0;
    }
}

BOOL chkLYRICS3(HANDLE FileHandle, DWORD* dSize)
{
    BOOL        result;
    DWORD       bytesRead;
    BYTE        buf[128];
    char        lysize[7];
    int         lyPoint = 9;

    *dSize = 0;

    //t@Č납128oCgǂݍ
    ::SetFilePointer(FileHandle, -128, NULL, FILE_END);
    result = ::ReadFile(FileHandle, &buf, 128, &bytesRead, NULL);
    if (!result || bytesRead != 128) {
        return FALSE;
    }
    if (::memcmp(buf, "TAG", 3) == 0) {
        //ID3Tag v1 ݂
        lyPoint += 128;
    }

    //LYRICS3 ̗L`FbN
    ::SetFilePointer(FileHandle, -lyPoint, NULL, FILE_END);
    result = ::ReadFile(FileHandle, &buf, 9, &bytesRead, NULL);
    if (!result || (int)bytesRead != 9) {
        return FALSE;
    }
    if (::memcmp(buf, "LYRICS200", 9) != 0) {
        return FALSE;
    } else {
        lyPoint += 6;
        ::SetFilePointer(FileHandle, -lyPoint, NULL, FILE_END);
        result = ::ReadFile(FileHandle, &buf, 6, &bytesRead, NULL);
        if (!result || (int)bytesRead != 6) {
            return FALSE;
        }
        ::memcpy(lysize, buf, 6);
        lysize[6] = '\0';
        *dSize = atoi(lysize) + 15;
        m_FileType += 32;
        return TRUE;
    }
}

BOOL input_setLyricsFile(const char* pszLyricsName, const wchar_t* pszLyricsNameW)
{
    FILE*       m_LyricsFile;               //̎t@C
    char        buf[1024];
    long        sec = 0;
    int         lyPoint;
    int         lySize;
    char*       pbuf;
    int         nextTagPoint;
    int         readDataSize;
    int         result;
    long        befTime = 0;
    BOOL        atTagFlag;

    /*
    if (m_state != STATE_STOP) {
        //~ȊO͎ss
        _setError(ERR_NOT_STATE_STOP);
        return FALSE;
    }
    */

    //
    input_clearLyrics();

	if (pszLyricsName != nullptr)
	{
		//t@CI[v`FbN
		if (!isVar(pszLyricsName)) {
			//͂Ȃꍇ͏I
			return TRUE;
		}

		if ((m_LyricsFile = ::fopen(pszLyricsName, "r")) == NULL) {
			_setError(ERR_LYRICS_FILE_OPEN);
			return FALSE;
		}
	}
	else
	{
		//t@CI[v`FbN
		if (!isVarW(pszLyricsNameW)) {
			//͂Ȃꍇ͏I
			return TRUE;
		}

		if ((m_LyricsFile = ::_wfopen(pszLyricsNameW, L"r")) == NULL) {
			_setError(ERR_LYRICS_FILE_OPEN);
			return FALSE;
		}
	}

    if ( m_KaraokeTagFlag ) {
        readDataSize = 1024;
    } else {
        readDataSize = 128;
    }
    while(fgets(buf, readDataSize, m_LyricsFile) != NULL){
        buf[readDataSize - 1] = '\0';

        //^OΉ
        atTagFlag = FALSE;
        if ( buf[0] == '@' ) atTagFlag = checkAtTag(buf, readDataSize, &m_atTag, m_lyricsLineNo);   //^OǍ

        m_liricsLinePointMax[m_lyricsLineNo] = 0;
        result = input_getTimeLyrics(buf, &sec, &lyPoint, &lySize, &nextTagPoint);

        //̎sǂݍݗp^O
        m_lyrics[m_lyricsCount].tagFlag = TRUE;
        m_lyrics[m_lyricsCount].sec = befTime;
        ::memset(m_lyrics[m_lyricsCount].lyrics, 0, 128);
        m_lyrics[m_lyricsCount].textSize = 0;
        m_lyrics[m_lyricsCount].lineno = m_lyricsLineNo;
        m_lyrics[m_lyricsCount].point = 0;
        m_lyrics[m_lyricsCount].atTagFlag = FALSE;
        m_lyricsCount++;
        if ( m_lyricsCount == 1000 ){
            ::fclose(m_LyricsFile);
            return TRUE;
        }

        if ( result == -1 ){
            //if ( m_lyricsCount > 0 && m_NotTagLineRead == 1 ) {
                //Õ^C^OZbgiOs̑j
                m_lyrics[m_lyricsCount-1].tagFlag = FALSE;
                m_lyrics[m_lyricsCount].tagFlag = FALSE;
                m_lyrics[m_lyricsCount].sec = befTime;
                ::memset(m_lyrics[m_lyricsCount].lyrics, 0, 128);
                m_lyrics[m_lyricsCount].textSize = lySize;
                ::memcpy(&m_lyrics[m_lyricsCount].lyrics, &buf[lyPoint], lySize);
                m_lyrics[m_lyricsCount].lineno = m_lyricsLineNo;
                m_liricsLinePointMax[m_lyricsLineNo]++;
                m_lyrics[m_lyricsCount].point = m_liricsLinePointMax[m_lyricsLineNo];
                m_lyrics[m_lyricsCount].atTagFlag = atTagFlag;
                m_lyricsCount++;
                m_lyricsLineNo++;
                if ( m_lyricsCount == 1000 ){
                    ::fclose(m_LyricsFile);
                    return TRUE;
                }
            //}
        } else {
            pbuf = buf;
            for(;;){
                m_lyrics[m_lyricsCount].tagFlag = TRUE;
                if ( sec == -1 ) {
                    m_lyrics[m_lyricsCount].sec = befTime;
                }
                else {
                    m_lyrics[m_lyricsCount].sec = sec;
                }
                ::memset(m_lyrics[m_lyricsCount].lyrics, 0, 128);
                m_lyrics[m_lyricsCount].textSize = lySize;
                ::memcpy(&m_lyrics[m_lyricsCount].lyrics, &pbuf[lyPoint], lySize);
                m_lyrics[m_lyricsCount].lineno = m_lyricsLineNo;
                m_liricsLinePointMax[m_lyricsLineNo]++;
                m_lyrics[m_lyricsCount].point = m_liricsLinePointMax[m_lyricsLineNo];
                if ( m_lyrics[m_lyricsCount].point == 1 ) {
                    m_lyrics[m_lyricsCount-1].sec = sec;
                    if ( (sec - befTime) > m_lyricsNextAdjustTime + 100) m_lyrics[m_lyricsCount-1].sec -= m_lyricsNextAdjustTime;
                }
                befTime = sec;
                m_lyrics[m_lyricsCount].atTagFlag = FALSE;

                m_lyricsCount++;
                if ( m_lyricsCount == 1000 ){
                    ::fclose(m_LyricsFile);
                    return TRUE;
                }
                if ( result == 0 ) {
                    m_lyricsLineNo++;
                    break;
                }
                pbuf += nextTagPoint;
                result = input_getTimeLyrics(pbuf, &sec, &lyPoint, &lySize, &nextTagPoint);
            }
        }
    }

    ::fclose(m_LyricsFile);
    return TRUE;

}

BOOL checkAtTag(char* buf, int bufSize, AT_TAG* pAtTadData, int LineNo)
{
    int target;
    int startPoint;
    int insertPoint;
    char atTagName[1024];
    char atTagValue[1024];
    int LTrimFlag = 0;

    //^O擾
    insertPoint = 0;
    startPoint = 0;
    for(target = startPoint; target < bufSize; target++){
        if ( buf[target] == '\0' ) return FALSE;
        if ( buf[target] == '=' ) break;
        if ( buf[target] != ' ' ){
            atTagName[insertPoint] = buf[target];
            insertPoint++;
        }
    }
    atTagName[insertPoint] = '\0';
    target++;

    //^O̒l擾
    insertPoint = 0;
    startPoint = target;
    
    for(target = startPoint; target < bufSize; target++){
        if ( buf[target] == '\n' || buf[target] == '\r' || buf[target] == '\0') break;
        if ( buf[target] == ' ' && LTrimFlag == 0 ){
            continue;
        }
        LTrimFlag = 1;
        atTagValue[insertPoint] = buf[target];
        insertPoint++;
    }
    atTagValue[insertPoint] = '\0';

    if (::memcmp(&atTagName, "@Artist", 7) == 0 ){
        ::strcpy(pAtTadData->Artist, atTagValue);
        pAtTadData->ArtistLine = LineNo;
    } else if (::memcmp(atTagName, "@Title", 6) == 0 ){
        ::strcpy(pAtTadData->Title, atTagValue);
        pAtTadData->TitleLine = LineNo;
    } else if (::memcmp(atTagName, "@Album", 6) == 0 ){
        ::strcpy(pAtTadData->Album, atTagValue);
        pAtTadData->AlbumLine = LineNo;
    } else if (::memcmp(atTagName, "@Bgfile", 7) == 0 ){
        ::strcpy(pAtTadData->Bgfile, atTagValue);
        pAtTadData->BgfileLine = LineNo;
    } else if (::memcmp(atTagName, "@Bgfolder", 9) == 8 ){
        ::strcpy(pAtTadData->Bgfolder, atTagValue);
        pAtTadData->BgfolderLine = LineNo;
    } else if (::memcmp(atTagName, "@TimeRatio", 10) == 0 ){
        pAtTadData->TimeRatio = (double)::atof(atTagValue);
        pAtTadData->TimeRatioLine = LineNo;
        if ( pAtTadData->TimeRatio != 0 ) m_atTag.lyricsTotalFlag = 2;
    } else if (::memcmp(atTagName, "@Offset", 7) == 0 ){
        pAtTadData->Offset = ::atoi(atTagValue);
        pAtTadData->OffsetLine = LineNo;
    } else if (::memcmp(atTagName, "@SilencemSec", 12) == 0 ){
        pAtTadData->SilencemSec = ::atoi(atTagValue);
        pAtTadData->SilencemSecLine = LineNo;
    } else if (::memcmp(atTagName, "@TaggingBy", 10) == 0 ){
        ::strcpy(pAtTadData->TaggingBy, atTagValue);
        pAtTadData->TaggingByLine = LineNo;
    } else if (::memcmp(atTagName, "@EditedBy", 9) == 0 ){
        ::strcpy(pAtTadData->EditedBy, atTagValue);
        pAtTadData->EditedByLine = LineNo;
    } else if (::memcmp(atTagName, "@Silence", 8) == 0 ){
        pAtTadData->Silence = ::atoi(atTagValue);
        pAtTadData->SilenceLine = LineNo;
    } else if (::memcmp(atTagName, "@Flames", 7) == 0 ){
        pAtTadData->Flames = ::atoi(atTagValue);
        pAtTadData->FlamesLine = LineNo;
    } else if (::memcmp(atTagName, "@TotalSec", 9) == 0 ){
        pAtTadData->TotalSec = ::atoi(atTagValue);
        pAtTadData->TotalSecLine = LineNo;
    } else if (::memcmp(atTagName, "@TimeType", 9) == 0 ){
        ::strcpy(pAtTadData->TimeType, atTagValue);
        pAtTadData->TimeTypeLine = LineNo;
        if ( m_atTag.TimeRatio == 0 ){
            if ( ::memcmp(atTagValue, "WinAmp", 6) == 0 ){
                m_atTag.lyricsTotalFlag = 0;
            } 
            else if ( ::memcmp(atTagValue, "Normal", 6) == 0 ){
                m_atTag.lyricsTotalFlag = 1;
            }
        }

    } else {
        return FALSE;
    }
    pAtTadData->existFlag = 1;
    return TRUE;

}

void reflectedAtMarkTag(AT_TAG atTadData)
{
    if ( atTadData.Offset != 0 ) m_lyricsAdjustTime = atTadData.Offset;
    if ( atTadData.TimeRatio != 1 ) m_timeRatio = atTadData.TimeRatio;
}

BOOL input_getLyricsText(LYRICS_WORK lyWork, char* buf, int NLFlag)
{
    int             mm;
    int             ss;
    int             ms;
    char            cmm[4];
    char            css[3];
    char            cms[3];

    if ( lyWork.point == 0 ) return FALSE;

    if ( NLFlag == 1 && lyWork.lineno != 0 && lyWork.point == 1 ){
        //::strcat(buf, "\r\n" );
        ::strcat(buf, "\n" );
    }

    if( lyWork.tagFlag == TRUE ){
        mm = int(lyWork.sec / 60000);
        ss = int(lyWork.sec / 1000) - mm * 60;
        ms = lyWork.sec - mm * 60000 - ss * 1000;
        itoa(mm, cmm, 10);
        if( strlen(cmm) == 1 ){
            cmm[2] = '\0';
            cmm[1] = cmm[0];
            cmm[0] = '0';
        }
        itoa(ss, css, 10); 
        if( strlen(css) == 1 ){
            css[2] = '\0';
            css[1] = css[0];
            css[0] = '0';
        }
        itoa((int)(ms / 10), cms, 10); 
        if( strlen(cms) == 1 ){
            cms[2] = '\0';
            cms[1] = cms[0];
            cms[0] = '0';
        }
        strcat(buf, "[" );
        strcat(buf, cmm );
        strcat(buf, ":" );
        strcat(buf, css );
        strcat(buf, ":" );
        strcat(buf, cms );
        strcat(buf, "]" );
    }
    ::strcat(buf, lyWork.lyrics);

    return TRUE;
}

BOOL input_setReadLyrics3Info(const char* pszName, const wchar_t* pszNameW)
{
    LYRICS3_INFO    lyrics3Info;
    int             lyCnt;
    int             mm;
    int             ss;
    int             ms;
    char            cmm[4];
    char            css[3];
    char            cms[3];

    if( m_lyricsCount == 0 ) return FALSE;

    ::memset(&lyrics3Info, 0, sizeof(LYRICS3_INFO));

    for(lyCnt = 0; lyCnt < m_lyricsCount; lyCnt++){
        if (m_lyrics[lyCnt].lineno != 0 && m_lyrics[lyCnt].point == 1 ){
            strcat(lyrics3Info.LYR, "\r\n" );
        }

        if(m_lyrics[lyCnt].tagFlag == TRUE ){
            mm = int(m_lyrics[lyCnt].sec / 60000);
            ss = int(m_lyrics[lyCnt].sec / 1000) - mm * 60;
            ms = m_lyrics[lyCnt].sec - mm * 60000 - ss * 1000;
            itoa(mm, cmm, 10);
            if( strlen(cmm) == 1 ){
                cmm[2] = '\0';
                cmm[1] = cmm[0];
                cmm[0] = '0';
            }
            itoa(ss, css, 10); 
            if( strlen(css) == 1 ){
                css[2] = '\0';
                css[1] = css[0];
                css[0] = '0';
            }
            itoa(ms, cms, 10); 
            if( strlen(cms) == 1 ){
                cms[2] = '\0';
                cms[1] = cms[0];
                cms[0] = '0';
            }
            strcat(lyrics3Info.LYR, "[" );
            strcat(lyrics3Info.LYR, cmm );
            strcat(lyrics3Info.LYR, ":" );
            strcat(lyrics3Info.LYR, css );
            strcat(lyrics3Info.LYR, ":" );
            strcat(lyrics3Info.LYR, cms );
            strcat(lyrics3Info.LYR, "]" );
        }
        strcat(lyrics3Info.LYR, m_lyrics[lyCnt].lyrics );
    }

    lyrics3Info.IND_LYR = '1';
    lyrics3Info.IND_TIMETAG = '1';

    return input_setFileLyrics3Info(pszName, pszNameW, &lyrics3Info, 1);
}

BOOL input_setFileLyrics3Info(const char* pszName, const wchar_t* pszNameW, LYRICS3_INFO* lpLyrics3Info, int setFlag)
{
    HANDLE          hFileLylics3;           //LYRICS3ҏWt@C
    BYTE            tag[128];
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    BYTE            buf[302000];
    int             siFlag = 0;
    DWORD           target = 0;
    int             tagFlag = 0;
    int             riffFlag = 0;
    int             lyFlag = 0;
    int             insPoint = 0;
    DWORD           lySize = 0;         //TCY
    DWORD           fieldSize;
    char            fieldSizeChar[6];
    int             fieldFigure;

	if (pszName != nullptr)
	{
		//t@CI[v`FbN
		if (!isVar(pszName)) {
			//͂Ȃꍇ͏I
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}

		//t@CI[v
		hFileLylics3 = ::CreateFile(pszName, GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFileLylics3 == INVALID_HANDLE_VALUE) {
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}
	}
	else
	{
		//t@CI[v`FbN
		if (!isVarW(pszNameW)) {
			//͂Ȃꍇ͏I
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}

		//t@CI[v
		hFileLylics3 = ::CreateFileW(pszNameW, GENERIC_READ + GENERIC_WRITE, FILE_SHARE_READ,
			NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (hFileLylics3 == INVALID_HANDLE_VALUE) {
			_setError(ERR_MP3_FILE_OPEN);
			return FALSE;
		}
	}

    //RIFF``FbN
    SetFilePointer(hFileLylics3, 0, NULL, FILE_BEGIN);
    result = ReadFile(hFileLylics3, buf, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileLylics3);
        return FALSE;
    }
    if (memcmp(buf, "RIFF", 4) == 0) {
        riffFlag = 1;
    }

    //ID3Tagv1 擾
    SetFilePointer(hFileLylics3, -(long)128, NULL, FILE_END);
    result = ReadFile(hFileLylics3, tag, 128, &bytesRead, NULL);
    if (!result || bytesRead != 128) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(hFileLylics3);
        return FALSE;
    }
    if (memcmp(tag, "TAG", 3) == 0) {
        tagFlag = 1;
        insPoint = 128;
    }

    //LYRICS3݃`FbN
    if (chkLYRICS3(hFileLylics3, &lySize)) {
        lyFlag = 1;
        insPoint += lySize;
    }

    ::memset(buf, '0', sizeof(buf));
    target=0;

    if ( setFlag == 1 ) {
        //LYRICS3i[
        ::memcpy(&buf[target], "LYRICSBEGIN", 11);
        target += 11;
        ::memcpy(&buf[target], "IND00002", 8);
        target += 8;
        buf[target] = lpLyrics3Info->IND_LYR;
        buf[target + 1] = lpLyrics3Info->IND_TIMETAG;
        target += 2;

        ::memcpy(&buf[target], "LYR", 3);
        target += 3;
        for (fieldSize = 60000; fieldSize > 0 && (lpLyrics3Info->LYR[fieldSize - 1] == ' ' ||
                                                  lpLyrics3Info->LYR[fieldSize - 1] == '\0') ; fieldSize --) {}
        ::ltoa(fieldSize, fieldSizeChar, 10);
        fieldFigure = ::strlen(fieldSizeChar);
        if ( fieldFigure < 5 ) target += (5 - fieldFigure);
        ::memcpy(&buf[target], fieldSizeChar, fieldFigure);
        target += fieldFigure;
        ::memcpy(&buf[target], lpLyrics3Info->LYR, fieldSize);
        target += fieldSize;

        ::memcpy(&buf[target], "INF", 3);
        target += 3;
        for (fieldSize = 60000; fieldSize > 0 && (lpLyrics3Info->INF[fieldSize - 1] == ' ' ||
                                                  lpLyrics3Info->INF[fieldSize - 1] == '\0') ; fieldSize --) {}
        ::ltoa(fieldSize, fieldSizeChar, 10);
        fieldFigure = ::strlen(fieldSizeChar);
        if ( fieldFigure < 5 ) target += (5 - fieldFigure);
        ::memcpy(&buf[target], fieldSizeChar, fieldFigure);
        target += fieldFigure;
        ::memcpy(&buf[target], lpLyrics3Info->INF, fieldSize);
        target += fieldSize;

        ::memcpy(&buf[target], "AUT", 3);
        target += 3;
        for (fieldSize = 250; fieldSize > 0 && (lpLyrics3Info->AUT[fieldSize - 1] == ' ' ||
                                                lpLyrics3Info->AUT[fieldSize - 1] == '\0') ; fieldSize --) {}
        ::ltoa(fieldSize, fieldSizeChar, 10);
        fieldFigure = ::strlen(fieldSizeChar);
        if ( fieldFigure < 5 ) target += (5 - fieldFigure);
        ::memcpy(&buf[target], fieldSizeChar, fieldFigure);
        target += fieldFigure;
        ::memcpy(&buf[target], lpLyrics3Info->AUT, fieldSize);
        target += fieldSize;

        ::memcpy(&buf[target], "EAL", 3);
        target += 3;
        for (fieldSize = 250; fieldSize > 0 && (lpLyrics3Info->EAL[fieldSize - 1] == ' ' ||
                                                lpLyrics3Info->EAL[fieldSize - 1] == '\0') ; fieldSize --) {}
        ::ltoa(fieldSize, fieldSizeChar, 10);
        fieldFigure = ::strlen(fieldSizeChar);
        if ( fieldFigure < 5 ) target += (5 - fieldFigure);
        ::memcpy(&buf[target], fieldSizeChar, fieldFigure);
        target += fieldFigure;
        ::memcpy(&buf[target], lpLyrics3Info->EAL, fieldSize);
        target += fieldSize;

        ::memcpy(&buf[target], "EAR", 3);
        target += 3;
        for (fieldSize = 250; fieldSize > 0 && (lpLyrics3Info->EAR[fieldSize - 1] == ' ' ||
                                                lpLyrics3Info->EAR[fieldSize - 1] == '\0') ; fieldSize --) {}
        ::ltoa(fieldSize, fieldSizeChar, 10);
        fieldFigure = ::strlen(fieldSizeChar);
        if ( fieldFigure < 5 ) target += (5 - fieldFigure);
        ::memcpy(&buf[target], fieldSizeChar, fieldFigure);
        target += fieldFigure;
        ::memcpy(&buf[target], lpLyrics3Info->EAR, fieldSize);
        target += fieldSize;

        ::memcpy(&buf[target], "ETT", 3);
        target += 3;
        for (fieldSize = 250; fieldSize > 0 && (lpLyrics3Info->ETT[fieldSize - 1] == ' ' ||
                                                lpLyrics3Info->ETT[fieldSize - 1] == '\0') ; fieldSize --) {}
        ::ltoa(fieldSize, fieldSizeChar, 10);
        fieldFigure = ::strlen(fieldSizeChar);
        if (fieldFigure > 0) target += (5 - fieldFigure);
        ::memcpy(&buf[target], fieldSizeChar, fieldFigure);
        target += fieldFigure;
        ::memcpy(&buf[target], lpLyrics3Info->ETT, fieldSize);
        target += fieldSize;

        ::memcpy(&buf[target], "IMG", 3);
        target += 3;
        for (fieldSize = 60000; fieldSize > 0 && (lpLyrics3Info->IMG[fieldSize - 1] == ' ' ||
                                                  lpLyrics3Info->IMG[fieldSize - 1] == '\0') ; fieldSize --) {}
        ::ltoa(fieldSize, fieldSizeChar, 10);
        fieldFigure = ::strlen(fieldSizeChar);
        if (fieldFigure > 0) target += (5 - fieldFigure);
        ::memcpy(&buf[target], fieldSizeChar, fieldFigure);
        target += fieldFigure;
        ::memcpy(&buf[target], lpLyrics3Info->IMG, fieldSize);
        target += fieldSize;

        ::ltoa(target, fieldSizeChar, 10);
        fieldFigure = ::strlen(fieldSizeChar);
        if (fieldFigure > 0) target += (6 - fieldFigure);
        ::memcpy(&buf[target], fieldSizeChar, fieldFigure);
        target += fieldFigure;

        ::memcpy(&buf[target], "LYRICS200", 9);
        target += 9;
    }

    if ( tagFlag == 1 ) {
        ::memcpy(&buf[target], tag, 128);
    } else {
        int Cnt;
        ::memcpy(&buf[target], "TAG", 3);
        for (Cnt = 3; Cnt < 127; Cnt++){
            ::memcpy(&buf[target+Cnt], " ", 1);
        }
        buf[target+127] = 0xFA;
    }
    target +=128;

    //LYRICS3
    ::SetFilePointer(hFileLylics3, -insPoint, NULL, FILE_END);
    result = ::WriteFile(hFileLylics3, &buf, target, &bytesWrite, NULL);
    if (!result || bytesWrite != target) {
        ::CloseHandle(hFileLylics3);
        _setError(ERR_MP3_FILE_WRITE);
        return FALSE;
    }
    ::SetEndOfFile(hFileLylics3);

    //SItB[hC
    if ( riffFlag == 1 ) {
        LIST_INFO_EX ListInfo;
        if ( !getListInfoEX(hFileLylics3, &ListInfo) ) {
            ::CloseHandle(hFileLylics3);
            return FALSE;
        }
        if ( !setListInfoEX(hFileLylics3, &ListInfo) ) {
            ::CloseHandle(hFileLylics3);
            return FALSE;
        }
    }

    ::CloseHandle(hFileLylics3);
    return TRUE;

}

BOOL input_getFileLyrics3Info(const char* pszName, const wchar_t* pszNameW, LYRICS3_INFO* lpLyrics3Info)
{
    HANDLE          hFileMpeg;              //Mp3t@C
    BOOL            result;

	// t@CI[v
	if (input_fileOpen(pszName, pszNameW,
		GENERIC_READ, FILE_SHARE_READ + FILE_SHARE_WRITE, OPEN_EXISTING, &hFileMpeg) == FALSE)
		return FALSE;

    result = input_getLyrics3Data(hFileMpeg, lpLyrics3Info);
    ::CloseHandle(hFileMpeg);

    return result;
}

BOOL input_getLyrics3Data(HANDLE FileHandle, LYRICS3_INFO* lpLyrics3Info)
{
    DWORD       bytesRead;
    BOOL        result;
    BYTE        tag[128];
    BYTE        id[3];
    char        fSizeData[6];
    int         fSize;
    BYTE*       lyBuf;
    DWORD       lysize = 0;         //TCY
    int         target = 0;
    int         tagFlag = 0;

    //
    ::memset(lpLyrics3Info, 0, sizeof(LYRICS3_INFO));

    //ID3Tag v1 `FbN
    SetFilePointer(FileHandle, -(long)128, NULL, FILE_END);
    result = ReadFile(FileHandle, tag, 128, &bytesRead, NULL);
    if (!result || bytesRead != 128) {
        _setError(ERR_MP3_FILE_READ);
        return FALSE;
    }
    if (memcmp(tag, "TAG", 3) == 0) {
        tagFlag = 1;
    }

    //LYRICS3݃`FbN擾
    result = chkLYRICS3(FileHandle, &lysize);
    if (!result || lysize == 0) {
        return FALSE;   //LYRICS3 f[^
    }
        
    //obt@m
    lyBuf = (BYTE*)::malloc(lysize);
    if (lyBuf == NULL){
        _setError(ERR_MALLOC);
        ::free(lyBuf);
        return FALSE;
    }

    //LYRICS3擪ʒuփ|C^ړ
    if ( tagFlag == 1 ){
        SetFilePointer(FileHandle, -(long)(128+lysize), NULL, FILE_END);
    } else {
        SetFilePointer(FileHandle, -(long)(lysize), NULL, FILE_END);
    }

    lysize -= 15;   //TCYLYRICS200̃oCg}CiX

    //LYRICS3 ǂݍ
    result = ReadFile(FileHandle, lyBuf, lysize, &bytesRead, NULL);
    if (!result || bytesRead != lysize) {
        _setError(ERR_MP3_FILE_READ);
        ::free(lyBuf);
        return FALSE;
    }

    //LYRICS3X^[gʒu`FbN
    if (memcmp(lyBuf, "LYRICSBEGIN", 11) != 0) {
        return FALSE;
    }

    target += 11;   //"LYRICSBEGIN" ړ

    //f[^
    do {
        ::memcpy(id, &lyBuf[target], 3);
        ::memcpy(fSizeData, &lyBuf[target+3], 5);
        fSizeData[5] = '\0';
        fSize = atoi(fSizeData);
        target += 8;
        if (::memcmp(id, "IND", 3) == 0) {
            if (fSize > 0) {
                lpLyrics3Info->IND_LYR = lyBuf[target];
            }
            if (fSize > 1) {
                lpLyrics3Info->IND_TIMETAG = lyBuf[target];
            }
        } else if (::memcmp(id, "LYR", 3) == 0) {
            if (fSize > 0)
                ::memcpy(lpLyrics3Info->LYR, &lyBuf[target], fSize);
        } else if (::memcmp(id, "INF", 3) == 0) {
            if (fSize > 0)
                ::memcpy(lpLyrics3Info->INF, &lyBuf[target], fSize);
        } else if (::memcmp(id, "AUT", 3) == 0) {
            if (fSize > 0)
                ::memcpy(lpLyrics3Info->AUT, &lyBuf[target], fSize);
        } else if (::memcmp(id, "EAL", 3) == 0) {
            if (fSize > 0)
                ::memcpy(lpLyrics3Info->EAL, &lyBuf[target], fSize);
        } else if (::memcmp(id, "EAR", 3) == 0) {
            if (fSize > 0)
                ::memcpy(lpLyrics3Info->EAR, &lyBuf[target], fSize);
        } else if (::memcmp(id, "ETT", 3) == 0) {
            if (fSize > 0)
                ::memcpy(lpLyrics3Info->ETT, &lyBuf[target], fSize);
        } else if (::memcmp(id, "IMG", 3) == 0) {
            if (fSize > 0)
                ::memcpy(lpLyrics3Info->IMG, &lyBuf[target], fSize);
        } else {
            //::free(lyBuf);
            //return FALSE;
        } 
        target += fSize;
    } while ( target < (int)lysize );

    ::free(lyBuf);
    return TRUE;

}

BOOL input_readLyrics3(HANDLE FileHandle)
{
    char        buf[1024];
    long        sec = 0;
    int         lyPoint;
    int         lySize;
    char*       pbuf;
    int         nextTagPoint;
    int         readDataSize;
    int         result;
    long        befTime = 0;
    BOOL        atTagFlag;

    LYRICS3_INFO Lyrics3Info;
    int lyricsSize = 0;
    int startPoint = 0;
    int lineSize = 0;
    int inputSize = 0;      //Max 128
    int target = 0;

/*
    if (m_state != STATE_STOP) {
        //~ȊO͎ss
        _setError(ERR_NOT_STATE_STOP);
        return FALSE;
    }
*/
    m_lyricsCount = 0;
    m_lyricsCurrent = 0;
    ::memset(m_lyrics, 0, sizeof(m_lyrics));

    //LYRICS3 f[^ǂݍ
    if (!(input_getLyrics3Data(FileHandle, &Lyrics3Info))) {
        return FALSE;
    }

    lyricsSize = ::strlen(Lyrics3Info.LYR);
    if (lyricsSize == 0) {
        return FALSE;
    }

    if ( m_KaraokeTagFlag ) {
        readDataSize = 1024;
    } else {
        readDataSize = 128;
    }

    //̎
    do {
        if (Lyrics3Info.LYR[target] == '\r' || Lyrics3Info.LYR[target] == '\n' || (target+1) == lyricsSize ) {
            //sQoCgڃ`FbN(t@Cł͍sȂ)
            if ((target+1) < lyricsSize) {
                if (Lyrics3Info.LYR[target+1] == '\r' || Lyrics3Info.LYR[target+1] == '\n'){
                    target++;
                }
            }
            target++;
            inputSize = lineSize;
            if ( inputSize > readDataSize ) inputSize = readDataSize;
            ::memset(buf, 0, sizeof(buf));
            ::memcpy(buf, &Lyrics3Info.LYR[startPoint], inputSize);

            buf[readDataSize - 1] = '\0';

            //^OΉ
            atTagFlag = FALSE;
            if ( buf[0] == '@' ) atTagFlag = checkAtTag(buf, readDataSize, &m_atTag, m_lyricsLineNo);   //^OǍ

            m_liricsLinePointMax[m_lyricsLineNo] = 0;
            result = input_getTimeLyrics(buf, &sec, &lyPoint, &lySize, &nextTagPoint);

            //̎sǂݍݗp^O
            m_lyrics[m_lyricsCount].tagFlag = TRUE;
            m_lyrics[m_lyricsCount].sec = befTime;
            ::memset(m_lyrics[m_lyricsCount].lyrics, 0, 128);
            m_lyrics[m_lyricsCount].textSize = 0;
            m_lyrics[m_lyricsCount].lineno = m_lyricsLineNo;
            m_lyrics[m_lyricsCount].point = 0;
            m_lyrics[m_lyricsCount].atTagFlag = FALSE;
            m_lyricsCount++;
            if ( m_lyricsCount == 1000 ){
                return TRUE;
            }

            if ( result == -1 ){
                //if ( m_lyricsCount > 0 && m_NotTagLineRead == 1 ) {
                    //Õ^C^OZbgiOs̑j
                    m_lyrics[m_lyricsCount-1].tagFlag = FALSE;
                    m_lyrics[m_lyricsCount].tagFlag = FALSE;
                    m_lyrics[m_lyricsCount].sec = befTime;
                    m_lyrics[m_lyricsCount].textSize = lySize;
                    ::memcpy(&m_lyrics[m_lyricsCount].lyrics, &buf[lyPoint], lySize);
                    m_lyrics[m_lyricsCount].lineno = m_lyricsLineNo;
                    m_liricsLinePointMax[m_lyricsLineNo]++;
                    m_lyrics[m_lyricsCount].point = m_liricsLinePointMax[m_lyricsLineNo];
                    m_lyrics[m_lyricsCount].atTagFlag = atTagFlag;

                    m_lyricsCount++;
                    m_lyricsLineNo++;
                    if ( m_lyricsCount == 1000 ){
                        return TRUE;
                    }
                //}
            } else {
                pbuf = buf;
                for(;;){
                    m_lyrics[m_lyricsCount].tagFlag = TRUE;
                    m_lyrics[m_lyricsCount].sec = sec;
                    befTime = sec;
                    m_lyrics[m_lyricsCount].textSize = lySize;
                    ::memcpy(&m_lyrics[m_lyricsCount].lyrics, &pbuf[lyPoint], lySize);
                    m_lyrics[m_lyricsCount].lineno = m_lyricsLineNo;
                    m_liricsLinePointMax[m_lyricsLineNo]++;
                    m_lyrics[m_lyricsCount].point = m_liricsLinePointMax[m_lyricsLineNo];
                    if ( m_lyrics[m_lyricsCount].point == 1 ) {
                        m_lyrics[m_lyricsCount-1].sec = sec;
                        if ( (sec - befTime) > m_lyricsNextAdjustTime + 100) m_lyrics[m_lyricsCount-1].sec -= m_lyricsNextAdjustTime;
                    }
                    m_lyrics[m_lyricsCount].atTagFlag = FALSE;

                    m_lyricsCount++;
                    if ( m_lyricsCount == 1000 ){
                        return TRUE;
                    }
                    if ( result == 0 ) {
                        m_lyricsLineNo++;
                        break;
                    }
                    pbuf += nextTagPoint;
                    result = input_getTimeLyrics(pbuf, &sec, &lyPoint, &lySize, &nextTagPoint);
                }
            }

            startPoint = target;
            lineSize = 0;
        } else {
            lineSize++;
            target++;
        }
    } while (target < lyricsSize);

    return TRUE;

}

int input_checkTimeTag(char* tagData, long* msec)
{
    char        mm[3];
    char        ss[3];
    char        ms[4];
    int         dataLength;
    int         target;
    int         lyricsPoint;

    ::memset(mm, 0, sizeof(mm));
    ::memset(ss, 0, sizeof(ss));
    lyricsPoint = 0;
    target = 0;

    if ( tagData[target] == '\0' ) {
        return -1;
    } else if ( tagData[target] != '[' ) {
        return 0;
    }

    //`FbN
    if ( tagData[target + 1] == '\0' ) {
        return -1;
    } else if ( tagData[target + 1] < 0x30 || tagData[target + 1] > 0x39 ) {
        return 0;
    } else if ( tagData[target + 2] == '\0' ) {
        return -1;
    } else if ( tagData[target + 2] == ':' ) {
        dataLength = 1;
    } else if ( tagData[target + 3] == '\0' ) {
        return -1;
    } else if ( tagData[target + 2] >= 0x30 && tagData[target + 2] <= 0x39 && tagData[target + 3] == ':') {
        dataLength = 2;
    } else {
        return 0;
    }
    ::memcpy(mm, &tagData[target + 1], dataLength);
    target = 1 + dataLength;

    //b`FbN
    if ( tagData[target + 1] == '\0' ) {
        return -1;
    } else if ( tagData[target + 1] < 0x30 || tagData[target + 1] > 0x39 ) {
        return 0;
    } else if ( tagData[target + 2] == '\0' ) {
        return -1;
    } else if ( tagData[target + 2] == ':' || tagData[target + 2] == ']') {
        dataLength = 1;
    } else if ( tagData[target + 3] == '\0' ) {
        return -1;
    } else if ( tagData[target + 2] >= 0x30 && tagData[target + 2] <= 0x39 
           && ( tagData[target + 3] == ':' || tagData[target + 3] == ']' )) {
        dataLength = 2;
    } else {
        return 0;
    }
    ::memcpy(ss, &tagData[target + 1], dataLength);
    *msec = (atoi(mm) * 60 + atoi(ss)) * 1000;

    target += 1 + dataLength;
    lyricsPoint = target + 1;

    //~b`FbN
    if ( tagData[target] == ':' ) {
        dataLength = 0;
        if ( tagData[target + 1] == '\0' ) {
            return -1;
        } else if ( tagData[target + 1] < 0x30 || tagData[target + 1] > 0x39 ) {
            return 0;
        } else if ( tagData[target + 2] == '\0' ) {
            return -1;
        } else if ( tagData[target + 2] == ']' ) {
            dataLength = 1;
            lyricsPoint = target + 3;
        } else if ( tagData[target + 2] < 0x30 || tagData[target + 2] > 0x39 ) {
            return 0;
        } else if ( tagData[target + 3] == '\0' ) {
            return -1;
        } else if ( tagData[target + 3] == ']' ) {
            dataLength = 2;
            lyricsPoint = target + 4;
        } else if ( tagData[target + 3] < 0x30 || tagData[target + 3] > 0x39 ) {
            return FALSE;
        } else if ( tagData[target + 4] == '\0' ) {
            return -1;
        } else if ( tagData[target + 4] == ']' ) {
            dataLength = 3;
            lyricsPoint = target + 5;
        } else {
            return 0;
        }
        
        ms[0] = '0';
        ms[1] = '0';
        ms[2] = '0';
        ms[3] = '\0';
        ::memcpy(ms, &tagData[target + 1], dataLength);
        *msec = *msec + atoi(ms);
    }
    return lyricsPoint;
}

int input_getTimeLyrics(char* tagData, long* msec, int* lyPoint, int* lyLen, int* nextPoint)
{
    int         target;
    int         result;
    long        sec;
    char*       lyrics;

    target = 0;
    *msec = 0;
    *lyLen = 0;
    *lyPoint = 0;
    *nextPoint = 0;

    if ( tagData[target] == '\0' ) {
        return -1;
    }
    
    //^C^O`FbN
    result = input_checkTimeTag(&tagData[target], &sec);
    if ( result <= 0 ) {
        *msec = -1;
        for(;;){
            if ( tagData[target] == '\0' || tagData[target] == '\r' || tagData[target] == '\n'  ){
                *lyLen = target;
                return -1;
            } else if ( tagData[target] == '[' && m_KaraokeTagFlag ) {
                result = input_checkTimeTag(&tagData[target], &sec);
                if ( result > 0 ) {
                    m_ExistKaraokeTag = TRUE;
                    *lyLen = target;
                    *nextPoint = target;
                    return 1;
                }
            }
            target++;
        }
    } else {
        m_ExistTimeTag = TRUE;
        target = 0;
        *lyPoint = result;
        *nextPoint = result;
        lyrics = tagData + result;
        *msec = sec;
        for(;;){
            //if ( lyrics[target] == '\0' ){
            if ( lyrics[target] == '\0' || lyrics[target] == '\r' || lyrics[target] == '\n' ){
                *lyLen = target;
                return 0;
            } else if ( lyrics[target] == '['  && m_KaraokeTagFlag ) {
                result = input_checkTimeTag(&lyrics[target], &sec);
                if ( result > 0 ) {
                    m_ExistKaraokeTag = TRUE;
                    *lyLen = target;
                    *nextPoint += target;
                    return 1;
                }
            }
            target++;
        }
    }
    return -1;
}

BOOL input_getLyrics(LYRICS_INFO* pLyricsInfo)
{
    LYRICS_INFO2 pLyricsInfo2;
    int point;

    ::memset(pLyricsInfo, 0, sizeof(LYRICS_INFO));

    if (m_lyricsCount == 0 || m_decInfo.frequency == 0) {
        _setError(ERR_LYRICS_NON_DATA);
        return FALSE;
    }

    //̎擾
    if ( !input_getLyrics2(&pLyricsInfo2, &point) ){
        return FALSE;
    }

    ::memcpy(pLyricsInfo->LyricsNext2, pLyricsInfo2.LyricsNext2, strlen(pLyricsInfo2.LyricsNext2));
    ::memcpy(pLyricsInfo->LyricsNext1, pLyricsInfo2.LyricsNext1, strlen(pLyricsInfo2.LyricsNext1));
    ::memcpy(pLyricsInfo->LyricsCurrent, pLyricsInfo2.LyricsCurrentAll, strlen(pLyricsInfo2.LyricsCurrent));
    ::memcpy(pLyricsInfo->LyricsPrev1, pLyricsInfo2.LyricsPrev1, strlen(pLyricsInfo2.LyricsPrev1));
    ::memcpy(pLyricsInfo->LyricsPrev2, pLyricsInfo2.LyricsPrev2, strlen(pLyricsInfo2.LyricsPrev2));
    pLyricsInfo->sec = pLyricsInfo2.sec;
    return TRUE;

/*
    //{tΉ
    long playMs;
    long playSamples;
    long lyricsAdjustTime;

    //JgTv擾
    playSamples = output_getCurrentSample();

    long samples = playSamples + m_firstSample;
    //ĐԎ擾
    //^OΉ
    if (m_atTagUseFlag == 1 && m_atTag.existFlag){
        if ( m_atTag.lyricsTotalFlag == 0 ){
            playMs = getWinampMs(m_decInfo.frequency, samples);
        }
        else if ( m_atTag.lyricsTotalFlag == 2 ){
            playMs = getFreeRateMs(m_atTag.TimeRatio, samples);
        } 
        else {
            playMs = input_sampleToMs(samples);
        }
        lyricsAdjustTime = m_atTag.Offset;
    } 
    else {
        if (m_lyricsTotalFlag == 0){
            playMs = getWinampMs(m_decInfo.frequency, samples);
        } 
        else if(m_lyricsTotalFlag == 2){
            playMs = getFreeRateMs(m_timeRatio, samples);
        }
        else {
            playMs = input_sampleToMs(samples);
        }
        lyricsAdjustTime = m_lyricsAdjustTime;
    }

    //Ԓ(m_lyricsAdjustTime[ms])
    playMs += lyricsAdjustTime;

    //{Đp
    playMs = (long)((double)playMs * m_playSpeed);

    //̎Ή
    int     lyCnt;
    if ( m_lyricsCount > 0 ){
        for (lyCnt = 0; lyCnt < m_lyricsCount; lyCnt++){
            //sw
            if ( m_lyrics[lyCnt].tagFlag == FALSE && m_NotTagLineRead == 0 ) continue;

            //^O\
            if ( m_lyrics[lyCnt].atTagFlag && m_atTagUseFlag != 0 ) continue;

            if ( (m_lyrics[lyCnt].sec) > playMs ){
                m_lyricsCurrent = lyCnt - 1;

                //sJbg
                while ( !m_lyrics[m_lyricsCurrent].tagFlag ) {
                    m_lyricsCurrent--;
                    if ( m_lyricsCurrent < 0 ) {
                        break;
                    }
                }

                 if (m_lyrics[m_lyricsCount -1].sec <= m_playSec) m_lyricsCurrent = m_lyricsCount -1;
                 if ( m_lyricsCurrent < 0 ) m_lyricsCurrent = 0;
                break;
             }
         }
         if ( lyCnt == m_lyricsCount ) m_lyricsCurrent = m_lyricsCount -1;
    }

    if ( m_lyricsCurrent < m_lyricsCount ) {
        if (m_lyricsCurrent < m_lyricsCount - 2)
            ::memcpy(pLyricsInfo->LyricsNext2, m_lyrics[m_lyricsCurrent+2].lyrics,  strlen(m_lyrics[m_lyricsCurrent+2].lyrics));
        if (m_lyricsCurrent < m_lyricsCount - 1)
            ::memcpy(pLyricsInfo->LyricsNext1, m_lyrics[m_lyricsCurrent+1].lyrics,  strlen(m_lyrics[m_lyricsCurrent+1].lyrics));
        ::memcpy(pLyricsInfo->LyricsCurrent, m_lyrics[m_lyricsCurrent].lyrics,  strlen(m_lyrics[m_lyricsCurrent].lyrics));
        if (m_lyricsCurrent > 0)
            ::memcpy(pLyricsInfo->LyricsPrev1, m_lyrics[m_lyricsCurrent-1].lyrics,  strlen(m_lyrics[m_lyricsCurrent-1].lyrics));
        if (m_lyricsCurrent > 1)
            ::memcpy(pLyricsInfo->LyricsPrev2, m_lyrics[m_lyricsCurrent-2].lyrics,  strlen(m_lyrics[m_lyricsCurrent-2].lyrics));
        pLyricsInfo->sec = m_lyrics[m_lyricsCurrent].sec;
    } else {
        _setError(ERR_LYRICS_NON_DATA);
        return FALSE;
    }
    return TRUE;
    */
}

BOOL input_checkKaraokeTag()
{
    return m_ExistKaraokeTag;
}

BOOL input_getLyrics2(LYRICS_INFO2* pLyricsInfo, int* pLyCnt)
{
    *pLyCnt = -1;
    ::memset(pLyricsInfo, 0, sizeof(LYRICS_INFO2));

    if (m_lyricsCount == 0 || m_decInfo.frequency == 0) {
        _setError(ERR_LYRICS_NON_DATA);
        return FALSE;
    }

    long playMs;
    long playSamples;
    long lyricsAdjustTime;

    //JgTv擾
    playSamples = output_getCurrentSample();

    long samples = playSamples + m_firstSample;
    //ĐԎ擾
    //^OΉ
    if (m_atTagUseFlag == 1 && m_atTag.existFlag){
        if ( m_atTag.lyricsTotalFlag == 0 ){
            playMs = getWinampMs(m_decInfo.frequency, samples);
        }
        else if ( m_atTag.lyricsTotalFlag == 2 ){
            playMs = getFreeRateMs(m_atTag.TimeRatio, samples);
        } 
        else {
            playMs = input_sampleToMs(samples);
        }
        lyricsAdjustTime = m_atTag.Offset;
    } 
    else {
        if (m_lyricsTotalFlag == 0){
            playMs = getWinampMs(m_decInfo.frequency, samples);
        } 
        else if(m_lyricsTotalFlag == 2){
            playMs = getFreeRateMs(m_timeRatio, samples);
        }
        else {
            playMs = input_sampleToMs(samples);
        }
        lyricsAdjustTime = m_lyricsAdjustTime;
    }

    //Ԓ(m_lyricsAdjustTime[ms])
    //playMs += lyricsAdjustTime;
    playMs -= lyricsAdjustTime;//C by Kobarin
    if((int)playMs < 0)playMs = 0;//ǉ by Kobarin

    //{Đp
    playMs = (long)((double)playMs * m_playSpeed);

    //̎Ή
    int     lyCnt;                          //̎ʒu
    int     curLine;                        //Jgs
    int     curPoint;                       //Jgʒu(JIP^O)
    long    curSec;                         //Jg̎
    long    length = 0;                     //^Ỏt
    long    lineTime = 0;                   //s^O
    int     noSelected;                     //1:JgsI̎

    //
    char    bufNext2[128] = "\0";
    char    bufNext1[128] = "\0";
    char    bufCurrent[128] = "\0";
    char    bufCurrentBegin[128] = "\0";
    char    bufCurrentLyrics[128] = "\0";
    char    bufCurrentAll[128] = "\0";
    char    bufPrev1[128] = "\0";
    char    bufPrev2[128] = "\0";

    //Jg̎s擾
    if ( m_lyricsCount > 0 ){
        for (lyCnt = 0; lyCnt < m_lyricsCount; lyCnt++){
            //sw
            if ( m_lyrics[lyCnt].tagFlag == FALSE && m_NotTagLineRead == 0 ) continue;

            if ( m_lyricsCurrent == 0 && !m_lyrics[lyCnt].tagFlag ) continue;

            //^O\
            if ( m_lyrics[lyCnt].atTagFlag && m_atTagUseFlag != 0 ) continue;

            //m_atTagUseFlag
            //^OԎ擾
            lineTime = m_lyrics[lyCnt].sec;

/*
            noSelected = 0;
            //̎s̐sǂ݂
            if ( lyCnt > 0 && m_lyrics[lyCnt].point == 1 && m_lyrics[lyCnt].tagFlag ) {
                if ( m_lyrics[lyCnt-1].sec < (lineTime - m_lyricsNextAdjustTime) ) lineTime -= m_lyricsNextAdjustTime;
            }
*/

            //Jgs`FbN(ĐԂ^O̎Ԃ傫ȂPÕ^OZbg)
            //if ( (m_lyrics[lyCnt].sec) > ((float)playMs) * (float)((float)lyricsTotalTime/(float)m_totalSec) * 1000){
            //if ( lineTime > playMs * (double)((double)lyricsTotalTime/(double)m_totalMs)){
            if ( lineTime > playMs){
                //JgsZbg
                m_lyricsCurrent = lyCnt - 1;
                //^Os̏ꍇO^OɕύX
                while ( !m_lyrics[m_lyricsCurrent].tagFlag ) {
                    m_lyricsCurrent--;
                    //Jgs 0 
                    if ( m_lyricsCurrent < 0 ) {
                        break;
                    }
                }

                //̎s̕bŏI^OȉȂJgs -1
                if ( m_lyrics[m_lyricsCount -1].sec <= m_playSec ) m_lyricsCurrent = m_lyricsCount -1;

                //Jgs 0 
                if ( m_lyricsCurrent < 0 ) m_lyricsCurrent = 0;

                //I𖳂H
                if (m_lyrics[m_lyricsCurrent].sec > playMs && m_lyrics[m_lyricsCurrent].point == 1) noSelected = 1;

                break;
            }
        }
        if ( lyCnt == m_lyricsCount ) m_lyricsCurrent = m_lyricsCount -1;
    }
    curLine = m_lyrics[m_lyricsCurrent].lineno;
    curPoint = m_lyrics[m_lyricsCurrent].point;
    curSec = m_lyrics[m_lyricsCurrent].sec;
    *pLyCnt = m_lyricsCurrent;

    strcat(bufCurrent, m_lyrics[m_lyricsCurrent].lyrics );
    for(lyCnt = 0; lyCnt < m_lyricsCount; lyCnt++){
        //sw
        if ( m_lyrics[lyCnt].tagFlag == FALSE && m_NotTagLineRead == 0 ) continue;

        //^O\
        if ( m_lyrics[lyCnt].atTagFlag && m_atTagUseFlag != 0 ) continue;

        if ( m_lyrics[lyCnt].sec > curSec && length == 0 ){
            length = m_lyrics[lyCnt].sec - curSec;
        }

        if ( m_lyrics[lyCnt].lineno == curLine - 2 ){
            strcat( bufPrev2, m_lyrics[lyCnt].lyrics );
        } else
        if ( m_lyrics[lyCnt].lineno == curLine - 1 ){
            strcat( bufPrev1, m_lyrics[lyCnt].lyrics );
        } else
        if ( m_lyrics[lyCnt].lineno == curLine ){
            strcat( bufCurrentAll, m_lyrics[lyCnt].lyrics );
            //if ( noSelected == 0 ) {
                if ( m_lyrics[lyCnt].point <= curPoint - 1 ){
                    strcat( bufCurrentBegin, m_lyrics[lyCnt].lyrics );
                }
                if ( m_lyrics[lyCnt].point <= curPoint ){
                    strcat( bufCurrentLyrics, m_lyrics[lyCnt].lyrics );
                }
            //}
        } else
        if ( m_lyrics[lyCnt].lineno == curLine + 1 ){
            strcat( bufNext1, m_lyrics[lyCnt].lyrics );
        } else
        if ( m_lyrics[lyCnt].lineno == curLine + 2 ){
            strcat( bufNext2, m_lyrics[lyCnt].lyrics );
        }
    }

    if ( m_lyricsCurrent < m_lyricsCount ) {
        ::memcpy(pLyricsInfo->LyricsNext2, bufNext2,  strlen(bufNext2));
        ::memcpy(pLyricsInfo->LyricsNext1, bufNext1,  strlen(bufNext1));
        ::memcpy(pLyricsInfo->LyricsCurrent, bufCurrent,  strlen(bufCurrent));
        ::memcpy(pLyricsInfo->LyricsCurrentBegin, bufCurrentBegin,  strlen(bufCurrentBegin));
        ::memcpy(pLyricsInfo->LyricsCurrentLyrics, bufCurrentLyrics,  strlen(bufCurrentLyrics));
        ::memcpy(pLyricsInfo->LyricsCurrentAll, bufCurrentAll,  strlen(bufCurrentAll));
        ::memcpy(pLyricsInfo->LyricsPrev1, bufPrev1,  strlen(bufPrev1));
        ::memcpy(pLyricsInfo->LyricsPrev2, bufPrev2,  strlen(bufPrev2));
        pLyricsInfo->sec = m_lyrics[m_lyricsCurrent].sec;
        pLyricsInfo->lineno = curLine;
        pLyricsInfo->point = curPoint;
        pLyricsInfo->length = length;
    } else {
        _setError(ERR_LYRICS_NON_DATA);
        return FALSE;
    }

    return TRUE;
}

BOOL getListInfoEX(HANDLE FileHandle, LIST_INFO_EX* lplistInfo)
{
    RiffChunk   riff;
    CommonChunk common;
    BOOL        result;
    DWORD       bytesRead;

    //
    ::memset(lplistInfo, 0, sizeof(LIST_INFO_EX));

    //t@C̓ɖ߂ARIFF`Nǂݍ
    ::SetFilePointer(FileHandle, 0, NULL, FILE_BEGIN);
    result = ::ReadFile(FileHandle, &riff, sizeof(riff), &bytesRead, NULL);
    if (!result || bytesRead != sizeof(riff)) {
        return FALSE;
    }
    if (::memcmp(riff.chunkID, "RIFF", 4) != 0) {
        return FALSE;
    }

    //data`NT
    for (;;) {
        result = ::ReadFile(FileHandle, &common, sizeof(common), &bytesRead, NULL);
        if (!result || bytesRead != sizeof(common)) {
            return FALSE;
        }

        if (::memcmp(common.chunkID, "fmt ", 4) == 0) {
            //MpegȊÕtH[}bg`FbN
            ::ReadFile(FileHandle, &m_waveFormat, sizeof(WAVEFORMATEX), &bytesRead, NULL);
            if (!result || bytesRead != sizeof(WAVEFORMATEX)) {
                return FALSE;
            }
            //tH[}bgTag`FbN
            if (m_waveFormat.wFormatTag != 85 && m_waveFormat.wFormatTag != 80) {
                //̑̌`
                return FALSE; 
            }
            ::SetFilePointer(FileHandle, -(long)sizeof(WAVEFORMATEX), NULL, FILE_CURRENT);
        }
        
        if (::memcmp(common.chunkID, "data", 4) == 0) {
            //data`N
            break;
        }
        else {
            //̃`N܂œǂݔ΂
            ::SetFilePointer(FileHandle, common.chunkSize, NULL, FILE_CURRENT);
        }
    }

    //RIFF MP3 LIST`NT
    if ( (common.chunkSize % 2) == 1 ) common.chunkSize++;
    if (::SetFilePointer(FileHandle, common.chunkSize, NULL, FILE_CURRENT) == 0xFFFFFFFF && GetLastError() != NO_ERROR ){
        return TRUE;
    };
    for (;;) {
        result = ::ReadFile(FileHandle, &common, sizeof(common), &bytesRead, NULL);
        if (!result || bytesRead != sizeof(common)) {
            return TRUE;
        }
        if (memcmp(common.chunkID, "LIST", 4) == 0) {
            //LIST`N
            break;
        }
        else {
            //̃`N܂œǂݔ΂
            ::SetFilePointer(FileHandle, common.chunkSize, NULL, FILE_CURRENT);
        }
    }
    BYTE* pRMPList = (BYTE*)::malloc(common.chunkSize);
    if (pRMPList == NULL) {
        return TRUE;
    }
    BYTE listID[4];
    result = ::ReadFile(FileHandle, listID, 4, &bytesRead, NULL);
    if (!result || bytesRead != 4) {
        ::free(pRMPList);
        return TRUE;
    }
    if (::memcmp(listID, "INFO", 4) != 0) {
        //INFOȊO͖
        ::free(pRMPList);
        return TRUE;
    }
    RMPInfo* pRMPInfo = (RMPInfo*)::malloc(sizeof(RMPInfo));
    if (pRMPInfo == NULL) {
        ::free(pRMPList);
        return TRUE;
    }
    DWORD remains = common.chunkSize - 4;
    DWORD baseSize = sizeof(RMPInfo) - sizeof(pRMPInfo->data);
    int dumy;
    while (remains >= sizeof(RMPInfo)) {
        result = ::ReadFile(FileHandle, pRMPInfo, baseSize, &bytesRead, NULL);
        if (!result || bytesRead != baseSize) {
            break;
        }
        dumy = 0;
        if ((pRMPInfo->length % 2) == 1 ) dumy = 1;

        pRMPInfo = (RMPInfo*)::realloc(pRMPInfo, baseSize + pRMPInfo->length + dumy);
        if (pRMPInfo == NULL) {
            break;
        }
        result = ::ReadFile(FileHandle, pRMPInfo->data, pRMPInfo->length + dumy, &bytesRead, NULL);
        if (!result || bytesRead != pRMPInfo->length + dumy) {
            break;
        }
        loadRMPInfoEX(lplistInfo, pRMPInfo);
    
        remains -= baseSize + pRMPInfo->length + dumy;
    }
    ::free(pRMPList);
    ::free(pRMPInfo);
    return TRUE;
}

void loadRMPInfoEX(LIST_INFO_EX* lplistInfo, RMPInfo* pRMPInfo)
{
    if (CHECK("INAM")) {        //track name
        SET(lplistInfo->INAM, LIST_SIZE);
    }
    else if (CHECK("IART")) {   //artist name
        SET(lplistInfo->IART, LIST_SIZE);
    }
    else if (CHECK("IPRD")) {
        SET(lplistInfo->IPRD, LIST_SIZE);
    }
    else if (CHECK("ICMT")) {
        SET(lplistInfo->ICMT, LIST_SIZE);
    }
    else if (CHECK("ICRD")) {
        SET(lplistInfo->ICRD, LIST_SIZE);
    }
    else if (CHECK("IGNR")) {
        SET(lplistInfo->IGNR, LIST_SIZE);
    }
    else if (CHECK("ICOP")) {
        SET(lplistInfo->ICOP, LIST_SIZE);
    }
    else if (CHECK("IENG")) {
        SET(lplistInfo->IENG, LIST_SIZE);
    }
    else if (CHECK("ISRC")) {
        SET(lplistInfo->ISRC, LIST_SIZE);
    }
    else if (CHECK("ISFT")) {
        SET(lplistInfo->ISFT, LIST_SIZE);
    }
    else if (CHECK("IKEY")) {
        SET(lplistInfo->IKEY, LIST_SIZE);
    }
    else if (CHECK("ITCH")) {
        SET(lplistInfo->ITCH, LIST_SIZE);
    }
    else if (CHECK("ICMS")) {
        SET(lplistInfo->ICMS, LIST_SIZE);
    }
    else if (CHECK("IMED")) {
        SET(lplistInfo->IMED, LIST_SIZE);
    }
    else if (CHECK("ISBJ")) {
        SET(lplistInfo->ISBJ, LIST_SIZE);
    }
    else if (CHECK("IMP3")) {
        SET(lplistInfo->IMP3, LIST_SIZE);
    }
    else if (CHECK("ILYC")) {
        SET(lplistInfo->ILYC, LIST_SIZE);
    }
}

//ǉ by Kobarin
//IɃJNX
class CAutoFreeBuf
{
    BYTE *m_pbuf;
    int  m_size;
    CAutoFreeBuf(void):m_size(0){m_pbuf = NULL;}
public:
    BYTE *buf(void){return m_pbuf;}
    int   size(void){return m_size;}
    CAutoFreeBuf(int size):m_size(size){m_pbuf = new BYTE[size];ZeroMemory(m_pbuf, size);}
    ~CAutoFreeBuf(void){delete[] m_pbuf;}
};

BOOL setListInfoEX(HANDLE FileHandle, LIST_INFO_EX* pListInfo)
{
    //BYTE            buf[400000];
    //C by Kobarin(X^bNj
    CAutoFreeBuf    autobuf(400000);
    BYTE *buf     = autobuf.buf();
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    DWORD           fSync;                  //wb_̈ʒu
    DWORD           fLast;                  //f[^`N̍Ō
    DWORD           fSize;
    DWORD           dSize;
    DWORD           listSize;
    DWORD           target = 0;
    BYTE            tag[128];
    int             tagFlag = 0;
    RiffChunk       riff;
    CommonChunk     common;
    int             lyFlag = 0;
    BYTE*           lyBuf;

    fSync = 0;
    dSize = GetFileSize(FileHandle, NULL);

    //RIFF`AID3 Tag v2`FbN
    SetFilePointer(FileHandle, 0, NULL, FILE_BEGIN);
    result = ReadFile(FileHandle, buf, 4, &bytesRead, NULL);
    if (memcmp(buf, "RIFF", 4) != 0) {
        ::CloseHandle(FileHandle);
        _setError(ERR_NON_RIFF);
        return FALSE;
    } else {
        //t@C̓ɖ߂ARIFF`Nǂݍ
        ::SetFilePointer(FileHandle, 0, NULL, FILE_BEGIN);
        result = ::ReadFile(FileHandle, &riff, sizeof(riff), &bytesRead, NULL);
        if (!result || bytesRead != sizeof(riff)) {
            ::CloseHandle(FileHandle);
            _setError(ERR_MP3_FILE_READ);
            return FALSE;
        }
        fSync = sizeof(riff);
        //data`NT
        for (;;) {
            result = ::ReadFile(FileHandle, &common, sizeof(common), &bytesRead, NULL);
            if (!result || bytesRead != sizeof(common)) {
                _setError(ERR_MP3_FILE_READ);
                ::CloseHandle(FileHandle);
                return FALSE;
            }

            if (::memcmp(common.chunkID, "data", 4) == 0) {
                //data`N
                break;
            }
            else {
                //̃`N܂œǂݔ΂
                fSync = ::SetFilePointer(FileHandle, common.chunkSize, NULL, FILE_CURRENT);
            }
        }

        //`NTCY`FbN
        if (common.chunkSize < dSize) {
            dSize = common.chunkSize;
        }

        fSync += bytesRead;
        fLast = fSync + dSize;
    }

    //ID3Tag v1`FbN
    SetFilePointer(FileHandle, -(long)128, NULL, FILE_END);
    result = ReadFile(FileHandle, tag, 128, &bytesRead, NULL);
    if (!result || bytesRead != 128) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(FileHandle);
        return FALSE;
    }
    if (memcmp(tag, "TAG", 3) == 0) {
        tagFlag = 1;
    }

    //LYRICS3݃`FbN
    DWORD lysize = 0;           //TCY
    DWORD lyfsize;          //tB[hTCY
    DWORD lysp = 0;         //X^[gʒu
    if (chkLYRICS3(FileHandle, &lysize)) {
        lyFlag = 1;
        lyfsize = lysize;

        //LYRICS3 `FbN
        if ((lysize % 2) == 1) {
            lyfsize++;
            lysp = 1;
            lyBuf = (BYTE*)::malloc(lyfsize);
            lyBuf[0] = 0x00;
        } else {
            lyBuf = (BYTE*)::malloc(lysize);
        }

        if (lyBuf == NULL){
            _setError(ERR_MALLOC);
            ::CloseHandle(FileHandle);
            ::free(lyBuf);
            return FALSE;
        }

        //|C^ړ
        if ( tagFlag == 1 ){
            SetFilePointer(FileHandle, -(long)(128+lysize), NULL, FILE_END);
        } else {
            SetFilePointer(FileHandle, -(long)(lysize), NULL, FILE_END);
        }

        result = ReadFile(FileHandle, &lyBuf[lysp], lysize, &bytesRead, NULL);
        if (!result || bytesRead != lysize) {
            _setError(ERR_MP3_FILE_READ);
            ::CloseHandle(FileHandle);
            ::free(lyBuf);
            return FALSE;
        }
        lysize = lyfsize;
    }

    //f[^TCY΍
    if ( (dSize % 2) == 1 ){
        buf[0] = 0x00;
        SetFilePointer(FileHandle, fLast, NULL, FILE_BEGIN);
        //result = ::WriteFile(FileHandle, &buf, 1, &bytesWrite, NULL);
        result = ::WriteFile(FileHandle, buf, 1, &bytesWrite, NULL);
        //C by Kobarin
        if (!result || bytesWrite != 1) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(FileHandle);
            if ( lyFlag == 1 ) ::free(lyBuf);
            return FALSE;
        }
        fLast++;
    }

    //::memset(&buf, 0, sizeof(buf));
    ::memset(buf, 0, autobuf.size());
    //C by Kobarin
    target = 12;
    listSize = ::strlen(pListInfo->INAM) + 1;
    for (listSize = LIST_SIZE; listSize > 0 && 
        (pListInfo->INAM[listSize - 1] == ' ' || 
         pListInfo->INAM[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "INAM", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->INAM, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 && 
        (pListInfo->IART[listSize - 1] == ' ' ||
         pListInfo->IART[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IART", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IART, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->IPRD[listSize - 1] == ' ' ||
         pListInfo->IPRD[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IPRD", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IPRD, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->ICMT[listSize - 1] == ' ' ||
         pListInfo->ICMT[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ICMT", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ICMT, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->IGNR[listSize - 1] == ' ' ||
         pListInfo->IGNR[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IGNR", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IGNR, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->ICRD[listSize - 1] == ' ' ||
         pListInfo->ICRD[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ICRD", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ICRD, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->ICOP[listSize - 1] == ' ' ||
         pListInfo->ICOP[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ICOP", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ICOP, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->IENG[listSize - 1] == ' ' ||
         pListInfo->IENG[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IENG", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IENG, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->ISRC[listSize - 1] == ' ' ||
         pListInfo->ISRC[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ISRC", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ISRC, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->ISFT[listSize - 1] == ' ' ||
         pListInfo->ISFT[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ISFT", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ISFT, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->IKEY[listSize - 1] == ' ' ||
         pListInfo->IKEY[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IKEY", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IKEY, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->ITCH[listSize - 1] == ' ' ||
         pListInfo->ITCH[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ITCH", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ITCH, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->ICMS[listSize - 1] == ' ' ||
         pListInfo->ICMS[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ICMS", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ICMS, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->IMED[listSize - 1] == ' ' ||
         pListInfo->IMED[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IMED", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IMED, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->ISBJ[listSize - 1] == ' ' ||
         pListInfo->ISBJ[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "ISBJ", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ISBJ, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->IMP3[listSize - 1] == ' ' ||
         pListInfo->IMP3[listSize - 1] == '\0'); listSize --) {}
    if ( !(listSize == 0 && m_NotDataSiWrite == 0) ){
        listSize++;
        ::memcpy(&buf[target], "IMP3", 4);
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->IMP3, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;
    }

    for (listSize = LIST_SIZE; listSize > 0 &&
        (pListInfo->ILYC[listSize - 1] == ' ' ||
         pListInfo->ILYC[listSize - 1] == '\0'); listSize --) {}
    listSize++;
    ::memcpy(&buf[target], "ILYC", 4);

    if ( lyFlag == 0 || m_Lyrics3InsertPoint == 2 ) {
        buf[target + 4] = (BYTE)(listSize & 0x000000ff);
        buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&buf[target + 8], pListInfo->ILYC, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;

        ::memcpy(&buf[target], "IID3", 4);
        buf[target + 4] = 0x80;

        target += 8;

        if ( tagFlag == 0 ) {
            int Cnt;
            ::memcpy(&buf[target], "TAG", 3);
            for (Cnt = 3; Cnt < 127; Cnt++){
                ::memcpy(&buf[target+Cnt], " ", 1);
            }
            buf[target+127] = 0xFA;
        } else {
            ::memcpy(&buf[target], tag, 128);
        }

        target += 128;
    } else {
        if ( m_Lyrics3InsertPoint == 1 ) {
            buf[target + 4] = (BYTE)(listSize & 0x000000ff);
            buf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
            buf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
            buf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
            ::memcpy(&buf[target + 8], pListInfo->ILYC, listSize - 1);
            target += 8 + listSize;
            if ((listSize % 2) == 1 ) target += 1;

            ::memcpy(&buf[target], "IID3", 4);
        }

        buf[target + 4] = (BYTE)((lysize + 128) & 0x000000ff);
        buf[target + 5] = (BYTE)(((lysize + 128) & 0x0000ffff) >> 8);
        buf[target + 6] = (BYTE)(((lysize + 128) & 0x00ffffff) >> 16);
        buf[target + 7] = (BYTE)(((lysize + 128) & 0xffffffff) >> 24);

        target += 8 + lysize + 128;
    }

    ::memcpy(&buf[0], "LIST", 4);
    buf[4] = (BYTE)((target - 8) & 0x000000ff);
    buf[5] = (BYTE)(((target - 8) & 0x0000ffff) >> 8);
    buf[6] = (BYTE)(((target - 8) & 0x00ffffff) >> 16);
    buf[7] = (BYTE)(((target - 8) & 0xffffffff) >> 24);
    ::memcpy(&buf[8], "INFO", 4);

    SetFilePointer(FileHandle, fLast, NULL, FILE_BEGIN);
    //result = ::WriteFile(FileHandle, &buf, target, &bytesWrite, NULL);
    result = ::WriteFile(FileHandle, buf, target, &bytesWrite, NULL);
    //C by Kobarin
    if (!result || bytesWrite != target) {
        _setError(ERR_MP3_FILE_WRITE);
        ::CloseHandle(FileHandle);
        if ( lyFlag == 1 ) ::free(lyBuf);
        return FALSE;
    }
    ::SetEndOfFile(FileHandle);

    if ( lyFlag != 0 && m_Lyrics3InsertPoint != 2 ) {
        SetFilePointer(FileHandle, -(long)(lysize + 128), NULL, FILE_END);
        result = ::WriteFile(FileHandle, lyBuf, lysize, &bytesWrite, NULL);
        if (!result || bytesWrite != lysize) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(FileHandle);
            if ( lyFlag == 1 ) ::free(lyBuf);
            return FALSE;
        }

        if ( tagFlag == 0 ) {
            int Cnt;
            ::memcpy(&buf[0], "TAG", 3);
            for (Cnt = 3; Cnt < 127; Cnt++){
                ::memcpy(&buf[target+Cnt], " ", 1);
            }
            buf[127] = 0xFA;
        } else {
            ::memcpy(&buf[0], tag, 128);
        }

        //result = ::WriteFile(FileHandle, &buf, 128, &bytesWrite, NULL);
        result = ::WriteFile(FileHandle, buf, 128, &bytesWrite, NULL);
        //C by Kobarin
        if (!result || bytesWrite != 128) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(FileHandle);
            if ( lyFlag == 1 ) ::free(lyBuf);
            return FALSE;
        }

        ::SetEndOfFile(FileHandle);
    }

    fSize = fLast + target;

    //::memset(&buf, 0, sizeof(buf));
    ::memset(buf, 0, autobuf.size());
    //C by Kobarin
    DWORD dw1 = (DWORD)buf;
    DWORD dw2 = (DWORD)(&buf);
    if(dw1 == dw2){
        int c = 0;
    }else{
        int d = 0;
    }
    buf[0] = (BYTE)((fSize - 8) & 0x000000ff);
    buf[1] = (BYTE)(((fSize - 8) & 0x0000ffff) >> 8);
    buf[2] = (BYTE)(((fSize - 8) & 0x00ffffff) >> 16);
    buf[3] = (BYTE)(((fSize - 8) & 0xffffffff) >> 24);

    SetFilePointer(FileHandle, 4, NULL, FILE_BEGIN);
    //result = ::WriteFile(FileHandle, &buf, 4, &bytesWrite, NULL);
    result = ::WriteFile(FileHandle, buf, 4, &bytesWrite, NULL);
    //C by Kobarin (&buf => buf)
    if (!result || bytesWrite != 4) {
        _setError(ERR_MP3_FILE_WRITE);
        ::CloseHandle(FileHandle);
        if ( lyFlag == 1 ) ::free(lyBuf);
        return FALSE;
    }

    ::CloseHandle(FileHandle);
    if ( lyFlag == 1 ) ::free(lyBuf);
    return TRUE;

}

BOOL setListInfoEX2(HANDLE FileHandle, LIST_INFO_EX2* pListInfo)
{
    BYTE            *pbuf;
    long            bufSize = 0;
    BOOL            result;
    DWORD           bytesRead;
    DWORD           bytesWrite;
    DWORD           fSync;                  //wb_̈ʒu
    DWORD           fLast;                  //f[^`N̍Ō
    DWORD           fSize;
    DWORD           dSize;
    DWORD           listSize;
    DWORD           listSizeMax;
    DWORD           target = 0;
    BYTE            tag[128];
    int             tagFlag = 0;
    RiffChunk       riff;
    CommonChunk     common;
    int             lyFlag = 0;
    BYTE*           lyBuf;

    fSync = 0;
    dSize = GetFileSize(FileHandle, NULL);

    //TCY擾
    if ( pListInfo->INAM != NULL )  bufSize = strlen(pListInfo->INAM);
    if ( pListInfo->IART != NULL )  bufSize += strlen(pListInfo->IART);
    if ( pListInfo->IPRD != NULL )  bufSize += strlen(pListInfo->IPRD);
    if ( pListInfo->ICMT != NULL )  bufSize += strlen(pListInfo->ICMT);
    if ( pListInfo->IGNR != NULL )  bufSize += strlen(pListInfo->IGNR);
    if ( pListInfo->ICRD != NULL )  bufSize += strlen(pListInfo->ICRD);
    if ( pListInfo->ICOP != NULL )  bufSize += strlen(pListInfo->ICOP);
    if ( pListInfo->IENG != NULL )  bufSize += strlen(pListInfo->IENG);
    if ( pListInfo->ISRC != NULL )  bufSize += strlen(pListInfo->ISRC);
    if ( pListInfo->ISFT != NULL )  bufSize += strlen(pListInfo->ISFT);
    if ( pListInfo->IKEY != NULL )  bufSize += strlen(pListInfo->IKEY);
    if ( pListInfo->ITCH != NULL )  bufSize += strlen(pListInfo->ITCH);
    if ( pListInfo->ICMS != NULL )  bufSize += strlen(pListInfo->ICMS);
    if ( pListInfo->IMED != NULL )  bufSize += strlen(pListInfo->IMED);
    if ( pListInfo->ISBJ != NULL )  bufSize += strlen(pListInfo->ISBJ);
    if ( pListInfo->IMP3 != NULL )  bufSize += strlen(pListInfo->IMP3);
    if ( pListInfo->ILYC != NULL )  bufSize += strlen(pListInfo->ILYC);
    bufSize += 2048;

    //m
    pbuf = (BYTE*)::malloc(bufSize);
    ::memset(pbuf, 0, bufSize);

    if (pbuf == NULL){
        ::CloseHandle(FileHandle);//ǉ by Kobarin
        _setError(ERR_MALLOC);
        ::free(pbuf);
        return FALSE;
    }

    //RIFF`AID3 Tag v2`FbN
    SetFilePointer(FileHandle, 0, NULL, FILE_BEGIN);
    result = ReadFile(FileHandle, pbuf, 4, &bytesRead, NULL);
    if (memcmp(pbuf, "RIFF", 4) != 0) {
        ::CloseHandle(FileHandle);
        _setError(ERR_NON_RIFF);
        ::free(pbuf);
        return FALSE;
    } else {
        //t@C̓ɖ߂ARIFF`Nǂݍ
        ::SetFilePointer(FileHandle, 0, NULL, FILE_BEGIN);
        result = ::ReadFile(FileHandle, &riff, sizeof(riff), &bytesRead, NULL);
        if (!result || bytesRead != sizeof(riff)) {
            ::CloseHandle(FileHandle);
            _setError(ERR_MP3_FILE_READ);
            ::free(pbuf);
            return FALSE;
        }
        fSync = sizeof(riff);
        //data`NT
        for (;;) {
            result = ::ReadFile(FileHandle, &common, sizeof(common), &bytesRead, NULL);
            if (!result || bytesRead != sizeof(common)) {
                _setError(ERR_MP3_FILE_READ);
                ::CloseHandle(FileHandle);
                ::free(pbuf);
                return FALSE;
            }

            if (::memcmp(common.chunkID, "data", 4) == 0) {
                //data`N
                break;
            }
            else {
                //̃`N܂œǂݔ΂
                fSync = ::SetFilePointer(FileHandle, common.chunkSize, NULL, FILE_CURRENT);
            }
        }

        //`NTCY`FbN
        if (common.chunkSize < dSize) {
            dSize = common.chunkSize;
        }

        fSync += bytesRead;
        fLast = fSync + dSize;
    }

    //ID3Tag v1`FbN
    SetFilePointer(FileHandle, -(long)128, NULL, FILE_END);
    result = ReadFile(FileHandle, tag, 128, &bytesRead, NULL);
    if (!result || bytesRead != 128) {
        _setError(ERR_MP3_FILE_READ);
        ::CloseHandle(FileHandle);
        ::free(pbuf);
        return FALSE;
    }
    if (memcmp(tag, "TAG", 3) == 0) {
        tagFlag = 1;
    }

    //LYRICS3݃`FbN
    DWORD lysize = 0;           //TCY
    DWORD lyfsize;          //tB[hTCY
    DWORD lysp = 0;         //X^[gʒu
    if (chkLYRICS3(FileHandle, &lysize)) {
        lyFlag = 1;
        lyfsize = lysize;

        //LYRICS3 `FbN
        if ((lysize % 2) == 1) {
            lyfsize++;
            lysp = 1;
            lyBuf = (BYTE*)::malloc(lyfsize);
            lyBuf[0] = 0x00;
        } else {
            lyBuf = (BYTE*)::malloc(lysize);
        }

        if (lyBuf == NULL){
            _setError(ERR_MALLOC);
            ::CloseHandle(FileHandle);
            ::free(lyBuf);
            ::free(pbuf);
            return FALSE;
        }

        //|C^ړ
        if ( tagFlag == 1 ){
            SetFilePointer(FileHandle, -(long)(128+lysize), NULL, FILE_END);
        } else {
            SetFilePointer(FileHandle, -(long)(lysize), NULL, FILE_END);
        }

        result = ReadFile(FileHandle, &lyBuf[lysp], lysize, &bytesRead, NULL);
        if (!result || bytesRead != lysize) {
            _setError(ERR_MP3_FILE_READ);
            ::CloseHandle(FileHandle);
            ::free(lyBuf);
            ::free(pbuf);
            return FALSE;
        }
        lysize = lyfsize;
    }

    //f[^TCY΍
    if ( (dSize % 2) == 1 ){
        pbuf[0] = 0x00;
        SetFilePointer(FileHandle, fLast, NULL, FILE_BEGIN);
        result = ::WriteFile(FileHandle, pbuf, 1, &bytesWrite, NULL);
        if (!result || bytesWrite != 1) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(FileHandle);
            if ( lyFlag == 1 ) ::free(lyBuf);
            ::free(pbuf);
            return FALSE;
        }
        fLast++;
    }

    //SItB[hZbg
    target = 12;
    target = makeListData(target, pbuf, pListInfo->INAM, "INAM");
    target = makeListData(target, pbuf, pListInfo->IART, "IART");
    target = makeListData(target, pbuf, pListInfo->IPRD, "IPRD");
    target = makeListData(target, pbuf, pListInfo->ICMT, "ICMT");
    target = makeListData(target, pbuf, pListInfo->IGNR, "IGNR");
    target = makeListData(target, pbuf, pListInfo->ICRD, "ICRD");
    target = makeListData(target, pbuf, pListInfo->ICOP, "ICOP");
    target = makeListData(target, pbuf, pListInfo->IENG, "IENG");
    target = makeListData(target, pbuf, pListInfo->ISRC, "ISRC");
    target = makeListData(target, pbuf, pListInfo->ISFT, "ISFT");
    target = makeListData(target, pbuf, pListInfo->IKEY, "IKEY");
    target = makeListData(target, pbuf, pListInfo->ITCH, "ITCH");
    target = makeListData(target, pbuf, pListInfo->ICMS, "ICMS");
    target = makeListData(target, pbuf, pListInfo->IMED, "IMED");
    target = makeListData(target, pbuf, pListInfo->ISBJ, "ISBJ");
    target = makeListData(target, pbuf, pListInfo->IMP3, "IMP3");

    if ( pListInfo->ILYC == NULL ) {
        listSizeMax = 0;
    } else {
        listSizeMax = ::strlen(pListInfo->ILYC);
    }

    for (listSize = listSizeMax; listSize > 0 &&
        (pListInfo->ILYC[listSize - 1] == ' ' ||
         pListInfo->ILYC[listSize - 1] == '\0'); listSize --) {}
    listSize++;
    ::memcpy(&pbuf[target], "ILYC", 4);

    if ( lyFlag == 0 || m_Lyrics3InsertPoint == 2 ) {
        pbuf[target + 4] = (BYTE)(listSize & 0x000000ff);
        pbuf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
        pbuf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
        pbuf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
        ::memcpy(&pbuf[target + 8], pListInfo->ILYC, listSize - 1);
        target += 8 + listSize;
        if ((listSize % 2) == 1 ) target += 1;

        ::memcpy(&pbuf[target], "IID3", 4);
        pbuf[target + 4] = 0x80;

        target += 8;

        if ( tagFlag == 0 ) {
            int Cnt;
            ::memcpy(&pbuf[target], "TAG", 3);
            for (Cnt = 3; Cnt < 127; Cnt++){
                ::memcpy(&pbuf[target+Cnt], " ", 1);
            }
            pbuf[target+127] = 0xFA;
        } else {
            ::memcpy(&pbuf[target], tag, 128);
        }

        target += 128;
    } else {
        if ( m_Lyrics3InsertPoint == 1 ) {
            pbuf[target + 4] = (BYTE)(listSize & 0x000000ff);
            pbuf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
            pbuf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
            pbuf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
            ::memcpy(&pbuf[target + 8], pListInfo->ILYC, listSize - 1);
            target += 8 + listSize;
            if ((listSize % 2) == 1 ) target += 1;

            ::memcpy(&pbuf[target], "IID3", 4);
        }

        pbuf[target + 4] = (BYTE)((lysize + 128) & 0x000000ff);
        pbuf[target + 5] = (BYTE)(((lysize + 128) & 0x0000ffff) >> 8);
        pbuf[target + 6] = (BYTE)(((lysize + 128) & 0x00ffffff) >> 16);
        pbuf[target + 7] = (BYTE)(((lysize + 128) & 0xffffffff) >> 24);

        target += 8 + lysize + 128;
    }

    ::memcpy(&pbuf[0], "LIST", 4);
    pbuf[4] = (BYTE)((target - 8) & 0x000000ff);
    pbuf[5] = (BYTE)(((target - 8) & 0x0000ffff) >> 8);
    pbuf[6] = (BYTE)(((target - 8) & 0x00ffffff) >> 16);
    pbuf[7] = (BYTE)(((target - 8) & 0xffffffff) >> 24);
    ::memcpy(&pbuf[8], "INFO", 4);

    SetFilePointer(FileHandle, fLast, NULL, FILE_BEGIN);
    result = ::WriteFile(FileHandle, pbuf, target, &bytesWrite, NULL);
    if (!result || bytesWrite != target) {
        _setError(ERR_MP3_FILE_WRITE);
        ::CloseHandle(FileHandle);
        if ( lyFlag == 1 ) ::free(lyBuf);
        ::free(pbuf);
        return FALSE;
    }
    ::SetEndOfFile(FileHandle);

    if ( lyFlag != 0 && m_Lyrics3InsertPoint != 2 ) {
        SetFilePointer(FileHandle, -(long)(lysize + 128), NULL, FILE_END);
        result = ::WriteFile(FileHandle, lyBuf, lysize, &bytesWrite, NULL);
        if (!result || bytesWrite != lysize) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(FileHandle);
            if ( lyFlag == 1 ) ::free(lyBuf);
            ::free(pbuf);
            return FALSE;
        }

        if ( tagFlag == 0 ) {
            int Cnt;
            ::memcpy(&pbuf[0], "TAG", 3);
            for (Cnt = 3; Cnt < 127; Cnt++){
                ::memcpy(&pbuf[target+Cnt], " ", 1);
            }
            pbuf[127] = 0xFA;
        } else {
            ::memcpy(&pbuf[0], tag, 128);
        }

        result = ::WriteFile(FileHandle, pbuf, 128, &bytesWrite, NULL);
        if (!result || bytesWrite != 128) {
            _setError(ERR_MP3_FILE_WRITE);
            ::CloseHandle(FileHandle);
            if ( lyFlag == 1 ) ::free(lyBuf);
            ::free(pbuf);
            return FALSE;
        }

        ::SetEndOfFile(FileHandle);
    }

    fSize = fLast + target;

    ::memset(pbuf, 0, sizeof(pbuf));
    pbuf[0] = (BYTE)((fSize - 8) & 0x000000ff);
    pbuf[1] = (BYTE)(((fSize - 8) & 0x0000ffff) >> 8);
    pbuf[2] = (BYTE)(((fSize - 8) & 0x00ffffff) >> 16);
    pbuf[3] = (BYTE)(((fSize - 8) & 0xffffffff) >> 24);

    SetFilePointer(FileHandle, 4, NULL, FILE_BEGIN);
    result = ::WriteFile(FileHandle, pbuf, 4, &bytesWrite, NULL);
    if (!result || bytesWrite != 4) {
        _setError(ERR_MP3_FILE_WRITE);
        ::CloseHandle(FileHandle);
        if ( lyFlag == 1 ) ::free(lyBuf);
        ::free(pbuf);
        return FALSE;
    }

    ::CloseHandle(FileHandle);
    if ( lyFlag == 1 ) ::free(lyBuf);
    ::free(pbuf);
    return TRUE;

}

DWORD makeListData(DWORD target, BYTE* pbuf, char* data, const char* key)
{
    DWORD   listSizeMax;
    DWORD   listSize;
    int     modValue = 0;       //QŊ؂Ȃꍇ 1

    if ( data == NULL ) {
        listSizeMax = 0;
    } else {
        listSizeMax = ::strlen(data);
    }
    for (listSize = listSizeMax; listSize > 0 && 
        (data[listSize - 1] == ' ' || 
         data[listSize - 1] == '\0'); listSize --) {}
    if ( listSize == 0 && m_NotDataSiWrite == 0 ) return target;
    listSize++;
    if ((listSize % 2) == 1 ) modValue = 1;

    ::memcpy(&pbuf[target], key, 4);
    pbuf[target + 4] = (BYTE)(listSize & 0x000000ff);
    pbuf[target + 5] = (BYTE)((listSize & 0x0000ffff) >> 8);
    pbuf[target + 6] = (BYTE)((listSize & 0x00ffffff) >> 16);
    pbuf[target + 7] = (BYTE)((listSize & 0xffffffff) >> 24);
    ::memcpy(&pbuf[target + 8], data, listSize - 1);
    target += 8 + listSize + modValue;
    return target;
}

void regRead()
{
    long data;

    if (::getRegDW("setReg", &data )) {
        if ( data == 0 ) return;
    }
    else {
        return;
    }

    //VBMP3 Option Set
    VBMP3_OPTION vbmp3Option;
    input_GetVbmp3Option(&vbmp3Option);
    if (::getRegDW("inputBlock", &data )) vbmp3Option.inputBlock = data;
    if (::getRegDW("outputBlock", &data )) vbmp3Option.outputBlock = data;
    if (::getRegDW("inputSleep", &data )) vbmp3Option.inputSleep = data;
    if (::getRegDW("outputSleep", &data )) vbmp3Option.outputSleep = data;
    input_SetVbmp3Option(&vbmp3Option);

    //Decode Option Set
    DEC_OPTION decOption;
    input_GetDecodeOption(&decOption);
    if (::getRegDW("reduction", &data )) decOption.reduction = data;
    if (::getRegDW("convert", &data )) decOption.convert = data;
    if (::getRegDW("freqLimit", &data )) decOption.freqLimit = data;
    input_SetDecodeOption(&decOption);

    //t[擾@
    if (::getRegDW("setFrameReadFlag", &data )) input_setFrameReadFlag(data);
    
    //Lyrics3gpL
    if (::getRegDW("setLyrics3Use", &data )) input_setLyrics3Use(data);

    //JIP^OgpL
    if (::getRegDW("setKaraokeUse", &data )) input_setKaraokeUse(data);
    
    //̎Ԑݒ
    if (::getRegDW("setLyricsTime", &data )) input_setLyricsTime(data);
    
    //^Os擾ݒ
    if (::getRegDW("setNotTagLyricsLine", &data )) input_setNotTagLyricsLine(data);
    
    //̎ǂݕbݒ
    if (::getRegDW("setLyricsNextAdjustTime", &data )) input_setLyricsNextAdjustTime(data);

    //^OgpL
    if (::getRegDW("useAtMarkTag", &data )) input_useAtMarkTag(data);
    
    //sb`w
    if (::getRegDW("setPitch", &data )) input_setPitch(data);

}

BOOL input_startCallBackTimer(TIMER_PROC pProc, UINT wDelay, UINT wResolution, int userData)
{
    if (callBackTimerFlag == TRUE) return FALSE;

    m_pTimerProc = pProc;

    m_timerId = timeSetEvent(wDelay, wResolution, TimeProc, NULL, TIME_PERIODIC);
    if ( m_timerId == 0 ) return FALSE;

    callBackTimerFlag = TRUE;

    m_timeCall = FALSE;
    return TRUE;
}

BOOL input_stopCallBackTimer()
{
    UINT result;
    int  cnt;

    if (callBackTimerFlag == FALSE) return FALSE;
    m_timeCall = FALSE;
    Sleep(50);
    for(cnt = 0; cnt < 100; cnt++){
        result = timeKillEvent(m_timerId);
        if ( result == 0 )  break;
        Sleep(10);
    }

    m_pTimerProc = NULL;
    callBackTimerFlag = FALSE;
    return TRUE;
}
//R[obN֐
void CALLBACK TimeProc(UINT IDEvent, UINT uReserved, 
                       DWORD dwUser, DWORD dwReserved1, DWORD dwReserved2)
{
    if ( m_OnCall == TRUE ) return;
    m_OnCall = TRUE;

    int bitRate;
    int state = input_getState();


    if ( m_pTimerProc == NULL ) return;

    if ( m_timeCall == TRUE ) return;

    if ( state == STATE_STOP ){
        m_pTimerProc(state, 0, 0, m_timerUserData);
    }
    else {
        if ( m_waveFlag == FALSE ) {
            bitRate = m_bitRate / 1000;
        } else {
            bitRate = m_bitRate;
        }

        //^C}R[obN
        m_pTimerProc(state, bitRate, m_playSec, m_timerUserData);
    }
    m_OnCall = FALSE;

}
